@@ -1,6 +1,6 @@
 #!/bin/sh
 # default, host is empty, no cross compilation
-# ./compile.sh [ i586-pc-msdosdjgpp | i386-pc-cygwin | i386-pc-mingw32 | powerpc-mac-darwin ]
+# ./compile.sh [ i586-pc-msdosdjgpp | i686-pc-cygwin | i386-pc-mingw32 | powerpc-mac-darwin ]
 # Comment the version definition to not compile the library
 VER_E2FSPROGS=
 VER_PROGSREISERFS="0.3.1-rc8"
@@ -24,6 +24,10 @@ else
       PKG_CONFIG_PATH=/usr/$crosscompile_target/sys-root/mingw/lib/pkgconfig
     fi
     if [ ! -d $PKG_CONFIG_PATH ];
+    then
+      PKG_CONFIG_PATH=/usr/$crosscompile_target/sys-root/usr/lib/pkgconfig
+    fi
+    if [ ! -d $PKG_CONFIG_PATH ];
     then
       unset PKG_CONFIG_PATH
     fi
@@ -34,30 +38,29 @@ case "$crosscompile_target" in
   "")
   ;;
   *-msdosdjgpp)
-	VER_LIBNTFS3G=
-	VER_NTFSPROGS="2.0.0"
-	VER_E2FSPROGS=1.41.8
+	VER_LIBNTFS3G="2014.2.15"
+	VER_NTFSPROGS=
+	VER_E2FSPROGS="1.42.8"
   ;;
   *-cygwin)
 	VER_LIBNTFS3G=
 	VER_NTFSPROGS="2.0.0"
-	VER_E2FSPROGS=1.42.2
-	export PKG_CONFIG_SYSROOT_DIR=/usr/i386-pc-cygwin/
+	VER_E2FSPROGS="1.42.8"
   ;;
   *-mingw32)
 	VER_LIBNTFS3G=
 	VER_NTFSPROGS="2.0.0"
-	VER_E2FSPROGS=1.42.2
+	VER_E2FSPROGS=
   ;;
   i686-apple-darwin9|powerpc-apple-darwin)
-	VER_LIBNTFS3G="2011.3.28-RC"
+	VER_LIBNTFS3G="2014.2.15"
 	VER_NTFSPROGS=
-	VER_E2FSPROGS=1.41.8
+	VER_E2FSPROGS="1.42.8"
   ;;
   *)
-	VER_LIBNTFS3G="2011.3.28-RC"
+	VER_LIBNTFS3G="2014.2.15"
 	VER_NTFSPROGS=
-	VER_E2FSPROGS=1.42.2
+	VER_E2FSPROGS="1.42.8"
   ;;
 esac
 prefix=/usr/$crosscompile_target
@@ -190,10 +193,10 @@ then
         cd $compiledir/ntfs-3g_ntfsprogs-$VER_LIBNTFS3G
         case "$crosscompile_target" in
           powerpc-apple-darwin|i686-apple-darwin9)
-	  	CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-default-device-io-ops --disable-crypto --disable-nfconv
+		CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-device-default-io-ops --disable-crypto --disable-ntfs-3g --disable-nfconv
 		;;
 	*)
-		CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-default-device-io-ops --disable-crypto
+		CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-device-default-io-ops --disable-crypto --disable-ntfs-3g
 		;;
 	esac
 # --disable-default-device-io-ops is need for NT 4
@@ -311,16 +314,13 @@ then
 # libewf should work under MacOSX but it hasn't been tested
 # use  --with-ncurses-lib=$prefix/usr/lib to get binaries that don't need libncurses
 # but users may be unable to navigate...
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --enable-sudo --with-sudo-bin=/usr/bin/sudo
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --enable-sudo --with-sudo-bin=/usr/bin/sudo  --disable-qt --disable-assert
                 ;;
 	  i686-apple-darwin9)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-sudo --with-sudo-bin=/usr/bin/sudo
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-sudo --with-sudo-bin=/usr/bin/sudo --disable-qt --disable-stack-protector
                 ;;
           i586-pc-msdosdjgpp)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-iconv
-                ;;
-          i386-pc-cygwin)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-qt
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-iconv --disable-qt
                 ;;
           i386-mingw32)
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-iconv --enable-missing-uuid-ok
@@ -329,7 +329,10 @@ then
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --enable-missing-uuid-ok --enable-qt
                 ;;
 	  arm-marvell-linux-gnu)
-		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-ntfs
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ewf --without-ntfs --disable-qt
+                ;;
+	  arm-none-linux-gnueabi|powerpc-linux-gnuspe)
+		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT --without-ntfs --disable-qt
                 ;;
           *)
 		$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT
