@@ -55,6 +55,7 @@
 #include "sysv.h"
 #include "ufs.h"
 #include "vmfs.h"
+#include "wbfs.h"
 #include "xfs.h"
 #include "zfs.h"
 #include "log.h"
@@ -62,11 +63,10 @@
 
 int search_NTFS_backup(unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
-  void *data=disk->pread_fast(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
       return -1;
   {
-    const struct ntfs_boot_sector*ntfs_header=(const struct ntfs_boot_sector*)data;
+    const struct ntfs_boot_sector *ntfs_header=(const struct ntfs_boot_sector*)buffer;
     /* NTFS recovery using backup sector */
     if(le16(ntfs_header->marker)==0xAA55 &&
 	recover_NTFS(disk, ntfs_header, partition, verbose, dump_ind, 1)==0)
@@ -77,12 +77,11 @@ int search_NTFS_backup(unsigned char *buffer, disk_t *disk, partition_t *partiti
 
 int search_HFS_backup(unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
-  void *data=disk->pread_fast(disk, buffer, 0x400, partition->part_offset);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 0x400, partition->part_offset)!= 0x400)
     return -1;
   {
-    const hfs_mdb_t *hfs_mdb=(const hfs_mdb_t *)data;
-    const struct hfsp_vh *vh=(const struct hfsp_vh *)data;
+    const hfs_mdb_t *hfs_mdb=(const hfs_mdb_t *)buffer;
+    const struct hfsp_vh *vh=(const struct hfsp_vh *)buffer;
     /* HFS recovery using backup sector */
     if(hfs_mdb->drSigWord==be16(HFS_SUPER_MAGIC) &&
 	recover_HFS(disk, hfs_mdb, partition, verbose, dump_ind, 1)==0)
@@ -102,11 +101,10 @@ int search_HFS_backup(unsigned char *buffer, disk_t *disk, partition_t *partitio
 
 int search_EXFAT_backup(unsigned char *buffer, disk_t *disk, partition_t *partition)
 {
-  void *data=disk->pread_fast(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
     return -1;
   {
-    const struct exfat_super_block *exfat_header=(const struct exfat_super_block *)data;
+    const struct exfat_super_block *exfat_header=(const struct exfat_super_block *)buffer;
     /* EXFAT recovery using backup sector */
     if(le16(exfat_header->signature)==0xAA55 &&
 	recover_EXFAT(disk, exfat_header, partition)==0)
@@ -120,32 +118,31 @@ int search_EXFAT_backup(unsigned char *buffer, disk_t *disk, partition_t *partit
 
 int search_FAT_backup(unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
-  void *data=disk->pread_fast(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset)!=DEFAULT_SECTOR_SIZE)
     return -1;
   {
-    const struct fat_boot_sector *fat_header=(const struct fat_boot_sector *)data;
+    const struct fat_boot_sector *fat_header=(const struct fat_boot_sector *)buffer;
     /* FAT32 recovery using backup sector */
     if(le16(fat_header->marker)==0xAA55 &&
 	recover_FAT(disk, fat_header, partition, verbose, dump_ind, 1)==0)
-    {
-      strncpy(partition->info,"FAT found using backup sector!",sizeof(partition->info));
       return 1;
-    }
   }
   return 0;
 }
 
 int search_type_0(const unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
+  /* Expect a buffer filled with 8k to handle the SWAP detection */
   const pv_disk_t *pv=(const pv_disk_t *)buffer;
+  const struct cramfs_super *cramfs=(const struct cramfs_super *)buffer;
   const struct disk_fatx *fatx_block=(const struct disk_fatx*)buffer;
   const struct disk_netware *netware_block=(const struct disk_netware *)buffer;
   const struct exfat_super_block *exfat_header=(const struct exfat_super_block *)buffer;
   const struct fat_boot_sector *fat_header=(const struct fat_boot_sector *)buffer;
   const struct luks_phdr *luks=(const struct luks_phdr *)buffer;
   const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)buffer;
-  const struct ntfs_boot_sector*ntfs_header=(const struct ntfs_boot_sector*)buffer;
+  const struct ntfs_boot_sector *ntfs_header=(const struct ntfs_boot_sector*)buffer;
+  const struct wbfs_head *wbfs=(const struct wbfs_head *)buffer;
   const struct xfs_sb *xfs=(const struct xfs_sb *)buffer;
   const union swap_header *swap_header=(const union swap_header *)buffer;
   static const uint8_t LUKS_MAGIC[LUKS_MAGIC_L] = {'L','U','K','S', 0xba, 0xbe};
@@ -161,7 +158,8 @@ int search_type_0(const unsigned char *buffer, disk_t *disk, partition_t *partit
     log_trace("search_type_0 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
-  if(memcmp(swap_header->magic.magic, "SWAP", 4)==0 &&
+  if((memcmp(swap_header->magic.magic, "SWAP", 4)==0 ||
+	memcmp(swap_header->magic8k.magic, "SWAP", 4)==0) &&
       recover_Linux_SWAP(swap_header, partition)==0)
     return 1;
   if(memcmp((const char *)pv->id,LVM_ID,sizeof(pv->id)) == 0 &&
@@ -201,6 +199,12 @@ int search_type_0(const unsigned char *buffer, disk_t *disk, partition_t *partit
     partition->part_offset-=le64(sb1->super_offset)*512;
     return 1;
   }
+  if(memcmp(&wbfs->magic,"WBFS",4)==0 &&
+      recover_WBFS(disk, wbfs, partition, verbose, dump_ind)==0)
+    return 1;
+  if(cramfs->magic==le32(CRAMFS_MAGIC) &&
+      recover_cramfs(disk, cramfs, partition, verbose, dump_ind)==0)
+    return 1;
   /* Try to locate logical partition that may host truecrypt encrypted filesystem */
   if(buffer[0x1fe]==0x55 && buffer[0x1ff]==0xAA &&
     recover_i386_logical(disk, buffer, partition)==0 &&
@@ -212,7 +216,7 @@ int search_type_0(const unsigned char *buffer, disk_t *disk, partition_t *partit
 int search_type_1(const unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
   const struct disklabel *bsd_header=(const struct disklabel *)(buffer+0x200);
-  const struct disk_super_block*beos_block=(const struct disk_super_block*)(buffer+0x200);
+  const struct disk_super_block *beos_block=(const struct disk_super_block*)(buffer+0x200);
   const struct cramfs_super *cramfs=(const struct cramfs_super *)(buffer+0x200);
   const struct lvm2_label_header *lvm2=(const struct lvm2_label_header *)(buffer+0x200);
   const struct sysv4_super_block *sysv4=(const struct sysv4_super_block *)(buffer+0x200);
@@ -275,19 +279,17 @@ int search_type_2(const unsigned char *buffer, disk_t *disk, partition_t *partit
 
 int search_type_8(unsigned char *buffer, disk_t *disk,partition_t *partition,const int verbose, const int dump_ind)
 {
-  void *data;
   if(verbose>2)
   {
     log_trace("search_type_8 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
-  data=disk->pread_fast(disk, buffer, 4096, partition->part_offset + 4096);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 4096, partition->part_offset + 4096) != 4096)
     return -1;
   { /* MD 1.2 */
-    const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)data;
+    const struct mdp_superblock_1 *sb1=(const struct mdp_superblock_1 *)buffer;
     if(le32(sb1->major_version)==1 &&
-        recover_MD(disk, (const struct mdp_superblock_s*)data, partition, verbose, dump_ind)==0)
+        recover_MD(disk, (const struct mdp_superblock_s*)buffer, partition, verbose, dump_ind)==0)
     {
       partition->part_offset-=(uint64_t)le64(sb1->super_offset)*512-4096;
       return 1;
@@ -298,19 +300,17 @@ int search_type_8(unsigned char *buffer, disk_t *disk,partition_t *partition,con
 
 int search_type_16(unsigned char *buffer, disk_t *disk,partition_t *partition,const int verbose, const int dump_ind)
 {
-  void *data;
   if(verbose>2)
   {
     log_trace("search_type_16 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
   /* 8k offset */
-  data=disk->pread_fast(disk, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 16 * 512);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 16 * 512) != 3 * DEFAULT_SECTOR_SIZE)
     return -1;
   {
-    const struct ufs_super_block *ufs=(const struct ufs_super_block *)data;
-    const struct vdev_boot_header *zfs=(const struct vdev_boot_header*)data;
+    const struct ufs_super_block *ufs=(const struct ufs_super_block *)buffer;
+    const struct vdev_boot_header *zfs=(const struct vdev_boot_header*)buffer;
     /* Test UFS */
     if((le32(ufs->fs_magic)==UFS_MAGIC || be32(ufs->fs_magic)==UFS_MAGIC ||
 	  le32(ufs->fs_magic)==UFS2_MAGIC || be32(ufs->fs_magic)==UFS2_MAGIC) &&
@@ -325,19 +325,16 @@ int search_type_16(unsigned char *buffer, disk_t *disk,partition_t *partition,co
 
 int search_type_64(unsigned char *buffer, disk_t *disk,partition_t *partition,const int verbose, const int dump_ind)
 {
-  void *data;
   if(verbose>2)
   {
     log_trace("search_type_64 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
   /* 32k offset */
-  data=disk->pread_fast(disk, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 63 * 512);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 3 * DEFAULT_SECTOR_SIZE, partition->part_offset + 63 * 512) != 3 * DEFAULT_SECTOR_SIZE)
     return -1;
-  data=(char*)data+0x200;
   {
-    const struct jfs_superblock* jfs=(const struct jfs_superblock*)data;
+    const struct jfs_superblock* jfs=(const struct jfs_superblock*)(buffer+0x200);
     /* Test JFS */
     if(memcmp(jfs->s_magic,"JFS1",4)==0 &&
 	recover_JFS(disk, jfs, partition, verbose, dump_ind)==0)
@@ -348,22 +345,20 @@ int search_type_64(unsigned char *buffer, disk_t *disk,partition_t *partition,co
 
 int search_type_128(unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
-  void *data;
   if(verbose>2)
   {
     log_trace("search_type_128 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
-  data=disk->pread_fast(disk, buffer, 11 * DEFAULT_SECTOR_SIZE, partition->part_offset + 126 * 512);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 11 * DEFAULT_SECTOR_SIZE, partition->part_offset + 126 * 512) != 11 * DEFAULT_SECTOR_SIZE)
     return -1;
-  data=(char*)data+0x400;
+  buffer=buffer+0x400;
   {
-    const struct reiserfs_super_block *rfs=(const struct reiserfs_super_block *)data;
-    const struct reiser4_master_sb *rfs4=(const struct reiser4_master_sb *)data;
-    const struct ufs_super_block *ufs=(const struct ufs_super_block *)data;
-    const struct btrfs_super_block *btrfs=(const struct btrfs_super_block*)data;
-    const struct gfs2_sb *gfs2=(const struct gfs2_sb *)data;
+    const struct reiserfs_super_block *rfs=(const struct reiserfs_super_block *)buffer;
+    const struct reiser4_master_sb *rfs4=(const struct reiser4_master_sb *)buffer;
+    const struct ufs_super_block *ufs=(const struct ufs_super_block *)buffer;
+    const struct btrfs_super_block *btrfs=(const struct btrfs_super_block*)buffer;
+    const struct gfs2_sb *gfs2=(const struct gfs2_sb *)buffer;
     /* 64k offset */
     /* Test ReiserFS */
     if((memcmp(rfs->s_magic,"ReIs",4) == 0 ||
@@ -387,17 +382,15 @@ int search_type_128(unsigned char *buffer, disk_t *disk, partition_t *partition,
 
 int search_type_2048(unsigned char *buffer, disk_t *disk, partition_t *partition, const int verbose, const int dump_ind)
 {
-  void *data;
   if(verbose>2)
   {
     log_trace("search_type_2048 lba=%lu\n",
 	(long unsigned)(partition->part_offset/disk->sector_size));
   }
-  data=disk->pread_fast(disk, buffer, 2*DEFAULT_SECTOR_SIZE, partition->part_offset + 2048 * 512);
-  if(data==NULL)
+  if(disk->pread(disk, buffer, 2*DEFAULT_SECTOR_SIZE, partition->part_offset + 2048 * 512) != 2*DEFAULT_SECTOR_SIZE)
     return -1;
   {
-    const struct vmfs_volume *sb_vmfs=(const struct vmfs_volume *)data;
+    const struct vmfs_volume *sb_vmfs=(const struct vmfs_volume *)buffer;
     if(le32(sb_vmfs->magic)==0xc001d00d &&
       recover_VMFS(disk, sb_vmfs, partition, verbose, dump_ind)==0)
       return 1;
@@ -413,8 +406,9 @@ int check_linux(disk_t *disk, partition_t *partition, const int verbose)
       check_cramfs(disk, partition, verbose)==0 ||
       check_xfs(disk, partition, verbose)==0 ||
       check_LUKS(disk, partition)==0 ||
-      check_btrfs(disk, partition, verbose)==0 ||
-      check_gfs2(disk, partition)==0)
+      check_btrfs(disk, partition)==0 ||
+      check_gfs2(disk, partition)==0 ||
+      check_ZFS(disk, partition)==0)
     return 0;
   return 1;
 }
