@@ -44,6 +44,7 @@
 #ifdef HAVE_TIME_H
 #include <time.h>
 #endif
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "log.h"
@@ -52,15 +53,11 @@ static int32_t secwest=0;
 
 static unsigned int up2power_aux(const unsigned int number);
 
+/* coverity[+alloc] */
 void *MALLOC(size_t size)
 {
   void *res;
-  if(size<=0)
-  {
-    log_critical("Try to allocate 0 byte of memory\n");
-    log_close();
-    exit(EXIT_FAILURE);
-  }
+  assert(size>0);
   /* Warning, memory leak checker must be posix_memalign/memalign aware, otherwise  *
    * reports may look strange. Aligned memory is required if the buffer is *
    * used for read/write operation with a file opened with O_DIRECT        */
@@ -185,27 +182,27 @@ static unsigned int up2power_aux(const unsigned int number)
 	return(1+up2power_aux(number/2));
 }
 
-void set_part_name(partition_t *partition,const char *src,const int max_size)
+void set_part_name(partition_t *partition, const char *src, const unsigned int max_size)
 {
-  int i;
+  unsigned int i;
   for(i=0; i<sizeof(partition->fsname)-1 && i<max_size && src[i]!='\0'; i++)
     partition->fsname[i]=src[i];
   partition->fsname[i]='\0';
 }
 
-void set_part_name_chomp(partition_t *partition, const unsigned char *src, const int max_size)
+void set_part_name_chomp(partition_t *partition, const unsigned char *src, const unsigned int max_size)
 {
-  int i;
-  for(i=0; i<sizeof(partition->fsname) && i<max_size && src[i]!='\0'; i++)
+  unsigned int i;
+  for(i=0; i<sizeof(partition->fsname)-1 && i<max_size && src[i]!='\0'; i++)
     partition->fsname[i]=src[i];
+  while(i>0 && src[i-1]==' ')
+    i--;
   partition->fsname[i]='\0';
-  for(i--; i>=0 && src[i]==' '; i--);
-  partition->fsname[i+1]='\0';
 }
 
 char* strip_dup(char* str)
 {
-  int     i;
+  unsigned int i;
   char *end;
   while(isspace(*str))
     str++;
@@ -244,7 +241,10 @@ void set_secwest(void)
   const time_t t = time(NULL);
   const struct  tm *tmptr = localtime(&t);
 #ifdef HAVE_STRUCT_TM_TM_GMTOFF
-  secwest = -1 * tmptr->tm_gmtoff;
+  if(tmptr)
+    secwest = -1 * tmptr->tm_gmtoff;
+  else
+    secwest = 0;
 #elif defined (DJGPP) || defined(__ANDROID__)
   secwest = 0;
 #else
@@ -254,7 +254,7 @@ void set_secwest(void)
   secwest = timezone;
 #endif
   /* account for daylight savings */
-  if (tmptr->tm_isdst)
+  if (tmptr && tmptr->tm_isdst)
     secwest -= 3600;
 #endif
 }
