@@ -34,7 +34,7 @@ struct efi_guid_s
   uint8_t  clock_seq_hi_and_reserved;
   uint8_t  clock_seq_low;
   uint8_t  node[6];
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 #define DEFAULT_SECTOR_SIZE     0x200u
 
@@ -331,7 +331,7 @@ struct arch_fnct_struct
   const char *part_name_option;
   const char *msg_part_type;
   list_part_t *(*read_part)(disk_t *disk, const int verbose,const int saveheader);
-  int (*write_part)(disk_t *disk, const list_part_t *list_part, const int ro, const int verbose, const int align);
+  int (*write_part)(disk_t *disk, const list_part_t *list_part, const int ro, const int verbose);
   list_part_t *(*init_part_order)(const disk_t *disk, list_part_t *list_part);
   /* geometry must be initialized to 0,0,0 in get_geometry_from_mbr()*/
   int (*get_geometry_from_mbr)(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
@@ -387,9 +387,9 @@ struct param_disk_struct
 
 struct partition_struct
 {
-  char          fsname[80];
-  char          partname[80];
-  char          info[80];
+  char          fsname[128];
+  char          partname[128];
+  char          info[128];
   uint64_t      part_offset;
   uint64_t      part_size;
   uint64_t      sborg_offset;
@@ -426,8 +426,8 @@ struct my_data_struct
 
 void *MALLOC(size_t size);
 unsigned int up2power(const unsigned int number);
-void set_part_name(partition_t *partition,const char *src,const int max_size);
-void set_part_name_chomp(partition_t *partition, const unsigned char *src, const int max_size);
+void set_part_name(partition_t *partition, const char *src, const unsigned int max_size);
+void set_part_name_chomp(partition_t *partition, const unsigned char *src, const unsigned int max_size);
 char* strip_dup(char* str);
 int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
 void set_secwest(void);
