@@ -149,49 +149,47 @@ void mode_string (const unsigned int mode, char *str)
 #endif
 }
 
-int dir_aff_log(const dir_data_t *dir_data, const file_data_t*dir_list)
+int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
 {
   int test_date=0;
-  const file_data_t *current_file;
+  struct td_list_head *file_walker = NULL;
   if(dir_data!=NULL)
   {
     log_info("Directory %s\n",dir_data->current_directory);
   }
-  for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
+  td_list_for_each(file_walker, &dir_list->list)
   {
+    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
     char		datestr[80];
     char str[11];
-    if(current_file->stat.st_mtime)
     {
-      struct tm		*tm_p;
-      tm_p = localtime(&current_file->stat.st_mtime);
-      snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	  tm_p->tm_mday, monstr[tm_p->tm_mon],
-	  1900 + tm_p->tm_year, tm_p->tm_hour,
-	  tm_p->tm_min);
-      /* FIXME: a check using current_file->name will be better */
-      if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
+      const struct tm *tm_p;
+      if(current_file->td_mtime && (tm_p = localtime(&current_file->td_mtime))!=NULL)
       {
-	test_date=1;
+	snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
+	    tm_p->tm_mday, monstr[tm_p->tm_mon],
+	    1900 + tm_p->tm_year, tm_p->tm_hour,
+	    tm_p->tm_min);
+	/* FIXME: a check using current_file->name will be better */
+	if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
+	{
+	  test_date=1;
+	}
+      } else {
+	strncpy(datestr, "                 ",sizeof(datestr));
       }
-    } else {
-      strncpy(datestr, "                 ",sizeof(datestr));
     }
-    mode_string(current_file->stat.st_mode,str);
+    mode_string(current_file->st_mode, str);
     if((current_file->status&FILE_STATUS_DELETED)!=0)
       log_info("X");
     else
       log_info(" ");
     log_info("%7lu %s %5u  %5u %9llu %s ",
-	(unsigned long int)current_file->stat.st_ino,
+	(unsigned long int)current_file->st_ino,
 	str,
-	(unsigned int)current_file->stat.st_uid,
-	(unsigned int)current_file->stat.st_gid,
-#ifdef DJGPP
-	(long long unsigned int)current_file->file_size,
-#else
-	(long long unsigned int)current_file->stat.st_size,
-#endif
+	(unsigned int)current_file->st_uid,
+	(unsigned int)current_file->st_gid,
+	(long long unsigned int)current_file->st_size,
 	datestr);
     if(dir_data!=NULL && (dir_data->param&FLAG_LIST_PATHNAME)!=0)
     {
@@ -216,94 +214,82 @@ int log_list_file(const disk_t *disk, const partition_t *partition, const dir_da
   }
   td_list_for_each(tmp, &list->list)
   {
-    file_info_t *current_file;
     char		datestr[80];
     char str[11];
-    current_file=td_list_entry(tmp, file_info_t, list);
+    const file_info_t *current_file=td_list_entry(tmp, file_info_t, list);
     if((current_file->status&FILE_STATUS_DELETED)!=0)
       log_info("X");
     else
       log_info(" ");
-    if(current_file->stat.st_mtime)
     {
-      struct tm		*tm_p;
-      tm_p = localtime(&current_file->stat.st_mtime);
-
-      snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	  tm_p->tm_mday, monstr[tm_p->tm_mon],
-	  1900 + tm_p->tm_year, tm_p->tm_hour,
-	  tm_p->tm_min);
-      /* FIXME: a check using current_file->name will be better */
-      if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2012)
+      const struct tm *tm_p;
+      if(current_file->td_mtime && (tm_p = localtime(&current_file->td_mtime))!=NULL)
       {
-	test_date=1;
+	snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
+	    tm_p->tm_mday, monstr[tm_p->tm_mon],
+	    1900 + tm_p->tm_year, tm_p->tm_hour,
+	    tm_p->tm_min);
+	/* FIXME: a check using current_file->name will be better */
+	if(1900+tm_p->tm_year>=2000 && 1900+tm_p->tm_year<=2014)
+	{
+	  test_date=1;
+	}
+      } else {
+	strncpy(datestr, "                 ",sizeof(datestr));
       }
-    } else {
-      strncpy(datestr, "                 ",sizeof(datestr));
     }
-    mode_string(current_file->stat.st_mode,str);
-    log_info("%7lu ",(unsigned long int)current_file->stat.st_ino);
+    mode_string(current_file->st_mode, str);
+    log_info("%7lu ",(unsigned long int)current_file->st_ino);
     log_info("%s %5u %5u ", 
-	str, (unsigned int)current_file->stat.st_uid, (unsigned int)current_file->stat.st_gid);
-#ifdef DJGPP
-    log_info("%9llu", (long long unsigned int)current_file->file_size);
-#else
-    log_info("%9llu", (long long unsigned int)current_file->stat.st_size);
-#endif
+	str, (unsigned int)current_file->st_uid, (unsigned int)current_file->st_gid);
+    log_info("%9llu", (long long unsigned int)current_file->st_size);
     log_info(" %s %s\n", datestr, current_file->name);
   }
   return test_date;
 }
 
-unsigned int delete_list_file(file_data_t *file_list)
+unsigned int delete_list_file(file_info_t *file_info)
 {
-  int nbr=0;
-  file_data_t *current_file=file_list;
-  while(current_file!=NULL)
+  unsigned int nbr=0;
+  struct td_list_head *file_walker = NULL;
+  struct td_list_head *file_walker_next = NULL;
+  td_list_for_each_safe(file_walker,file_walker_next, &file_info->list)
   {
-    file_data_t *next=current_file->next;
-    free(current_file);
-    current_file=next;
+    file_info_t *tmp;
+    tmp=td_list_entry(file_walker, file_info_t, list);
+    free(tmp->name);
+    td_list_del(file_walker);
+    free(tmp);
     nbr++;
   }
   return nbr;
 }
 
-void delete_list_file_info(struct td_list_head *list)
-{
-  struct td_list_head *file_walker = NULL;
-  struct td_list_head *file_walker_next = NULL;
-  td_list_for_each_safe(file_walker,file_walker_next, list)
-  {
-    file_info_t *file_info;
-    file_info=td_list_entry(file_walker, file_info_t, list);
-    free(file_info->name);
-    td_list_del(file_walker);
-    free(file_info);
-  }
-}
-
 static int dir_whole_partition_log_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode)
 {
-  file_data_t *dir_list;
-  file_data_t *current_file;
+  struct td_list_head *file_walker = NULL;
 #define MAX_DIR_NBR 256
   static unsigned int dir_nbr=0;
   static unsigned long int inode_known[MAX_DIR_NBR];
   const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
+  file_info_t dir_list = {
+    .list = TD_LIST_HEAD_INIT(dir_list.list),
+    .name = NULL
+  };
   if(dir_nbr==MAX_DIR_NBR)
     return 1;	/* subdirectories depth is too high => Back */
   if(dir_data->verbose>0)
     log_info("\ndir_partition inode=%lu\n",inode);
-  dir_list=dir_data->get_dir(disk, partition, dir_data, inode);
-  dir_aff_log(dir_data, dir_list);
+  dir_data->get_dir(disk, partition, dir_data, inode, &dir_list);
+  dir_aff_log(dir_data, &dir_list);
   /* Not perfect for FAT32 root cluster */
   inode_known[dir_nbr++]=inode;
-  for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
+  td_list_for_each(file_walker, &dir_list.list)
   {
-    if(LINUX_S_ISDIR(current_file->stat.st_mode)!=0)
+    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+    if(LINUX_S_ISDIR(current_file->st_mode)!=0)
     {
-      const unsigned long int new_inode=current_file->stat.st_ino;
+      const unsigned long int new_inode=current_file->st_ino;
       unsigned int new_inode_ok=1;
       unsigned int i;
       if(new_inode<2)
@@ -327,7 +313,7 @@ static int dir_whole_partition_log_aux(disk_t *disk, const partition_t *partitio
       }
     }
   }
-  delete_list_file(dir_list);
+  delete_list_file(&dir_list);
   dir_nbr--;
   return 0;
 }
@@ -340,18 +326,23 @@ int dir_whole_partition_log(disk_t *disk, const partition_t *partition, dir_data
 
 int filesort(const struct td_list_head *a, const struct td_list_head *b)
 {
-  const file_info_t *file_a=td_list_entry(a, file_info_t, list);
-  const file_info_t *file_b=td_list_entry(b, file_info_t, list);
+  const file_info_t *file_a=td_list_entry_const(a, const file_info_t, list);
+  const file_info_t *file_b=td_list_entry_const(b, const file_info_t, list);
   /* Directories must be listed before files */
-  const int res=((file_b->stat.st_mode&LINUX_S_IFDIR)-(file_a->stat.st_mode&LINUX_S_IFDIR));
+  const int res=((file_b->st_mode&LINUX_S_IFDIR)-(file_a->st_mode&LINUX_S_IFDIR));
   if(res)
     return res;
   /* . and .. must listed before the other directories */
-  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, ".")==0)
+  if((file_a->st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, ".")==0)
     return -1;
-  if((file_a->stat.st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, "..")==0 &&
-      !strcmp(file_b->name, ".")==0)
+  if((file_a->st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, "..")==0 &&
+      strcmp(file_b->name, ".")!=0)
     return -1;
+  if((file_b->st_mode&LINUX_S_IFDIR) && strcmp(file_b->name, ".")==0)
+    return 1;
+  if((file_b->st_mode&LINUX_S_IFDIR) && strcmp(file_b->name, "..")==0 &&
+      strcmp(file_a->name, ".")!=0)
+    return 1;
   /* Files and directories are sorted by name */
   return strcmp(file_a->name, file_b->name);
 }
@@ -413,8 +404,8 @@ static mode_t mode_xlate(unsigned int lmode)
  *
  * Give a file a particular mode.
  *
- * Return:  1  Success, set the file's mode
- *	    0  Error, failed to change the file's mode
+ * Return:  0  Success, set the file's mode
+ *	    -1  Error, failed to change the file's mode
  */
 int set_mode(const char *pathname, unsigned int mode)
 {
@@ -587,7 +578,7 @@ static unsigned int filename_convert(char *dst, const char*src, const unsigned i
 
 char *gen_local_filename(const char *filename)
 {
-  int l=strlen(filename);
+  const int l=strlen(filename);
   char *dst=(char *)MALLOC(l+1);
   filename_convert(dst, filename, l);
 #if defined(DJGPP) || defined(__CYGWIN__) || defined(__MINGW32__)
@@ -654,7 +645,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
       unsigned int l;
       l=filename_convert(dst_org, src_org, n);
       dst=dst_org+l;
-      mkdir(localdir, 0775);
+      (void)mkdir(localdir, 0775);
     }
 #endif
   }
@@ -733,7 +724,7 @@ FILE *fopen_local(char **localfilename, const char *localroot, const char *filen
 	unsigned int l;
 	l=filename_convert(dst_org, src_org, n);
 	dst=dst_org+l;
-	mkdir(*localfilename, 0775);
+	(void)mkdir(*localfilename, 0775);
       }
 #endif
     }
