@@ -101,6 +101,7 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
   int quit=0;
   int ask_destination=1;
   WINDOW *window=(WINDOW*)dir_data->display;
+  const char *needle=NULL;
   do
   {
     int offset=0;
@@ -135,22 +136,29 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	  wattrset(window, A_REVERSE);
 	  waddstr(window, ">");
 	}
+	else if((current_file->status&FILE_STATUS_MARKED)!=0)
+	  waddstr(window, "*");
 	else
 	  waddstr(window, " ");
-	if((current_file->status&FILE_STATUS_DELETED)!=0 && has_colors())
-	  wbkgdset(window,' ' | COLOR_PAIR(1));
-	else if((current_file->status&FILE_STATUS_MARKED)!=0 && has_colors())
-	  wbkgdset(window,' ' | COLOR_PAIR(2));
-	if(current_file->td_mtime!=0)
+	if(has_colors())
 	{
-	  const struct tm *tm_p = localtime(&current_file->td_mtime);
-	  snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	      tm_p->tm_mday, monstr[tm_p->tm_mon],
-	      1900 + tm_p->tm_year, tm_p->tm_hour,
-	      tm_p->tm_min);
-	  /* May have to use %d instead of %e */
-	} else {
-	  strncpy(datestr, "                 ",sizeof(datestr));
+	  if((current_file->status&FILE_STATUS_MARKED)!=0)
+	    wbkgdset(window,' ' | COLOR_PAIR(2));
+	  else if((current_file->status&FILE_STATUS_DELETED)!=0)
+	    wbkgdset(window,' ' | COLOR_PAIR(1));
+	}
+	{
+	  const struct tm *tm_p;
+	  if(current_file->td_mtime!=0 && (tm_p = localtime(&current_file->td_mtime))!=NULL)
+	  {
+	    snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
+		tm_p->tm_mday, monstr[tm_p->tm_mon],
+		1900 + tm_p->tm_year, tm_p->tm_hour,
+		tm_p->tm_min);
+	    /* May have to use %d instead of %e */
+	  } else {
+	    strncpy(datestr, "                 ",sizeof(datestr));
+	  }
 	}
 	mode_string(current_file->st_mode, str);
 	wprintw(window, "%s %5u %5u ", 
@@ -158,8 +166,8 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	wprintw(window, "%9llu", (long long unsigned int)current_file->st_size);
 	/* screen may overlap due to long filename */
 	wprintw(window, " %s %s", datestr, current_file->name);
-	if(((current_file->status&FILE_STATUS_DELETED)!=0 ||
-	      (current_file->status&FILE_STATUS_MARKED)!=0) && has_colors())
+	if((current_file->status&(FILE_STATUS_DELETED|FILE_STATUS_MARKED))!=0 &&
+	    has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	if(&current_file->list==pos)
 	  wattroff(window, A_REVERSE);
@@ -485,29 +493,92 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 		copy_done(window, copy_ok, copy_bad);
 	      }
 	    }
-	    break;	
+	    break;
+	  case '/':
 	  case 'f':
+	    needle=ask_string_ncurses("Filename to find ? ");
+	    if(needle!=NULL && needle[0]!='\0' && pos->next!=&dir_list->list)
 	    {
-	      const char *needle=ask_string_ncurses("Filename to find ? ");
+	      const file_info_t *tmp;
+	      struct td_list_head *pos_org=pos;
+	      const int pos_num_org=pos_num;
+	      tmp=td_list_entry(pos, file_info_t, list);
+	      while(pos->next!=&dir_list->list &&
+		  strcasestr(tmp->name, needle)==NULL)
+	      {
+		pos=pos->next;
+		tmp=td_list_entry(pos, file_info_t, list);
+		pos_num++;
+	      }
+	      if(strcasestr(tmp->name, needle)==NULL)
+	      {
+		pos=pos_org;
+		pos_num=pos_num_org;
+	      }
+	    }
+	    break;
+	  case 'n':
+	    if(needle!=NULL && needle[0]!='\0' && pos->next!=&dir_list->list)
+	    {
+	      const file_info_t *tmp;
+	      struct td_list_head *pos_org=pos;
+	      const int pos_num_org=pos_num;
+	      pos=pos->next;
+	      tmp=td_list_entry(pos, file_info_t, list);
+	      pos_num++;
+	      while(pos->next!=&dir_list->list &&
+		  strcasestr(tmp->name, needle)==0)
+	      {
+		pos=pos->next;
+		tmp=td_list_entry(pos, file_info_t, list);
+		pos_num++;
+	      }
+	      if(strcasestr(tmp->name, needle)==0)
+	      {
+		pos=pos_org;
+		pos_num=pos_num_org;
+	      }
+	    }
+	    break;
+	  case 'N':
+	    if(needle!=NULL && needle[0]!='\0' && pos->prev!=&dir_list->list)
+	    {
+	      const file_info_t *tmp;
+	      struct td_list_head *pos_org=pos;
+	      const int pos_num_org=pos_num;
+	      pos=pos->prev;
+	      tmp=td_list_entry(pos, file_info_t, list);
+	      pos_num--;
+	      while(pos->prev!=&dir_list->list &&
+		  strcasestr(tmp->name, needle)==0)
+	      {
+		pos=pos->prev;
+		tmp=td_list_entry(pos, file_info_t, list);
+		pos_num--;
+	      }
+	      if(strcasestr(tmp->name, needle)==0)
+	      {
+		pos=pos_org;
+		pos_num=pos_num_org;
+	      }
+	    }
+	    break;
+	  case 'F':
+	    {
+	      needle=ask_string_ncurses("Filter ? ");
 	      if(needle!=NULL && needle[0]!='\0')
 	      {
-		file_info_t *tmp;
-		struct td_list_head *pos_org=pos;
-		const int pos_num_org=pos_num;
-		tmp=td_list_entry(pos, file_info_t, list);
-		while(pos->next!=&dir_list->list &&
-		  strcmp(tmp->name, needle)!=0)
+		struct td_list_head *tmpw= NULL;
+		td_list_for_each(tmpw, &dir_list->list)
 		{
-		  pos=pos->next;
-		  tmp=td_list_entry(pos, file_info_t, list);
-		  pos_num++;
-		}
-		if(pos==&dir_list->list)
-		{
-		  pos=pos_org;
-		  pos_num=pos_num_org;
+		  file_info_t *tmp=td_list_entry(tmpw, file_info_t, list);
+		  if(strcasestr(tmp->name, needle) != NULL)
+		  {
+		    tmp->status^=FILE_STATUS_MARKED;
+		  }
 		}
 	      }
+	      status^=FILE_STATUS_MARKED;
 	    }
 	    break;
 	}
