@@ -23,7 +23,7 @@
 extern "C" {
 #endif
 
-int dir_partition_exfat_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose);
+dir_partition_t dir_partition_exfat_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
