@@ -43,6 +43,8 @@
 #include "intrf.h"
 #include "log.h"
 #include "log_part.h"
+#include "fat_common.h"
+
 extern const arch_fnct_t arch_i386;
 extern const arch_fnct_t arch_mac;
 
@@ -81,7 +83,7 @@ static int log_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upa
   log_info("reserved     %u\n", le16(fh1->reserved));
   log_info("fats         %u\n", fh1->fats);
   log_info("dir_entries  %u\n", get_dir_entries(fh1));
-  log_info("sectors      %u\n", sectors(fh1));
+  log_info("sectors      %u\n", fat_sectors(fh1));
   log_info("media        %02X\n", fh1->media);
   log_info("fat_length   %u\n", le16(fh1->fat_length));
   log_info("secs_track   %u\n", le16(fh1->secs_track));
@@ -129,7 +131,7 @@ int log_fat2_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector
   log_info("reserved     %u %u\n", le16(fh1->reserved),le16(fh2->reserved));
   log_info("fats         %u %u\n", fh1->fats,fh2->fats);
   log_info("dir_entries  %u %u\n", get_dir_entries(fh1),get_dir_entries(fh2));
-  log_info("sectors      %u %u\n", sectors(fh1),sectors(fh2));
+  log_info("sectors      %u %u\n", fat_sectors(fh1),fat_sectors(fh2));
   log_info("media        %02X %02X\n", fh1->media,fh2->media);
   log_info("fat_length   %u %u\n", le16(fh1->fat_length),le16(fh2->fat_length));
   log_info("secs_track   %u %u\n", le16(fh1->secs_track),le16(fh2->secs_track));
@@ -198,10 +200,11 @@ static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_head
 {
   const char*buffer=(const char*)fat_header;
   partition->fsname[0]='\0';
+  partition->blocksize=fat_sector_size(fat_header)* fat_header->sectors_per_cluster;
   switch(partition->upart_type)
   {
     case UP_FAT12:
-      snprintf(partition->info,sizeof(partition->info),"FAT12");
+      snprintf(partition->info, sizeof(partition->info), "FAT12, blocksize=%u", partition->blocksize);
       if(buffer[38]==0x29)	/* BS_BootSig */
       {
         set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
@@ -210,7 +213,7 @@ static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_head
       }
       break;
     case UP_FAT16:
-      snprintf(partition->info,sizeof(partition->info),"FAT16");
+      snprintf(partition->info, sizeof(partition->info), "FAT16, blocksize=%u", partition->blocksize);
       if(buffer[38]==0x29)	/* BS_BootSig */
       {
         set_part_name_chomp(partition,((const unsigned char*)fat_header)+FAT1X_PART_NAME,11);
@@ -219,7 +222,10 @@ static int set_FAT_info(disk_t *disk_car, const struct fat_boot_sector *fat_head
       }
       break;
     case UP_FAT32:
-      snprintf(partition->info,sizeof(partition->info),"FAT32");
+      if(partition->sb_offset==0)
+	snprintf(partition->info, sizeof(partition->info), "FAT32, blocksize=%u", partition->blocksize);
+      else
+	snprintf(partition->info, sizeof(partition->info), "FAT32 found using backup sector, blocksize=%u", partition->blocksize);
       fat32_set_part_name(disk_car,partition,fat_header);
       break;
     default:
@@ -233,15 +239,13 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
 {
   /* Offset can be offset to FAT1 or to FAT2 */
   /* log_trace("get_next_cluster(upart_type=%u,offset=%u,cluster=%u\n",upart_type,offset,cluster); */
-  unsigned char *buffer;
-  const unsigned int buffer_size=(upart_type==UP_FAT12?2*disk_car->sector_size:disk_car->sector_size);
-  buffer=(unsigned char*)MALLOC(buffer_size);
   switch(upart_type)
   {
     case UP_FAT12:
       {
 	unsigned int next_cluster;
 	unsigned long int offset_s,offset_o;
+	unsigned char *buffer=(unsigned char*)MALLOC(2*disk_car->sector_size);
         offset_s=(cluster+cluster/2)/disk_car->sector_size;
         offset_o=(cluster+cluster/2)%disk_car->sector_size;
         if((unsigned)disk_car->pread(disk_car, buffer, 2 * disk_car->sector_size,
@@ -262,6 +266,7 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
       {
 	unsigned int next_cluster;
 	unsigned long int offset_s,offset_o;
+	unsigned char *buffer=(unsigned char*)MALLOC(disk_car->sector_size);
         const uint16_t *p16=(const uint16_t*)buffer;
         offset_s=cluster/(disk_car->sector_size/2);
         offset_o=cluster%(disk_car->sector_size/2);
@@ -280,6 +285,7 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
       {
 	unsigned int next_cluster;
 	unsigned long int offset_s,offset_o;
+	unsigned char *buffer=(unsigned char*)MALLOC(disk_car->sector_size);
         const uint32_t *p32=(const uint32_t*)buffer;
         offset_s=cluster/(disk_car->sector_size/4);
         offset_o=cluster%(disk_car->sector_size/4);
@@ -301,7 +307,6 @@ unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, con
       }
     default:
       log_critical("fat.c get_next_cluster unknown fat type\n");
-      free(buffer);
       return 0;
   }
 }
@@ -388,8 +393,7 @@ unsigned int fat32_get_prev_cluster(disk_t *disk_car,const partition_t *partitio
 
   for(prev_cluster=2;prev_cluster<=no_of_cluster+1;prev_cluster++)
   {
-    unsigned int offset_o;
-    offset_o=prev_cluster%(disk_car->sector_size/4);
+    const unsigned int offset_o=prev_cluster%(disk_car->sector_size/4);
     if((offset_o==0)||(prev_cluster==2))
     {
       if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, hd_offset) != disk_car->sector_size)
@@ -472,27 +476,16 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
       log_error("check_FAT: Bad number %u of FAT\n", fat_header->fats);
       return 1;
   }
-  if(fat_header->media!=0xF0 && fat_header->media<0xF8)
-  {	/* Legal values are 0xF0, 0xF8-0xFF */
-    screen_buffer_add("check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
-    log_error("check_FAT: Bad media descriptor (0x%2x!=0xf8)\n",fat_header->media);
-    return 1;
-  }
-  if(fat_header->media!=0xF8)
-  { /* the only value I have ever seen is 0xF8 */
-    screen_buffer_add("check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
-    log_warning("check_FAT: Unusual media descriptor (0x%2x!=0xf8)\n",fat_header->media);
-  }
   if(fat_sector_size(fat_header)!=disk_car->sector_size)
   {
-    screen_buffer_add("check_FAT: Incorrect number of bytes per sector %u (FAT) != %u (HD)\n",
+    screen_buffer_add("check_FAT: number of bytes per sector mismatches %u (FAT) != %u (HD)\n",
 	fat_sector_size(fat_header), disk_car->sector_size);
-    log_error("check_FAT: Incorrect number of bytes per sector %u (FAT) != %u (HD)\n",
+    log_error("check_FAT: number of bytes per sector mismatches %u (FAT) != %u (HD)\n",
 	fat_sector_size(fat_header), disk_car->sector_size);
     return 1;
   }
   fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
-  part_size=(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect));
+  part_size=(fat_sectors(fat_header)>0?fat_sectors(fat_header):le32(fat_header->total_sect));
   start_fat1=le16(fat_header->reserved);
   start_fat2=start_fat1+(fat_header->fats>1?fat_length:0);
   start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+fat_sector_size(fat_header)-1)/fat_sector_size(fat_header);
@@ -502,6 +495,12 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
   {
     log_info("number of cluster = %lu\n",no_of_cluster);
   }
+  if(fat_header->media!=0xF0 && fat_header->media<0xF8)
+  {	/* Legal values are 0xF0, 0xF8-0xFF */
+    screen_buffer_add("check_FAT: Bad media descriptor (0x%02x!=0xf8)\n",fat_header->media);
+    log_error("check_FAT: Bad media descriptor (0x%02x!=0xf8)\n",fat_header->media);
+    return 1;
+  }
   if(no_of_cluster<4085)
   {
     if(verbose>0)
@@ -511,7 +510,7 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
           offset2head(disk_car,partition->part_offset),
           offset2sector(disk_car,partition->part_offset));
     }
-    if(sectors(fat_header)==0)
+    if(fat_sectors(fat_header)==0)
     {
       screen_buffer_add(msg_CHKFAT_SIZE);
       log_error(msg_CHKFAT_SIZE);
@@ -541,6 +540,11 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
       screen_buffer_add("Should be marked as FAT12\n");
       log_warning("Should be marked as FAT12\n");
     }
+    if(fat_header->media!=0xF0)
+    {
+      screen_buffer_add("check_FAT: Unusual media descriptor (0x%02x!=0xf0)\n", fat_header->media);
+      log_warning("check_FAT: Unusual media descriptor (0x%02x!=0xf0)\n", fat_header->media);
+    }
   }
   else if(no_of_cluster<65525)
   {
@@ -576,6 +580,11 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
       screen_buffer_add("Should be marked as FAT16\n");
       log_warning("Should be marked as FAT16\n");
     }
+    if(fat_header->media!=0xF8)
+    { /* the only value I have ever seen is 0xF8 */
+      screen_buffer_add("check_FAT: Unusual media descriptor (0x%02x!=0xf8)\n", fat_header->media);
+      log_warning("check_FAT: Unusual media descriptor (0x%02x!=0xf8)\n", fat_header->media);
+    }
   }
   else
   {
@@ -586,7 +595,7 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
           offset2head(disk_car,partition->part_offset),
           offset2sector(disk_car,partition->part_offset));
     }
-    if(sectors(fat_header)!=0)
+    if(fat_sectors(fat_header)!=0)
     {
       screen_buffer_add(msg_CHKFAT_SIZE);
       log_error(msg_CHKFAT_SIZE);
@@ -617,6 +626,16 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
       screen_buffer_add("Should be marked as FAT32\n");
       log_warning("Should be marked as FAT32\n");
     }
+    if(fat_header->media!=0xF8)
+    { /* the only value I have ever seen is 0xF8 */
+      screen_buffer_add("check_FAT: Unusual media descriptor (0x%02x!=0xf8)\n", fat_header->media);
+      log_warning("check_FAT: Unusual media descriptor (0x%02x!=0xf8)\n", fat_header->media);
+    }
+    if(fat_header->BS_DrvNum!=0 && (fat_header->BS_DrvNum<0x80 || fat_header->BS_DrvNum>0x87))
+    {
+      screen_buffer_add("Warning: Unusual drive number (0x%02x!=0x80)\n", fat_header->BS_DrvNum);
+      log_warning("Warning: Unusual drive number (0x%02x!=0x80)\n", fat_header->BS_DrvNum);
+    }
   }
   if(partition->part_size>0)
   {
@@ -660,14 +679,14 @@ int test_FAT(disk_t *disk_car, const struct fat_boot_sector *fat_header, partiti
     comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
   if(le16(fat_header->heads)!=disk_car->geom.heads_per_cylinder)
   {
-    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",
+    screen_buffer_add("Warning: number of heads/cylinder mismatches %u (FAT) != %u (HD)\n",
 	le16(fat_header->heads), disk_car->geom.heads_per_cylinder);
     log_warning("heads/cylinder %u (FAT) != %u (HD)\n",
 	le16(fat_header->heads), disk_car->geom.heads_per_cylinder);
   }
   if(le16(fat_header->secs_track)!=disk_car->geom.sectors_per_head)
   {
-    screen_buffer_add("Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",
+    screen_buffer_add("Warning: number of sectors per track mismatches %u (FAT) != %u (HD)\n",
 	le16(fat_header->secs_track), disk_car->geom.sectors_per_head);
     log_warning("sect/track %u (FAT) != %u (HD)\n",
 	le16(fat_header->secs_track), disk_car->geom.sectors_per_head);
@@ -725,30 +744,23 @@ int comp_FAT(disk_t *disk, const partition_t *partition, const unsigned long int
   return 0;
 }
 
-unsigned int fat_sector_size(const struct fat_boot_sector *fat_header)
-{ return (fat_header->sector_size[1]<<8)+fat_header->sector_size[0]; }
-
-unsigned int get_dir_entries(const struct fat_boot_sector *fat_header)
-{ return (fat_header->dir_entries[1]<<8)+fat_header->dir_entries[0]; }
-
-unsigned int sectors(const struct fat_boot_sector *fat_header)
-{ return (fat_header->sectors[1]<<8)+fat_header->sectors[0]; }
-
 unsigned long int fat32_get_free_count(const unsigned char *boot_fat32, const unsigned int sector_size)
 {
-  return (boot_fat32[sector_size+0x1E8+3]<<24)+(boot_fat32[sector_size+0x1E8+2]<<16)+(boot_fat32[sector_size+0x1E8+1]<<8)+boot_fat32[sector_size+0x1E8];
+  const struct fat_fsinfo *fsinfo=(const struct fat_fsinfo *)&boot_fat32[sector_size];
+  return le32(fsinfo->freecnt);
 }
 
 unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const unsigned int sector_size)
 {
-  return (boot_fat32[sector_size+0x1EC+3]<<24)+(boot_fat32[sector_size+0x1EC+2]<<16)+(boot_fat32[sector_size+0x1EC+1]<<8)+boot_fat32[sector_size+0x1EC];
+  const struct fat_fsinfo *fsinfo=(const struct fat_fsinfo *)&boot_fat32[sector_size];
+  return le32(fsinfo->nextfree);
 }
 
 int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind, const int backup)
 {
   if(test_FAT(disk_car, fat_header, partition, verbose, dump_ind))
     return 1;
-  partition->part_size=(uint64_t)(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect)) *
+  partition->part_size=(uint64_t)(fat_sectors(fat_header)>0?fat_sectors(fat_header):le32(fat_header->total_sect)) *
     fat_sector_size(fat_header);
   /* test_FAT has set partition->upart_type */
   partition->sborg_offset=0;
@@ -776,7 +788,7 @@ int recover_FAT(disk_t *disk_car, const struct fat_boot_sector*fat_header, parti
             offset2head(disk_car,partition->part_offset),
             offset2sector(disk_car,partition->part_offset));
       }
-      if(sectors(fat_header)!=0)
+      if(fat_sectors(fat_header)!=0)
         partition->part_type_i386=P_16FAT;
       else if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
         partition->part_type_i386=P_16FATBD;
@@ -819,10 +831,10 @@ static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const s
   partition->fsname[0]='\0';
   if((fat_header->sectors_per_cluster>0)&&(fat_header->sectors_per_cluster<=128))
   {
-    unsigned char *buffer=(unsigned char*)MALLOC(fat_header->sectors_per_cluster*disk_car->sector_size);
-    if((unsigned)disk_car->pread(disk_car, buffer,
-	  fat_header->sectors_per_cluster * disk_car->sector_size,
-	  partition->part_offset + (le16(fat_header->reserved) + fat_header->fats * le32(fat_header->fat32_length) + (uint64_t)(le32(fat_header->root_cluster) - 2) * fat_header->sectors_per_cluster) * disk_car->sector_size) != fat_header->sectors_per_cluster * disk_car->sector_size)
+    const unsigned int cluster_size=fat_header->sectors_per_cluster*disk_car->sector_size;
+    unsigned char *buffer=(unsigned char*)MALLOC(cluster_size);
+    if((unsigned)disk_car->pread(disk_car, buffer, cluster_size,
+	  partition->part_offset + (le16(fat_header->reserved) + fat_header->fats * le32(fat_header->fat32_length) + (uint64_t)(le32(fat_header->root_cluster) - 2) * fat_header->sectors_per_cluster) * disk_car->sector_size) != cluster_size) 
     {
       log_error("fat32_set_part_name() cannot read FAT32 root cluster.\n");
     }
@@ -858,7 +870,7 @@ static int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const s
 
 int check_OS2MB(disk_t *disk, partition_t *partition, const int verbose)
 {
-  unsigned char buffer[0x200];
+  unsigned char buffer[disk->sector_size];
   if((unsigned)disk->pread(disk, &buffer, disk->sector_size, partition->part_offset) != disk->sector_size)
   {
     screen_buffer_add("check_OS2MB: Read error\n");
@@ -1056,7 +1068,7 @@ int fat32_free_info(disk_t *disk_car,const partition_t *partition, const unsigne
 int check_VFAT_volume_name(const char *name, const unsigned int max_size)
 {
   unsigned int i;
-  for(i=0;name[i]!='\0' && i<max_size;i++)
+  for(i=0; i<max_size && name[i]!='\0'; i++)
   {
     if(name[i] < 0x20)
       return 1;
