@@ -2,7 +2,7 @@
 
     File: fat.h
 
-    Copyright (C) 1998-2004,2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2004,2007-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@
 
 #ifndef _FAT_H
 #define _FAT_H
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define FAT1X_PART_NAME 0x2B
 #define FAT32_PART_NAME 0x47
 #define FAT_NAME1       0x36
@@ -29,8 +33,6 @@
 #define NTFS_NAME       0x03
 #define OS2_NAME        0x03
 
-#define NBR_SECT        8
-
 /*
  * FAT partition boot sector information, taken from the Linux
  * kernel sources.
@@ -71,7 +73,17 @@ struct fat_boot_sector {
 	/* */
 	uint8_t	nothing[420];	/* 0x5A */
 	uint16_t	marker;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
+
+struct fat_fsinfo {
+  uint32_t leadsig;		/* 0x41615252 */
+  uint8_t reserved1[480];
+  uint32_t strucsig;		/* 0x61417272 */
+  uint32_t freecnt;     	/* free clusters 0xfffffffff if unknown */
+  uint32_t nextfree;		/* next free cluster */
+  uint8_t reserved3[12];
+  uint32_t magic3;		/* 0xAA550000 */
+} __attribute__ ((gcc_struct, __packed__));
 
 struct msdos_dir_entry {
 	int8_t	name[8],ext[3];		/* 00 name and extension */
@@ -86,7 +98,7 @@ struct msdos_dir_entry {
         uint16_t        date;		/* 18 */
         uint16_t        start;		/* 1A */
 	uint32_t	size;		/* 1C file size (in bytes) */
-};
+} __attribute__ ((gcc_struct, __packed__));
 
 /* Up to 13 characters of the name */
 struct msdos_dir_slot {
@@ -101,28 +113,19 @@ struct msdos_dir_slot {
 };
 
 
-int comp_FAT(disk_t *disk_car,const partition_t *partition, const unsigned long int fat_size, const unsigned long int sect_res);
+int comp_FAT(disk_t *disk,const partition_t *partition, const unsigned long int fat_size, const unsigned long int sect_res);
 int log_fat2_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size);
 
-unsigned int get_next_cluster(disk_t *disk_car,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster);
-int set_next_cluster(disk_t *disk_car,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster, const unsigned int next_cluster);
+unsigned int get_next_cluster(disk_t *disk,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster);
+int set_next_cluster(disk_t *disk,const partition_t *partition, const upart_type_t upart_type,const int offset, const unsigned int cluster, const unsigned int next_cluster);
 
 int is_fat(const partition_t *partition);
-int is_fat12(const partition_t *partition);
-int is_fat16(const partition_t *partition);
-int is_fat32(const partition_t *partition);
 int is_part_fat(const partition_t *partition);
 int is_part_fat12(const partition_t *partition);
 int is_part_fat16(const partition_t *partition);
 int is_part_fat32(const partition_t *partition);
-unsigned int get_dir_entries(const struct fat_boot_sector *fat_header);
-int dump_fat_info(const struct fat_boot_sector*fh1, const upart_type_t upart_type, const unsigned int sector_size);
-int dump_2fat_info(const struct fat_boot_sector*fh1, const struct fat_boot_sector*fh2, const upart_type_t upart_type, const unsigned int sector_size);
-int fat32_set_part_name(disk_t *disk_car, partition_t *partition, const struct fat_boot_sector*fat_header);
-unsigned int fat_sector_size(const struct fat_boot_sector *fat_header);
-unsigned int sectors(const struct fat_boot_sector *fat_header);
-unsigned int fat32_get_prev_cluster(disk_t *disk_car,const partition_t *partition, const unsigned int fat_offset, const unsigned int cluster, const unsigned int no_of_cluster);
-int fat32_free_info(disk_t *disk_car,const partition_t *partition, const unsigned int fat_offset, const unsigned int no_of_cluster, unsigned int *next_free, unsigned int*free_count);
+unsigned int fat32_get_prev_cluster(disk_t *disk,const partition_t *partition, const unsigned int fat_offset, const unsigned int cluster, const unsigned int no_of_cluster);
+int fat32_free_info(disk_t *disk,const partition_t *partition, const unsigned int fat_offset, const unsigned int no_of_cluster, unsigned int *next_free, unsigned int*free_count);
 unsigned long int fat32_get_free_count(const unsigned char *boot_fat32, const unsigned int sector_size);
 unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const unsigned int sector_size);
 
@@ -147,16 +150,15 @@ unsigned long int fat32_get_next_free(const unsigned char *boot_fat32, const uns
 #define FAT16_EOC	0xFFF8
 #define FAT32_BAD	0x0FFFFFF7
 #define FAT32_EOC	0x0FFFFFF8
-#endif
 #define FAT1x_BOOT_SECTOR_SIZE 0x200
 
-int recover_FAT(disk_t *disk_car,const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind, const int backup);
-int check_FAT(disk_t *disk_car,partition_t *partition, const int verbose);
-int test_FAT(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind);
-int recover_HPFS(disk_t *disk_car, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind);
-int check_HPFS(disk_t *disk_car,partition_t *partition, const int verbose);
-int test_HPFS(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind);
-int recover_OS2MB(disk_t *disk_car, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind);
-int check_OS2MB(disk_t *disk_car,partition_t *partition, const int verbose);
-int test_OS2MB(disk_t *disk_car,const struct fat_boot_sector *fat_header, partition_t *partition,const int verbose, const int dump_ind);
+int recover_FAT(disk_t *disk, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind, const int backup);
+int check_FAT(disk_t *disk, partition_t *partition, const int verbose);
+int test_FAT(disk_t *disk, const struct fat_boot_sector *fat_header, partition_t *partition, const int verbose, const int dump_ind);
+int recover_OS2MB(const disk_t *disk, const struct fat_boot_sector*fat_header, partition_t *partition, const int verbose, const int dump_ind);
+int check_OS2MB(disk_t *disk, partition_t *partition, const int verbose);
 int check_VFAT_volume_name(const char *name, const unsigned int max_size);
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
+#endif
