@@ -41,6 +41,7 @@
 #include "intrf.h"
 #include "intrfn.h"
 #include "dir.h"
+#include "fat.h"
 #include "fat_dir.h"
 #include "list.h"
 #include "filegen.h"
@@ -51,6 +52,8 @@
 #include "fat_unformat.h"
 #include "pnext.h"
 #include "setdate.h"
+#include "fat_common.h"
+#include <assert.h>
 
 #define READ_SIZE 4*1024*1024
 static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const int verbose, unsigned int *sectors_per_cluster, uint64_t *offset_org, alloc_data_t *list_search_space)
@@ -64,6 +67,7 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
   alloc_data_t *current_search_space;
   unsigned char *buffer_start=(unsigned char *)MALLOC(READ_SIZE);
   unsigned char *buffer=buffer_start;
+  assert(disk->sector_size!=0);
   current_search_space=td_list_entry(list_search_space->list.next, alloc_data_t, list);
   if(current_search_space!=list_search_space)
     offset=current_search_space->start;
@@ -79,12 +83,9 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
   while(current_search_space!=list_search_space && nbr_subdir<10)
   {
     const uint64_t old_offset=offset;
-    if(buffer[0]=='.' &&
-	memcmp(buffer,         ".          ", 8+3)==0 &&
-	memcmp(&buffer[0x20], "..         ", 8+3)==0)
+    if(buffer[0]=='.' && is_fat_directory(buffer))
     {
-      const unsigned long int cluster=(buffer[0*0x20+0x15]<<24) + (buffer[0*0x20+0x14]<<16) +
-	(buffer[0*0x20+0x1B]<<8) + buffer[0*0x20+0x1A];
+      const unsigned long int cluster=fat_get_cluster_from_entry((const struct msdos_dir_entry *)buffer);
       log_info("sector %lu, cluster %lu\n",
 	  (unsigned long)(offset/disk->sector_size), cluster);
       sector_cluster[nbr_subdir].cluster=cluster;
@@ -103,9 +104,9 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
       {
 	const time_t current_time=time(NULL);
 	if(current_time==previous_time)
-	  diff_offset>>=1;
-	else
 	  diff_offset<<=1;
+	else
+	  diff_offset>>=1;
 	if(diff_offset < disk->sector_size)
 	  diff_offset=disk->sector_size;
 	next_offset=offset+diff_offset;
@@ -124,6 +125,12 @@ static int pfind_sectors_per_cluster(disk_t *disk, partition_t *partition, const
       }
       if(disk->pread(disk, buffer_start, READ_SIZE, offset) != READ_SIZE)
       {
+#ifdef HAVE_NCURSES
+	wmove(stdscr,11,0);
+	wclrtoeol(stdscr);
+	wprintw(stdscr,"Error reading sector %10lu\n",
+	    (unsigned long)((offset - partition->part_offset) / disk->sector_size));
+#endif
       }
     }
   } /* end while(current_search_space!=list_search_space) */
@@ -146,7 +153,7 @@ static int fat_copy_file(disk_t *disk, const partition_t *partition, const unsig
 #ifdef __MINGW32__
   mkdir(new_file);
 #else
-  mkdir(new_file, 0775);
+  (void)mkdir(new_file, 0775);
 #endif
 #endif
   snprintf(new_file, 1024, "%s.%u/inode_%u/%s", recup_dir, dir_num, inode_num,
@@ -234,18 +241,16 @@ static pstatus_t fat_unformat_aux(struct ph_param *params, const struct ph_optio
   offset=set_search_start(params, &current_search_space, list_search_space);
   if(options->verbose>0)
     info_list_search_space(list_search_space, current_search_space, disk->sector_size, 0, options->verbose);
-  disk->pread(disk, buffer, READ_SIZE, offset);
+  disk->pread(disk, buffer_start, READ_SIZE, offset);
   for(;offset < offset_end; offset+=cluster_size)
   {
-    if(buffer[0]=='.' &&
-	memcmp(buffer,         ".          ", 8+3)==0 &&
-	memcmp(&buffer[0x20], "..         ", 8+3)==0)
+    if(buffer[0]=='.' && is_fat_directory(buffer))
     {
       file_info_t dir_list = {
 	.list = TD_LIST_HEAD_INIT(dir_list.list),
 	.name = NULL
       };
-      dir_fat_aux(buffer,read_size,0,0, &dir_list);
+      dir_fat_aux(buffer, read_size, 0, &dir_list);
       if(!td_list_empty(&dir_list.list))
       {
 	struct td_list_head *file_walker = NULL;
@@ -324,7 +329,7 @@ static pstatus_t fat_unformat_aux(struct ph_param *params, const struct ph_optio
 	    (unsigned long long)((offset-partition->part_offset)/disk->sector_size),
 	    (unsigned long long)((partition->part_size-1)/disk->sector_size));
       }
-      if(disk->pread(disk, buffer, READ_SIZE, offset) != READ_SIZE)
+      if(disk->pread(disk, buffer_start, READ_SIZE, offset) != READ_SIZE)
       {
 #ifdef HAVE_NCURSES
 	wmove(stdscr,11,0);
@@ -412,7 +417,7 @@ pstatus_t fat_unformat(struct ph_param *params, const struct ph_options *options
     params->blocksize=sectors_per_cluster * params->disk->sector_size;
 #ifdef HAVE_NCURSES
     if(options->expert>0)
-      params->blocksize=menu_choose_blocksize(params->blocksize, params->disk->sector_size, &offset);
+      menu_choose_blocksize(&params->blocksize, &offset, params->disk->sector_size);
 #endif
     update_blocksize(params->blocksize, list_search_space, offset);
   }
