@@ -34,6 +34,11 @@
 #include "crc.h"
 
 /* #define DEBUG_ACE */
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
 
 static void register_header_check_ace(file_stat_t *file_stat);
 
@@ -57,7 +62,8 @@ struct header_ace {
                            bit 0 indicates if field add size is preset */
   uint32_t addsize;    /** an optional field which represents the size of
                            an additional block without specified structure */
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
+
 typedef struct header_ace ace_header_t;
 
 static void file_check_ace(file_recovery_t *file_recovery)
@@ -65,7 +71,7 @@ static void file_check_ace(file_recovery_t *file_recovery)
   file_recovery->offset_error = 0;
   file_recovery->offset_ok = 0;
   file_recovery->file_size = 0;
-  if(fseek(file_recovery->handle, 0, SEEK_SET)<0)
+  if(my_fseek(file_recovery->handle, 0, SEEK_SET)<0)
     return ;
 #ifdef DEBUG_ACE
   log_trace("file_check_ace\n");
@@ -84,7 +90,7 @@ static void file_check_ace(file_recovery_t *file_recovery)
       file_recovery->file_size=0;
       return ;
     }
-    if(fseek(file_recovery->handle, -sizeof(h)+4, SEEK_CUR)<0)
+    if(my_fseek(file_recovery->handle, -sizeof(h)+4, SEEK_CUR)<0)
     {
       file_recovery->offset_error=file_recovery->file_size;
       file_recovery->file_size=0;
@@ -114,7 +120,7 @@ static void file_check_ace(file_recovery_t *file_recovery)
     }
 
     /* Minimal size is type+flags */
-    if (le16(h.size) < 1+2)
+    if (le16(h.size) < 1U + 2U)
     {
 #ifdef DEBUG_ACE
       log_trace("file_ace: Invalid block size %u\n", le16(h.size));
@@ -128,15 +134,12 @@ static void file_check_ace(file_recovery_t *file_recovery)
       /* Header hardly ever bigger than a filename */
 #define BUF_SIZE 4096
       unsigned char buffer[BUF_SIZE];
-      int len=le16(h.size);
+      unsigned int len=le16(h.size);
       uint32_t crc32=0xFFFFFFFF;
-
       while (len>0)
       {
-        const int count = ((len>BUF_SIZE) ? BUF_SIZE : len);
-        const int bytes = fread(buffer, 1, count, file_recovery->handle);
-
-        if (bytes != count)
+        const unsigned int count = ((len>BUF_SIZE) ? BUF_SIZE : len);
+        if(fread(buffer, 1, count, file_recovery->handle) != count)
         {
 #ifdef DEBUG_ACE
           log_trace("file_ace: truncated file\n");
@@ -146,7 +149,7 @@ static void file_check_ace(file_recovery_t *file_recovery)
           return ;
         }
         crc32=get_crc32(buffer, count, crc32);
-        len -= count;
+	len -= count;
       }
       if (le16(h.crc16) != (crc32&0xFFFF))
       {
@@ -159,12 +162,12 @@ static void file_check_ace(file_recovery_t *file_recovery)
       }
     }
     /* Add its header size */
-    file_recovery->file_size += 2 + 2 + le16(h.size);	/* +2: CRC16, +2: size */
+    file_recovery->file_size += 2U + 2 + le16(h.size);	/* +2: CRC16, +2: size */
     /* If addsize flag, add complementary size */
     if (le16(h.flags)&1)
     {
       file_recovery->file_size += le32(h.addsize);
-      if(fseek(file_recovery->handle, file_recovery->file_size, SEEK_SET)<0)
+      if(my_fseek(file_recovery->handle, file_recovery->file_size, SEEK_SET)<0)
       {
 	file_recovery->offset_error=file_recovery->file_size;
 	file_recovery->file_size=0;
