@@ -33,10 +33,10 @@
 
 static void register_header_check_blend(file_stat_t *file_stat);
 static int header_check_blend(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static int data_check_blend4le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static int data_check_blend8le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static int data_check_blend4be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static int data_check_blend8be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t data_check_blend4le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t data_check_blend8le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t data_check_blend4be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t data_check_blend8be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_blend= {
   .extension="blend",
@@ -58,32 +58,32 @@ static void register_header_check_blend(file_stat_t *file_stat)
 
 static int header_check_blend(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,blend_header,sizeof(blend_header))==0 && (buffer[7]=='_' || buffer[7]=='-'))
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->calculated_file_size=12;
-    if(buffer[8]=='v')
-    { /* Little endian */
-      if(buffer[7]=='_')
-        file_recovery_new->data_check=&data_check_blend4le;
-      else
-        file_recovery_new->data_check=&data_check_blend8le;
-    }
-    else
-    { /* Big endian */
-      if(buffer[7]=='_')
-        file_recovery_new->data_check=&data_check_blend4be;
-      else
-        file_recovery_new->data_check=&data_check_blend8be;
-    }
-    file_recovery_new->extension=file_hint_blend.extension;
-    file_recovery_new->file_check=&file_check_size;
+  if(buffer[7]!='_' && buffer[7]!='-')
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_blend.extension;
+  if(file_recovery_new->blocksize < 0x14)
     return 1;
+  file_recovery_new->calculated_file_size=12;
+  if(buffer[8]=='v')
+  { /* Little endian */
+    if(buffer[7]=='_')
+      file_recovery_new->data_check=&data_check_blend4le;
+    else
+      file_recovery_new->data_check=&data_check_blend8le;
   }
-  return 0;
+  else
+  { /* Big endian */
+    if(buffer[7]=='_')
+      file_recovery_new->data_check=&data_check_blend4be;
+    else
+      file_recovery_new->data_check=&data_check_blend8be;
+  }
+  file_recovery_new->file_check=&file_check_size;
+  return 1;
 }
 
-static int data_check_blend4le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_blend4le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 0x14 < file_recovery->file_size + buffer_size/2)
@@ -100,14 +100,14 @@ static int data_check_blend4le(const unsigned char *buffer, const unsigned int b
     if(memcmp(&buffer[i],blend_header_footer,sizeof(blend_header_footer))==0)
     {
       file_recovery->calculated_file_size+=0x14;
-      return 2;
+      return DC_STOP;
     }
     file_recovery->calculated_file_size+=0x14+len;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int data_check_blend8le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_blend8le(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + 0x18 < file_recovery->file_size + buffer_size/2)
   {
@@ -123,14 +123,14 @@ static int data_check_blend8le(const unsigned char *buffer, const unsigned int b
     if(memcmp(&buffer[i],blend_header_footer,sizeof(blend_header_footer))==0)
     {
       file_recovery->calculated_file_size+=0x18;
-      return 2;
+      return DC_STOP;
     }
     file_recovery->calculated_file_size+=0x18+len;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int data_check_blend4be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_blend4be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + 0x14 < file_recovery->file_size + buffer_size/2)
   {
@@ -146,14 +146,14 @@ static int data_check_blend4be(const unsigned char *buffer, const unsigned int b
     if(memcmp(&buffer[i],blend_header_footer,sizeof(blend_header_footer))==0)
     {
       file_recovery->calculated_file_size+=0x14;
-      return 2;
+      return DC_STOP;
     }
     file_recovery->calculated_file_size+=0x14+len;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int data_check_blend8be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_blend8be(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + 0x18 < file_recovery->file_size + buffer_size/2)
   {
@@ -169,9 +169,9 @@ static int data_check_blend8be(const unsigned char *buffer, const unsigned int b
     if(memcmp(&buffer[i],blend_header_footer,sizeof(blend_header_footer))==0)
     {
       file_recovery->calculated_file_size+=0x18;
-      return 2;
+      return DC_STOP;
     }
     file_recovery->calculated_file_size+=0x18+len;
   }
-  return 1;
+  return DC_CONTINUE;
 }
