@@ -32,7 +32,6 @@
 #include "filegen.h"
 
 static void register_header_check_cab(file_stat_t *file_stat);
-static int header_check_cab(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_cab= {
   .extension="cab",
@@ -44,7 +43,6 @@ const file_hint_t file_hint_cab= {
   .register_header_check=&register_header_check_cab
 };
 
-static const unsigned char cab_header[4]  = { 'M','S','C','F'};
 struct cab_header {
   uint32_t magic;
   uint32_t hdr_checksum;
@@ -58,17 +56,12 @@ struct cab_header {
   uint16_t flags;
   uint16_t setid;
   uint16_t number;
-} __attribute__ ((__packed__));
-
-static void register_header_check_cab(file_stat_t *file_stat)
-{
-  register_header_check(0, cab_header,sizeof(cab_header), &header_check_cab, file_stat);
-}
+} __attribute__ ((gcc_struct, __packed__));
 
 static int header_check_cab(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct cab_header *cab_hdr=(const struct cab_header*)buffer;
-  if(memcmp(buffer,cab_header,sizeof(cab_header))==0 && le16(cab_hdr->cab_version)==0x0103)
+  if(le16(cab_hdr->cab_version)==0x0103 && le32(cab_hdr->filesize) >= sizeof(struct cab_header))
   {
     reset_file_recovery(file_recovery_new);
     file_recovery_new->extension=file_hint_cab.extension;
@@ -79,3 +72,9 @@ static int header_check_cab(const unsigned char *buffer, const unsigned int buff
   }
   return 0;
 }
+
+static void register_header_check_cab(file_stat_t *file_stat)
+{
+  static const unsigned char cab_header[4]  = { 'M','S','C','F'};
+  register_header_check(0, cab_header,sizeof(cab_header), &header_check_cab, file_stat);
+}
