@@ -29,9 +29,9 @@
 #include <stdio.h>
 #include "types.h"
 #include "filegen.h"
+#include "common.h"
 
 static void register_header_check_djv(file_stat_t *file_stat);
-static int header_check_djv(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_djv= {
   .extension="djv",
@@ -43,23 +43,29 @@ const file_hint_t file_hint_djv= {
   .register_header_check=&register_header_check_djv
 };
 
-static const unsigned char djv_header[8]= { 'A','T','&','T','F','O','R','M'};
-
-static void register_header_check_djv(file_stat_t *file_stat)
+struct djv_header
 {
-  register_header_check(0, djv_header,sizeof(djv_header), &header_check_djv, file_stat);
-}
+  uint32_t magic;
+  uint32_t type;
+  uint32_t size;
+} __attribute__ ((gcc_struct, __packed__));
 
 static int header_check_djv(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,djv_header,sizeof(djv_header))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_djv.extension;
-    file_recovery_new->calculated_file_size=((uint64_t)buffer[8]<<24)+((uint64_t)buffer[8+1]<<16)+((uint64_t)buffer[8+2]<<8)+(uint64_t)buffer[8+3]+12;
-    file_recovery_new->data_check=&data_check_size;
-    file_recovery_new->file_check=&file_check_size;
-    return 1;
-  }
-  return 0;
+  const struct djv_header *hdr=(const struct djv_header *)buffer;
+  const uint64_t size=be32(hdr->size);
+  if(size==0 || size +12 > file_hint_djv.max_filesize)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_djv.extension;
+  file_recovery_new->calculated_file_size=size+12;
+  file_recovery_new->data_check=&data_check_size;
+  file_recovery_new->file_check=&file_check_size;
+  return 1;
+}
+
+static void register_header_check_djv(file_stat_t *file_stat)
+{
+  static const unsigned char djv_header[8]= { 'A','T','&','T','F','O','R','M'};
+  register_header_check(0, djv_header,sizeof(djv_header), &header_check_djv, file_stat);
 }
