@@ -31,7 +31,6 @@
 #include "filegen.h"
 
 static void register_header_check_dwg(file_stat_t *file_stat);
-static int header_check_dwg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_dwg= {
   .extension="dwg",
@@ -43,17 +42,23 @@ const file_hint_t file_hint_dwg= {
   .register_header_check=&register_header_check_dwg
 };
 
-static const unsigned char dwg_header_12[11]= {'A', 'C', '1', '0', '1', '2', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_13[11]= {'A', 'C', '1', '0', '1', '3', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_14[11]= {'A', 'C', '1', '0', '1', '4', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_15[11]= {'A', 'C', '1', '0', '1', '5', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_18[11]= {'A', 'C', '1', '0', '1', '8', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_21[11]= {'A', 'C', '1', '0', '2', '1', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_23[11]= {'A', 'C', '1', '0', '2', '3', 0x00, 0x00, 0x00, 0x00, 0x00};
-static const unsigned char dwg_header_24[11]= {'A', 'C', '1', '0', '2', '4', 0x00, 0x00, 0x00, 0x00, 0x00};
+static int header_check_dwg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_dwg.extension;
+  return 1;
+}
 
 static void register_header_check_dwg(file_stat_t *file_stat)
 {
+  static const unsigned char dwg_header_12[11]= {'A', 'C', '1', '0', '1', '2', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_13[11]= {'A', 'C', '1', '0', '1', '3', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_14[11]= {'A', 'C', '1', '0', '1', '4', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_15[11]= {'A', 'C', '1', '0', '1', '5', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_18[11]= {'A', 'C', '1', '0', '1', '8', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_21[11]= {'A', 'C', '1', '0', '2', '1', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_23[11]= {'A', 'C', '1', '0', '2', '3', 0x00, 0x00, 0x00, 0x00, 0x00};
+  static const unsigned char dwg_header_24[11]= {'A', 'C', '1', '0', '2', '4', 0x00, 0x00, 0x00, 0x00, 0x00};
   register_header_check(0, dwg_header_12,sizeof(dwg_header_12), &header_check_dwg, file_stat);
   register_header_check(0, dwg_header_13,sizeof(dwg_header_13), &header_check_dwg, file_stat);
   register_header_check(0, dwg_header_14,sizeof(dwg_header_14), &header_check_dwg, file_stat);
@@ -63,20 +68,3 @@ static void register_header_check_dwg(file_stat_t *file_stat)
   register_header_check(0, dwg_header_23,sizeof(dwg_header_23), &header_check_dwg, file_stat);
   register_header_check(0, dwg_header_24,sizeof(dwg_header_24), &header_check_dwg, file_stat);
 }
-
-static int header_check_dwg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
-{
-  if(memcmp(buffer,dwg_header_12,sizeof(dwg_header_12))==0 ||
-      memcmp(buffer,dwg_header_13,sizeof(dwg_header_13))==0 ||
-      memcmp(buffer,dwg_header_14,sizeof(dwg_header_14))==0 ||
-      memcmp(buffer,dwg_header_15,sizeof(dwg_header_15))==0 ||
-      memcmp(buffer,dwg_header_18,sizeof(dwg_header_18))==0 ||
-      memcmp(buffer,dwg_header_21,sizeof(dwg_header_21))==0 ||
-      memcmp(buffer,dwg_header_24,sizeof(dwg_header_24))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_dwg.extension;
-    return 1;
-  }
-  return 0;
-}
