@@ -37,6 +37,12 @@
 #include "log.h"
 #endif
 
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
 static void register_header_check_hdf(file_stat_t *file_stat);
 
 const file_hint_t file_hint_hdf= {
@@ -49,12 +55,11 @@ const file_hint_t file_hint_hdf= {
   .register_header_check=&register_header_check_hdf
 };
 
-static const unsigned char hdf_header[4]=  { 0x0e, 0x03, 0x13, 0x01};
 struct ddh_struct
 {
   uint16_t	size;
   uint32_t	next;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 struct dd_struct
 {
@@ -62,7 +67,7 @@ struct dd_struct
   uint16_t	ref;
   uint32_t	offset;
   uint32_t	length;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 static void file_check_hdf(file_recovery_t *file_recovery)
 {
@@ -75,7 +80,8 @@ static void file_check_hdf(file_recovery_t *file_recovery)
     struct ddh_struct ddh;
     const struct dd_struct *p;
     unsigned int i;
-    if(fseek(file_recovery->handle, offset, SEEK_SET) < 0 ||
+    unsigned int size;
+    if(my_fseek(file_recovery->handle, offset, SEEK_SET) < 0 ||
 	fread(&ddh, sizeof(ddh), 1, file_recovery->handle) !=1 ||
 	be16(ddh.size)==0 ||
 	fread(dd, sizeof(struct dd_struct)*be16(ddh.size), 1, file_recovery->handle) !=1)
@@ -89,7 +95,8 @@ static void file_check_hdf(file_recovery_t *file_recovery)
 #ifdef DEBUG_HDF
     log_info("size=%u next=%lu\n", be16(ddh.size), be32(ddh.next));
 #endif
-    for(i=0, p=dd; i < be16(ddh.size); i++,p++)
+    size=be16(ddh.size);
+    for(i=0, p=dd; i < size; i++,p++)
     {
 #ifdef DEBUG_HDF
       log_info("tag=0x%04x, ref=%u, offset=%lu, length=%lu\n",
@@ -115,17 +122,17 @@ static void file_check_hdf(file_recovery_t *file_recovery)
 
 static int header_check_hdf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(&buffer[0], hdf_header, sizeof(hdf_header))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_hdf.extension;
-    file_recovery_new->file_check=&file_check_hdf;
-    return 1;
-  }
-  return 0;
+  const struct ddh_struct *ddh=(const struct ddh_struct *)&buffer[4];
+  if(be16(ddh->size)==0)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_hdf.extension;
+  file_recovery_new->file_check=&file_check_hdf;
+  return 1;
 }
 
 static void register_header_check_hdf(file_stat_t *file_stat)
 {
+  static const unsigned char hdf_header[4]=  { 0x0e, 0x03, 0x13, 0x01};
   register_header_check(0, hdf_header, sizeof(hdf_header), &header_check_hdf, file_stat);
 }
