@@ -60,7 +60,7 @@ struct ico_header
   uint16_t	reserved;
   uint16_t	type;
   uint16_t	count;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 struct ico_directory
 {
@@ -72,7 +72,7 @@ struct ico_directory
   uint16_t	bits_per_pixel;
   uint32_t	bitmap_size;
   uint32_t	bitmap_offset;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 static int header_check_ico(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
@@ -89,7 +89,8 @@ static int header_check_ico(const unsigned char *buffer, const unsigned int buff
        le16(ico_dir->bits_per_pixel)==8 ||
        le16(ico_dir->bits_per_pixel)==16 ||
        le16(ico_dir->bits_per_pixel)==32) &&
-       le32(ico_dir->bitmap_offset) >= sizeof(struct ico_header)+le16(ico->count)*sizeof(struct ico_directory))
+       le32(ico_dir->bitmap_offset) >= sizeof(struct ico_header)+le16(ico->count)*sizeof(struct ico_directory) &&
+       le32(ico_dir->bitmap_size) > 0)
   {
     reset_file_recovery(file_recovery_new);
     file_recovery_new->extension=file_hint_ico.extension;
