@@ -42,6 +42,7 @@
 #include <jpeglib.h>
 #include "suspend.h"
 #endif
+#include <ctype.h>      /* isprint */
 #include "filegen.h"
 #include "common.h"
 #include "log.h"
@@ -49,16 +50,23 @@
 #include "file_tiff.h"
 #include "setdate.h"
 
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_indd;
 extern const file_hint_t file_hint_mov;
 extern const file_hint_t file_hint_riff;
-extern int data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+extern data_check_t data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 
 static void register_header_check_jpg(file_stat_t *file_stat);
 static int header_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_jpg(file_recovery_t *file_recovery);
-int data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+data_check_t data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static int jpg_check_dht(const unsigned char *buffer, const unsigned int buffer_size, const unsigned i, const unsigned int size);
 
 const file_hint_t file_hint_jpg= {
   .extension="jpg",
@@ -70,25 +78,10 @@ const file_hint_t file_hint_jpg= {
   .register_header_check=&register_header_check_jpg
 };
 
-static const unsigned char jpg_header_app0[4]= { 0xff,0xd8,0xff,0xe0};
-static const unsigned char jpg_header_app1[4]= { 0xff,0xd8,0xff,0xe1};
-static const unsigned char jpg_header_app12[4]= { 0xff,0xd8,0xff,0xec};
-static const unsigned char jpg_header_com[4]= { 0xff,0xd8,0xff,0xfe};
-static const unsigned char jpg_footer[2]= { 0xff,0xd9};
-static const unsigned char jpg_header_app0_avi[0x0c]= {
-  0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'A', 'V', 'I', '1', 0x00, 0x00
-};
-static const unsigned char jpg_header_app0_jfif11_null[0x14]= {
-  0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'J', 'F', 'I', 'F', 0x00, 0x01,
-  0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
-};
-
 static void register_header_check_jpg(file_stat_t *file_stat)
 {
-  register_header_check(0, jpg_header_app0,sizeof(jpg_header_app0), &header_check_jpg, file_stat);
-  register_header_check(0, jpg_header_app1,sizeof(jpg_header_app1), &header_check_jpg, file_stat);
-  register_header_check(0, jpg_header_app12,sizeof(jpg_header_app12), &header_check_jpg, file_stat);
-  register_header_check(0, jpg_header_com,sizeof(jpg_header_com), &header_check_jpg, file_stat);
+  static const unsigned char jpg_header[3]= { 0xff,0xd8,0xff};
+  register_header_check(0, jpg_header, sizeof(jpg_header), &header_check_jpg, file_stat);
 }
 
 static void jpg_get_size(const unsigned char *buffer, const unsigned int buffer_size, unsigned int *height, unsigned int *width)
@@ -122,7 +115,7 @@ struct MP_IFD_Field
   uint16_t type;
   uint32_t count;
   char     value[0];
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 struct MP_Entry
 {
@@ -131,7 +124,82 @@ struct MP_Entry
   uint32_t offset;
   uint16_t dep1;
   uint16_t dep2;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
+
+static uint64_t check_mpo_be(const unsigned char *mpo, const uint64_t mpo_offset, const unsigned int size)
+{
+  const uint16_t *tmp16;
+  const uint32_t *tmp32=(const uint32_t *)(&mpo[4]);
+  unsigned int offset=be32(*tmp32);
+  unsigned int i;
+  unsigned int nbr;
+  unsigned int NumberOfImages=0;
+  unsigned int MPEntry_offset=0;
+  const struct MP_Entry* MPEntry;
+  uint64_t max_offset=0;
+#ifdef DEBUG_JPEG
+  log_info("check_mpo_be\n");
+#endif
+  if(offset+2 >= size)
+    return 0;
+  tmp16=(const uint16_t*)(&mpo[offset]);
+  nbr=be16(*tmp16);
+  offset+=2;
+  for(i=0; i< nbr; i++)
+  {
+    const struct MP_IFD_Field *field=(const struct MP_IFD_Field *)(&mpo[offset]);
+    if(offset+12 > size)
+      return 0;
+    switch(be16(field->tag))
+    {
+      case 0xb000:
+	/* MPFVersion, type must be undefined */
+	if(be16(field->type)!=7 || be32(field->count)!=4)
+	  return 0;
+	break;
+      case 0xb001:
+	/* NumberOfImages, type must be long */
+	if(be16(field->type)!=4 || be32(field->count)!=1)
+	  return 0;
+	{
+	  const uint32_t *tmp=(const uint32_t *)&field->value[0];
+	  NumberOfImages=be32(*tmp);
+	}
+	break;
+      case 0xb002:
+	/* MPEntry, type must be undefined */
+	if(be16(field->type)!=7 || be32(field->count)!=16*NumberOfImages)
+	  return 0;
+	{
+	  const uint32_t *tmp=(const uint32_t *)&field->value[0];
+	  MPEntry_offset=be32(*tmp);
+	}
+	break;
+    }
+    offset+=12;
+  }
+#ifdef DEBUG_JPEG
+  log_info("MPEntry_offset=%u, NumberOfImages=%u\n", MPEntry_offset, NumberOfImages);
+#endif
+  if(MPEntry_offset + 16*NumberOfImages > size)
+    return 0;
+  for(i=0, MPEntry=(const struct MP_Entry*)(&mpo[MPEntry_offset]);
+      i<NumberOfImages;
+      i++, MPEntry++)
+  {
+    uint64_t tmp=be32(MPEntry->offset)+be32(MPEntry->size);
+#ifdef DEBUG_JPEG
+    log_info("offset=%lu, size=%lu\n",
+	(long unsigned)be32(MPEntry->offset),
+	(long unsigned)be32(MPEntry->size));
+#endif
+    if(be32(MPEntry->offset)>0)
+      tmp+=mpo_offset;
+    if(max_offset < tmp)
+      max_offset = tmp;
+  }
+  return max_offset;
+}
 
 static uint64_t check_mpo_le(const unsigned char *mpo, const uint64_t mpo_offset, const unsigned int size)
 {
@@ -215,11 +283,22 @@ static void file_check_mpo(file_recovery_t *fr)
   unsigned int size=0;
   size_t nbytes;
   const unsigned char *mpo;
-  uint64_t max_offset=0;
+  {
+    /* Check the first jpg */
+    const uint64_t fs=fr->file_size;
+    file_check_jpg(fr);
+    if(fr->file_size==0)
+      return ;
+    fr->file_size=fs;
+  }
   do
   {
     offset+=2+size;
-    fseek(fr->handle, offset, SEEK_SET);
+    if(my_fseek(fr->handle, offset, SEEK_SET) < 0)
+    {
+      fr->file_size=0;
+      return ;
+    }
     nbytes=fread(&buffer, 1, sizeof(buffer), fr->handle);
 //    log_info("file_check_mpo offset=%llu => nbytes=%d, buffer=%02x %02x\n",
 //    (long long unsigned)offset, nbytes, buffer[0], buffer[1]);
@@ -235,7 +314,7 @@ static void file_check_mpo(file_recovery_t *fr)
 #ifdef DEBUG_JPEG
   log_info("Found at %lu\n", (long unsigned)offset);
 #endif
-  if(2+size < nbytes)
+  if(2+size > nbytes)
     size=nbytes-2;
   if(size<12)
   {
@@ -246,16 +325,60 @@ static void file_check_mpo(file_recovery_t *fr)
   size-=8;
   mpo=buffer+8;
   if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
-    max_offset=check_mpo_le(mpo, offset, size);
+  {
+    const uint64_t max_offset=check_mpo_le(mpo, offset, size);
+    fr->file_size=(max_offset > fr->file_size ? 0 : max_offset);
+  }
   else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
-    return ;
-  fr->file_size=( max_offset > fr->file_size ? 0: max_offset);
+  {
+    const uint64_t max_offset=check_mpo_be(mpo, offset, size);
+    fr->file_size=(max_offset > fr->file_size ? 0 : max_offset);
+  }
+  else
+    fr->file_size=0;
+}
+
+static int is_marker_valid(const unsigned int marker)
+{
+  switch(marker)
+  {
+    case 0x02 ... 0xbf:	/* Reserved */
+    case 0xc0:		/* SOF0 Start of Frame */
+    case 0xc4:		/* Define Huffman table */
+    case 0xc8:		/* Start of Frame (JPG) (Reserved for JPEG extensions) */
+    case 0xcc:		/* DAC arithmetic-coding conditioning*/
+    case 0xcf:		/* SOF15 */
+    case 0xd0 ... 0xd7:	/* JPEG_RST0 .. JPEG_RST7 markers */
+//    case 0xd8:	/* SOI Start of Image */
+//    case 0xd9:	/* EOI End of Image */
+//    case 0xda:	/* SOS: Start Of Scan */
+    case 0xdb:		/* DQT: Define Quantization Table */
+    case 0xdc:		/* DNL: Define Number of Lines */
+    case 0xdd:		/* DRI: define restart interval */
+    case 0xde:		/* DHP: define hierarchical progression */
+    case 0xe0 ... 0xef:	/* APP0 - APP15 */
+    case 0xf0 ... 0xfd:	/* Reserved for JPEG extensions */
+    case 0xfe:		/* COM */
+    case 0xff:
+      return 1;
+    default:
+      return 0;
+  }
 }
 
 static int header_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
+  unsigned int i=2;
+  time_t jpg_time=0;
+  if(file_recovery->file_stat!=NULL)
   {
+    static const unsigned char jpg_header_app0_avi[0x0c]= {
+      0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'A', 'V', 'I', '1', 0x00, 0x00
+    };
+    static const unsigned char jpg_header_app0_jfif11_null[0x14]= {
+      0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'J', 'F', 'I', 'F', 0x00, 0x01,
+      0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+    };
     unsigned int width=0;
     unsigned int height=0;
     jpg_get_size(buffer, buffer_size, &height, &width);
@@ -268,7 +391,7 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
     {
       /* Don't recover the thumb instead of the jpg itself */
       if( file_recovery->file_size <= 1024 &&
-	  memcmp(buffer, jpg_header_app12, sizeof(jpg_header_app12))==0)
+	buffer[3]==0xec)		/* APP12 */
       {
 	log_info("jpg %llu %llu\n",
 	    (long long unsigned)file_recovery->calculated_file_size,
@@ -277,17 +400,19 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
       }
       /* Don't recover the thumb instead of the jpg itself */
       if(file_recovery->file_size <= 4096 &&
-	  memcmp(buffer, jpg_header_app0, sizeof(jpg_header_app0))==0 &&
+	  buffer[3]==0xe0 &&
 	  width>0 && width<200 && height>0 && height<200)
       {
 	return 0;
       }
       /* Some JPG have two APP1 markers, avoid to dicard the first one */
-      if( memcmp(buffer, jpg_header_app1, sizeof(jpg_header_app1))==0 &&
+      if( buffer[3]==0xe1	&&
 	  memcmp(&buffer[6], "http://ns.adobe.com/xap/", 24)==0)
       {
 	return 0;
       }
+      if(file_recovery->file_check==&file_check_mpo)
+	return 0;
     }
     /* Don't extract jpg inside AVI */
     if( file_recovery->file_stat->file_hint==&file_hint_riff &&
@@ -298,31 +423,23 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
     if( file_recovery->file_stat->file_hint==&file_hint_mov &&
 	memcmp(buffer,  jpg_header_app0_jfif11_null, sizeof(jpg_header_app0_jfif11_null))==0)
       return 0;
+    if(buffer[3]==0xdb)	/* DQT */
+      return 0;
+    if(buffer[3]==0xc4)	/* DHT - needed to recover .cr2 */
+      return 0;
+    if(buffer[3]==0xe0 && (buffer[6]!='J' || buffer[7]!='F'))	/* Should be JFIF/JFXX */
+      return 0;
+    if(buffer[3]==0xe1 && (buffer[6]!='E' || buffer[7]!='x' || buffer[8]!='i'|| buffer[9]!='f'))	/* Should be EXIF */
+      return 0;
+    if(buffer[3]==0xfe && (!isprint(buffer[6]) || !isprint(buffer[7])))
+      return 0;
   }
-  if(buffer[0]==0xff && buffer[1]==0xd8)
+  while(i+4<buffer_size && buffer[i]==0xff && is_marker_valid(buffer[i+1]))
   {
-    int mpo=0;
-    unsigned int i=2;
-    time_t jpg_time=0;
-    while(i+4<buffer_size && buffer[i]==0xff)
+    if(buffer[i+1]==0xff)
+      i++;
+    else
     {
-      /* 0x00 */
-      /* 0x01 */
-      /* 0xc4 Define Huffman table */
-      /* 0xc0 SOF0 Start of Frame */
-      /* 0xcc DAC arithmetic-coding conditioning*/
-      /* 0xcf SOF15 */
-      /* 0xd0-d7 JPEG_RST0 .. JPEG_RST7 markers */
-      /* 0xd8 SOI Start of Image */
-      /* 0xd9 EOI End of Image */
-      /* 0xda SOS: Start Of Scan */
-      /* 0xdb DQT: Define Quantization Table */
-      /* 0xdc DNL: Define Number of Lines */
-      /* 0xdd DRI: define restart interval */
-      /* 0xde DHP: define hierarchical progression */
-      /* 0xe0 APP0 */
-      /* 0xef APP15 */
-      /* 0xfe COM */
       if(buffer[i+1]==0xe1)
       { /* APP1 Exif information */
 	if(i+0x0A < buffer_size && 2+(buffer[i+2]<<8)+buffer[i+3] > 0x0A)
@@ -333,62 +450,28 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
 	  jpg_time=get_date_from_tiff_header((const TIFFHeader*)&buffer[i+0x0A], tiff_size);
 	}
       }
-      else if(buffer[i+1]==0xe2)
+      else if(buffer[i+1]==0xc4)
       {
-	/* APP2 Exif information */
-	if(i+8 < buffer_size &&
-	    buffer[i+4]=='M' && buffer[i+5]=='P' && buffer[i+6]=='F' && buffer[i+7]==0)
-	{
-	  /* Multi-picture format */
-	  mpo=1;
-	}
-      }
-      else if((buffer[i+1]>=0xe0 && buffer[i+1]<=0xef) ||
-	 buffer[i+1]==0xfe ||
-	 buffer[i+1]==0xdb)
-      {
-      }
-      else
-      {
-	reset_file_recovery(file_recovery_new);
-	file_recovery_new->min_filesize=(i>288?i:288);
-	file_recovery_new->calculated_file_size=2;
-	file_recovery_new->time=jpg_time;
-	if(mpo > 0)
-	{
-	  file_recovery_new->extension="jpg";
-	  file_recovery_new->file_check=&file_check_mpo;
-	}
-	else
-	{
-	  file_recovery_new->extension=file_hint_jpg.extension;
-	  file_recovery_new->file_check=&file_check_jpg;
-	  file_recovery_new->data_check=&data_check_jpg;
-	}
-	return 1;
+	/* DHT */
+	if(jpg_check_dht(buffer, buffer_size, i, 2+(buffer[i+2]<<8)+buffer[i+3])!=0)
+	  return 0;
       }
       i+=2+(buffer[i+2]<<8)+buffer[i+3];
     }
-    if(i >= 6*512)
-    {
-      reset_file_recovery(file_recovery_new);
-      file_recovery_new->min_filesize=i;
-      file_recovery_new->calculated_file_size=2;
-      file_recovery_new->time=jpg_time;
-      if(mpo > 0)
-      {
-	file_recovery_new->extension="mpo";
-      }
-      else
-      {
-	file_recovery_new->extension=file_hint_jpg.extension;
-	file_recovery_new->file_check=&file_check_jpg;
-	file_recovery_new->data_check=&data_check_jpg;
-      }
-      return 1;
-    }
   }
-  return 0;
+  if(i+1 < file_recovery->blocksize && buffer[i+1]!=0xda)
+    return 0;
+  if(i+1 < 512 && buffer[i+1]!=0xda)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->min_filesize=i;
+  file_recovery_new->calculated_file_size=0;
+  file_recovery_new->time=jpg_time;
+  file_recovery_new->extension=file_hint_jpg.extension;
+  file_recovery_new->file_check=&file_check_jpg;
+  if(buffer_size >= 4)
+    file_recovery_new->data_check=&data_check_jpg;
+  return 1;
 }
 
 #if defined(HAVE_LIBJPEG) && defined(HAVE_JPEGLIB_H)
@@ -511,7 +594,7 @@ static void jpg_init_source (j_decompress_ptr cinfo)
  * the front of the buffer rather than discarding it.
  */
 
-static int jpg_fill_input_buffer (j_decompress_ptr cinfo)
+static boolean jpg_fill_input_buffer (j_decompress_ptr cinfo)
 {
   my_source_mgr * src = (my_source_mgr *) cinfo->src;
   size_t nbytes;
@@ -694,7 +777,7 @@ static inline int jpeg_session_resume(struct jpeg_session_struct *jpeg_session)
   if(resume_memory((j_common_ptr)&jpeg_session->cinfo))
     return -1;
   src = (my_source_mgr *) jpeg_session->cinfo.src;
-  if(fseek(jpeg_session->handle, jpeg_session->offset + src->file_size, SEEK_SET) < 0)
+  if(my_fseek(jpeg_session->handle, jpeg_session->offset + src->file_size, SEEK_SET) < 0)
     return -1;
   return 0;
 }
@@ -707,8 +790,11 @@ static inline void jpeg_session_suspend(struct jpeg_session_struct *jpeg_session
 
 static void jpeg_session_start(struct jpeg_session_struct *jpeg_session)
 {
-  fseek(jpeg_session->handle, jpeg_session->offset, SEEK_SET);
-  jpeg_create_decompress(&jpeg_session->cinfo);
+  if(my_fseek(jpeg_session->handle, jpeg_session->offset, SEEK_SET) < 0)
+  {
+    log_critical("jpeg_session_start: fseek failed.\n");
+  }
+  jpeg_create_decompress(&jpeg_session->cinfo); 
   jpeg_testdisk_src(&jpeg_session->cinfo, jpeg_session->handle, jpeg_session->offset, jpeg_session->blocksize);
   (void) jpeg_read_header(&jpeg_session->cinfo, TRUE);
   jpeg_session->cinfo.two_pass_quantize = FALSE;
@@ -730,8 +816,8 @@ static uint64_t jpg_xy_to_offset(FILE *infile, const unsigned int x, const unsig
   static struct my_error_mgr jerr;
   static uint64_t file_size_max;
   static struct jpeg_session_struct jpeg_session;
-  unsigned int checkpoint_status=0;
-  int avoid_leak=0;
+  unsigned int checkpoint_status;
+  int avoid_leak;
   jpeg_init_session(&jpeg_session);
   jpeg_session.handle=infile;
   jpeg_session.offset=offset;
@@ -764,6 +850,8 @@ static uint64_t jpg_xy_to_offset(FILE *infile, const unsigned int x, const unsig
     }
     file_size_max+=blocksize;
   }
+  checkpoint_status=0;
+  avoid_leak=0;
   while(file_size_max<offset_rel2)
   {
     if(checkpoint_status==0 || jpeg_session_resume(&jpeg_session)<0)
@@ -1131,7 +1219,8 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
     src->file_size_max=file_recovery->file_size;
   }
   /* Image is very big, skip some tests */
-  if(jpeg_session.output_height * jpeg_session.row_stride > 500 * 1024 * 1024)
+  if((uint64_t)jpeg_session.output_height * jpeg_session.row_stride > 500 * 1024 * 1024 ||
+      jpeg_session.output_height<9)
     jpeg_session.flags=0;
   /* 0x100/2=0x80, medium value */
   if(jpeg_session.flags==0)
@@ -1177,6 +1266,7 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
     file_recovery->file_size=file_recovery->calculated_file_size;
   else
   {
+    static const unsigned char jpg_footer[2]= { 0xff,0xd9};
     file_recovery->file_size=jpeg_size;
     file_search_footer(file_recovery, jpg_footer, sizeof(jpg_footer), 0);
   }
@@ -1185,34 +1275,63 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
 
 static int jpg_check_dht(const unsigned char *buffer, const unsigned int buffer_size, const unsigned i, const unsigned int size)
 {
+  unsigned int j=i+4;
   /* DHT must not be shorter than 18 bytes, 1+16+1 */
   /* DHT should not be longer than 1088 bytes, 4*(1+16+255) */
   if(size<18)
     return 2;
-  if(i + 4 < buffer_size)
+  while(j < buffer_size && j < i+size)
   {
-    unsigned int j=i+4;
+    const unsigned int tc=buffer[j]>>4;
     const unsigned int n=buffer[j] & 0x0f;
+    unsigned int l;
+    unsigned int sum=0;
+    /* Table class: 0 = DC table or lossless table, 1 = AC table */
+    if(tc > 1)
+      return 2;
     /* Must be between 0 and 3 Huffman table */
     if(n > 3)
       return 2;
-    {
-      unsigned int l;
-      unsigned int sum=0;
-      if(j < buffer_size && (buffer[j]&0x0f)>3)
-	return 2;
-      j++;
-      for(l=0; l < 16; l++)
-	if(j < buffer_size)
-	  sum+=buffer[j+l];
-      if(sum>255)
-	return 2;
-      j+=16;
-      j+=sum;
-    }
-    if(j > i+size)
+    j++;
+    for(l=0; l < 16; l++)
+      if(j < buffer_size)
+	sum+=buffer[j+l];
+    if(sum>255)
       return 2;
+    j+=16;
+    j+=sum;
   }
+  if(j > i+size)
+    return 2;
+  return 0;
+}
+
+struct sof_header
+{
+  uint16_t      marker;
+  uint16_t      length;         /* 8 + 3 * nbr */
+  unsigned char precision;      /* 2-16 8 for SOF0 */
+  uint16_t      height;         /* 0-65535 */
+  uint16_t      width;          /* 1-65535 */
+  unsigned char nbr;            /* 1-255 */
+  unsigned char data[0];
+} __attribute__ ((gcc_struct, __packed__));
+
+static int jpg_check_sof0(const unsigned char *buffer, const unsigned int buffer_size, const unsigned i)
+{
+  const struct sof_header *h=(const struct sof_header *)&buffer[i];
+  if(i+4 > buffer_size)
+    return 0;
+  if(be16(h->length) < sizeof(struct sof_header)-2)
+    return 1;
+  if(i+2+8 > buffer_size)
+    return 0;
+  if(h->precision!=8 || be16(h->width)==0 || h->nbr==0)
+    return 1;
+  if(be16(h->length) < 8+h->nbr*3)
+    return 1;
+//  if(i+2+be16(h->length) > buffer_size)
+//    return 0;
   return 0;
 }
 
@@ -1220,14 +1339,14 @@ static void jpg_search_marker(file_recovery_t *file_recovery)
 {
   FILE* infile=file_recovery->handle;
   unsigned char buffer[40*8192];
-  int nbytes;
+  size_t nbytes;
   uint64_t offset;
   unsigned int i;
   if(file_recovery->blocksize==0)
     return ;
   offset=file_recovery->offset_error / file_recovery->blocksize * file_recovery->blocksize;
   i=file_recovery->offset_error % file_recovery->blocksize;
-  if(fseek(infile, offset, SEEK_SET) < 0)
+  if(my_fseek(infile, offset, SEEK_SET) < 0)
     return ;
   do
   {
@@ -1267,9 +1386,9 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
   FILE* infile=file_recovery->handle;
   unsigned char buffer[40*8192];
   uint64_t thumb_offset=0;
-  int nbytes;
+  size_t nbytes;
   file_recovery->extra=0;
-  if(fseek(infile, 0, SEEK_SET) < 0)
+  if(my_fseek(infile, 0, SEEK_SET) < 0)
     return 0;
   if((nbytes=fread(&buffer, 1, sizeof(buffer), infile))>0)
   {
@@ -1325,6 +1444,8 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 	  const char *ifbytecount=NULL;
 	  if(nbytes - (i+0x0A) < tiff_size)
 	    tiff_size=nbytes - (i+0x0A);
+	  if(file_recovery->time==0)
+	    file_recovery->time=get_date_from_tiff_header(tiff, tiff_size);
 	  thumb_data=find_tag_from_tiff_header(tiff, tiff_size, TIFFTAG_JPEGIFOFFSET, &potential_error);
 	  if(thumb_data!=NULL)
 	  {
@@ -1414,7 +1535,7 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 #ifdef DEBUG_JPEG
 		j_old=j;
 #endif
-		j+=2+(buffer[j+2]<<8)+buffer[j+3];
+		j+=2U+(buffer[j+2]<<8)+buffer[j+3];
 	      }
 	      if(thumb_sos_found>0 && extract_thumb>0
 		  && offset < nbytes && buffer[offset]==0xff)
@@ -1553,7 +1674,7 @@ static void file_check_jpg(file_recovery_t *file_recovery)
 #endif
 }
 
-static int data_check_jpg2(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_jpg2(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
 #if 0
   unsigned int old_marker=0;
@@ -1564,15 +1685,6 @@ static int data_check_jpg2(const unsigned char *buffer, const unsigned int buffe
     file_recovery->data_check=&data_check_jpg;
     return data_check_jpg(buffer, buffer_size, file_recovery);
   }
-  if(buffer[buffer_size/2]==0xff && buffer[buffer_size/2]==0xd8 && 
-      file_recovery->calculated_file_size != file_recovery->file_size)
-  {
-#ifdef DEBUG_JPEG
-    log_info("%s data_check_jpg2 0xffd8 at 0x%llx\n", file_recovery->filename, 
-	(long long unsigned)file_recovery->calculated_file_size);
-#endif
-    return 2;
-  }
   while(file_recovery->calculated_file_size + buffer_size/2  > file_recovery->file_size &&
       file_recovery->calculated_file_size < file_recovery->file_size + buffer_size/2)
   {
@@ -1583,7 +1695,7 @@ static int data_check_jpg2(const unsigned char *buffer, const unsigned int buffe
       {
 	/* JPEG_EOI */
 	file_recovery->calculated_file_size++;
-	return 2;
+	return DC_STOP;
       }
       else if(buffer[i] >= 0xd0 && buffer[i] <= 0xd7)
       {
@@ -1596,7 +1708,7 @@ static int data_check_jpg2(const unsigned char *buffer, const unsigned int buffe
 	  log_info("Rejected due to JPEG_RST marker\n");
 #endif
 	  file_recovery->calculated_file_size++;
-	  return 2;
+	  return DC_STOP;
 	}
 	/* TODO: store old_marker in file_recovery */
 	old_marker=buffer[i];
@@ -1609,22 +1721,16 @@ static int data_check_jpg2(const unsigned char *buffer, const unsigned int buffe
 	    (long long unsigned)file_recovery->calculated_file_size);
 #endif
 	file_recovery->offset_error=file_recovery->calculated_file_size;
-	return 2;
+	return DC_STOP;
       }
     }
     file_recovery->calculated_file_size++;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-int data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+data_check_t data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
-  if(buffer[buffer_size/2]==0xff && buffer[buffer_size/2]==0xd8 && 
-      file_recovery->calculated_file_size != file_recovery->file_size)
-  {
-    log_info("data_check_jpg ffd8\n");
-    return 2;
-  }
   /* Skip the SOI */
   if(file_recovery->calculated_file_size==0)
     file_recovery->calculated_file_size+=2;
@@ -1644,16 +1750,61 @@ int data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size,
 	  (long long unsigned)file_recovery->calculated_file_size+2+size);
 #endif
       file_recovery->calculated_file_size+=2+size;
-      if(buffer[i+1]==0xc4)	/* DHT */
+      if(buffer[i+1]==0xc0)	/* SOF0 */
+      {
+	if(jpg_check_sof0(buffer, buffer_size, i)!=0)
+	  return DC_STOP;
+      }
+      else if(buffer[i+1]==0xc4)	/* DHT */
       {
 	if(jpg_check_dht(buffer, buffer_size, i, 2+size)!=0)
-	  return 2;
+	  return DC_STOP;
       }
-      if(buffer[i+1]==0xda)	/* SOS: Start Of Scan */
+      else if(buffer[i+1]==0xda)	/* SOS: Start Of Scan */
       {
 	file_recovery->data_check=&data_check_jpg2;
 	return data_check_jpg2(buffer, buffer_size, file_recovery);
       }
+      else if(buffer[i+1]==0xe2)	/* APP2 Exif information */
+      {
+	if(i+8 < buffer_size &&
+	    buffer[i+4]=='M' && buffer[i+5]=='P' && buffer[i+6]=='F' && buffer[i+7]==0)
+	{
+	  unsigned int size_test=size;
+	  if(i + 2 + size_test >= buffer_size)
+	  {
+	    size_test=buffer_size-i-2;
+	  }
+	  if(i>=buffer_size/2)
+	  {
+	    file_recovery->calculated_file_size-=2+size;
+	    return DC_CONTINUE;
+	  }
+	  if(size>12)
+	  {
+	    const uint64_t offset=file_recovery->calculated_file_size-(2+size)+8;
+	    const unsigned char *mpo=buffer+i+8;
+	    uint64_t calculated_file_size=0;
+	    size_test-=8;
+	    if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
+	    {
+	      calculated_file_size=check_mpo_le(mpo, offset, size-8);
+	    }
+	    else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
+	    {
+	      calculated_file_size=check_mpo_be(mpo, offset, size-8);
+	    }
+	    if(calculated_file_size > 0)
+	    {
+	      /* Multi-picture format */
+	      file_recovery->calculated_file_size=calculated_file_size;
+	      file_recovery->data_check=&data_check_size;
+	      file_recovery->file_check=&file_check_mpo;
+	      return DC_CONTINUE;
+	    }
+	  }
+	}
+      }
     }
     else
     {
@@ -1661,10 +1812,10 @@ int data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size,
       log_info("data_check_jpg %02x at %llu\n", buffer[i],
 	  (long long unsigned)file_recovery->calculated_file_size);
 #endif
-      return 2;
+      return DC_STOP;
     }
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
 const char*td_jpeg_version(void)
