@@ -28,10 +28,10 @@
 #include "filegen.h"
 
 extern const file_hint_t file_hint_1cd;
+extern const file_hint_t file_hint_3dm;
 extern const file_hint_t file_hint_7z;
 extern const file_hint_t file_hint_DB;
 extern const file_hint_t file_hint_a;
-extern const file_hint_t file_hint_abcdp;
 extern const file_hint_t file_hint_abr;
 extern const file_hint_t file_hint_acb;
 extern const file_hint_t file_hint_accdb;
@@ -47,6 +47,7 @@ extern const file_hint_t file_hint_amr;
 extern const file_hint_t file_hint_apa;
 extern const file_hint_t file_hint_ape;
 extern const file_hint_t file_hint_apple;
+extern const file_hint_t file_hint_ari;
 extern const file_hint_t file_hint_arj;
 extern const file_hint_t file_hint_asf;
 extern const file_hint_t file_hint_asl;
@@ -55,6 +56,7 @@ extern const file_hint_t file_hint_atd;
 extern const file_hint_t file_hint_au;
 extern const file_hint_t file_hint_axx;
 extern const file_hint_t file_hint_bac;
+extern const file_hint_t file_hint_berkeley;
 extern const file_hint_t file_hint_bim;
 extern const file_hint_t file_hint_binvox;
 extern const file_hint_t file_hint_bkf;
@@ -76,10 +78,12 @@ extern const file_hint_t file_hint_csh;
 extern const file_hint_t file_hint_ctg;
 extern const file_hint_t file_hint_cwk;
 extern const file_hint_t file_hint_d2s;
+extern const file_hint_t file_hint_dad;
 extern const file_hint_t file_hint_dar;
 extern const file_hint_t file_hint_dat;
 extern const file_hint_t file_hint_dbf;
 extern const file_hint_t file_hint_dbn;
+extern const file_hint_t file_hint_dcm;
 extern const file_hint_t file_hint_ddf;
 extern const file_hint_t file_hint_dex;
 extern const file_hint_t file_hint_dim;
@@ -136,6 +140,7 @@ extern const file_hint_t file_hint_gif;
 extern const file_hint_t file_hint_gm6;
 extern const file_hint_t file_hint_gp5;
 extern const file_hint_t file_hint_gpg;
+extern const file_hint_t file_hint_gsm;
 extern const file_hint_t file_hint_gz;
 extern const file_hint_t file_hint_hdf;
 extern const file_hint_t file_hint_hds;
@@ -144,6 +149,7 @@ extern const file_hint_t file_hint_hr9;
 extern const file_hint_t file_hint_http;
 extern const file_hint_t file_hint_icc;
 extern const file_hint_t file_hint_ico;
+extern const file_hint_t file_hint_idx;
 extern const file_hint_t file_hint_ifo;
 extern const file_hint_t file_hint_imb;
 extern const file_hint_t file_hint_indd;
@@ -159,6 +165,7 @@ extern const file_hint_t file_hint_lit;
 extern const file_hint_t file_hint_logic;
 extern const file_hint_t file_hint_lnk;
 extern const file_hint_t file_hint_lso;
+extern const file_hint_t file_hint_luks;
 extern const file_hint_t file_hint_lxo;
 extern const file_hint_t file_hint_lzh;
 extern const file_hint_t file_hint_lzo;
@@ -176,6 +183,7 @@ extern const file_hint_t file_hint_mid;
 extern const file_hint_t file_hint_mig;
 extern const file_hint_t file_hint_mk5;
 extern const file_hint_t file_hint_mkv;
+extern const file_hint_t file_hint_mlv;
 extern const file_hint_t file_hint_mobi;
 extern const file_hint_t file_hint_mov;
 extern const file_hint_t file_hint_mp3;
@@ -203,6 +211,7 @@ extern const file_hint_t file_hint_pcx;
 extern const file_hint_t file_hint_pdf;
 extern const file_hint_t file_hint_pds;
 extern const file_hint_t file_hint_pfx;
+extern const file_hint_t file_hint_plist;
 extern const file_hint_t file_hint_plt;
 extern const file_hint_t file_hint_png;
 extern const file_hint_t file_hint_pnm;
@@ -233,6 +242,7 @@ extern const file_hint_t file_hint_reg;
 extern const file_hint_t file_hint_res;
 extern const file_hint_t file_hint_rfp;
 extern const file_hint_t file_hint_riff;
+extern const file_hint_t file_hint_rlv;
 extern const file_hint_t file_hint_rm;
 extern const file_hint_t file_hint_rns;
 extern const file_hint_t file_hint_rpm;
@@ -240,6 +250,7 @@ extern const file_hint_t file_hint_rw2;
 extern const file_hint_t file_hint_rx2;
 extern const file_hint_t file_hint_save;
 extern const file_hint_t file_hint_ses;
+extern const file_hint_t file_hint_sgcta;
 extern const file_hint_t file_hint_shn;
 extern const file_hint_t file_hint_sib;
 extern const file_hint_t file_hint_sig;
@@ -281,6 +292,7 @@ extern const file_hint_t file_hint_win;
 extern const file_hint_t file_hint_wks;
 extern const file_hint_t file_hint_wmf;
 extern const file_hint_t file_hint_wnk;
+extern const file_hint_t file_hint_woff;
 extern const file_hint_t file_hint_wpb;
 extern const file_hint_t file_hint_wpd;
 extern const file_hint_t file_hint_wtv;
@@ -288,21 +300,23 @@ extern const file_hint_t file_hint_wv;
 extern const file_hint_t file_hint_x3f;
 extern const file_hint_t file_hint_xcf;
 extern const file_hint_t file_hint_xfi;
+extern const file_hint_t file_hint_xfs;
 extern const file_hint_t file_hint_xm;
 extern const file_hint_t file_hint_xsv;
 extern const file_hint_t file_hint_xpt;
 extern const file_hint_t file_hint_xv;
 extern const file_hint_t file_hint_xz;
+extern const file_hint_t file_hint_z2d;
 extern const file_hint_t file_hint_zip;
 
 file_enable_t list_file_enable[]=
 {
   { .enable=0, .file_hint=&file_hint_sig  },
   { .enable=0, .file_hint=&file_hint_1cd  },
+  { .enable=0, .file_hint=&file_hint_3dm  },
   { .enable=0, .file_hint=&file_hint_7z   },
   { .enable=0, .file_hint=&file_hint_DB    },
   { .enable=0, .file_hint=&file_hint_a    },
-  { .enable=0, .file_hint=&file_hint_abcdp},
   { .enable=0, .file_hint=&file_hint_abr  },
   { .enable=0, .file_hint=&file_hint_acb  },
   { .enable=0, .file_hint=&file_hint_accdb},
@@ -318,6 +332,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_apa  },
   { .enable=0, .file_hint=&file_hint_ape  },
   { .enable=0, .file_hint=&file_hint_apple },
+  { .enable=0, .file_hint=&file_hint_ari  },
   { .enable=0, .file_hint=&file_hint_arj  },
   { .enable=0, .file_hint=&file_hint_asf  },
   { .enable=0, .file_hint=&file_hint_asl  },
@@ -326,6 +341,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_au   },
   { .enable=0, .file_hint=&file_hint_axx  },
   { .enable=0, .file_hint=&file_hint_bac  },
+  { .enable=0, .file_hint=&file_hint_berkeley },
   { .enable=0, .file_hint=&file_hint_bim  },
   { .enable=0, .file_hint=&file_hint_binvox  },
   { .enable=0, .file_hint=&file_hint_bkf  },
@@ -347,10 +363,12 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_ctg  },
   { .enable=0, .file_hint=&file_hint_cwk  },
   { .enable=0, .file_hint=&file_hint_d2s  },
+  { .enable=0, .file_hint=&file_hint_dad  },
   { .enable=0, .file_hint=&file_hint_dar  },
   { .enable=0, .file_hint=&file_hint_dat  },
   { .enable=0, .file_hint=&file_hint_dbf  },
   { .enable=0, .file_hint=&file_hint_dbn  },
+  { .enable=0, .file_hint=&file_hint_dcm  },
   { .enable=0, .file_hint=&file_hint_ddf  },
   { .enable=0, .file_hint=&file_hint_dex  },
   { .enable=0, .file_hint=&file_hint_dim  },
@@ -406,6 +424,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_gm6  },
   { .enable=0, .file_hint=&file_hint_gp5  },
   { .enable=0, .file_hint=&file_hint_gpg  },
+  { .enable=0, .file_hint=&file_hint_gsm  },
   { .enable=0, .file_hint=&file_hint_gz   },
   { .enable=0, .file_hint=&file_hint_hdf  },
   { .enable=0, .file_hint=&file_hint_hds  },
@@ -414,6 +433,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_http },
   { .enable=0, .file_hint=&file_hint_icc  },
   { .enable=0, .file_hint=&file_hint_ico  },
+  { .enable=0, .file_hint=&file_hint_idx  },
   { .enable=0, .file_hint=&file_hint_ifo  },
   { .enable=0, .file_hint=&file_hint_imb  },
   { .enable=0, .file_hint=&file_hint_indd  },
@@ -429,6 +449,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_logic},
   { .enable=0, .file_hint=&file_hint_lnk  },
   { .enable=0, .file_hint=&file_hint_lso  },
+  { .enable=0, .file_hint=&file_hint_luks },
   { .enable=0, .file_hint=&file_hint_lxo  },
   { .enable=0, .file_hint=&file_hint_lzh  },
   { .enable=0, .file_hint=&file_hint_lzo  },
@@ -446,6 +467,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_mig  },
   { .enable=0, .file_hint=&file_hint_mk5  },
   { .enable=0, .file_hint=&file_hint_mkv  },
+  { .enable=0, .file_hint=&file_hint_mlv  },
   { .enable=0, .file_hint=&file_hint_mobi },
   { .enable=0, .file_hint=&file_hint_mov  },
   { .enable=0, .file_hint=&file_hint_mp3  },
@@ -473,6 +495,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_pdf  },
   { .enable=0, .file_hint=&file_hint_pds  },
   { .enable=0, .file_hint=&file_hint_pfx  },
+  { .enable=0, .file_hint=&file_hint_plist  },
   { .enable=0, .file_hint=&file_hint_plt  },
   { .enable=0, .file_hint=&file_hint_png  },
   { .enable=0, .file_hint=&file_hint_pnm  },
@@ -503,6 +526,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_res  },
   { .enable=0, .file_hint=&file_hint_rfp  },
   { .enable=0, .file_hint=&file_hint_riff },
+  { .enable=0, .file_hint=&file_hint_rlv  },
   { .enable=0, .file_hint=&file_hint_rm   },
   { .enable=0, .file_hint=&file_hint_rns  },
   { .enable=0, .file_hint=&file_hint_rpm  },
@@ -510,6 +534,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_rx2  },
   { .enable=0, .file_hint=&file_hint_save  },
   { .enable=0, .file_hint=&file_hint_ses  },
+  { .enable=0, .file_hint=&file_hint_sgcta  },
   { .enable=0, .file_hint=&file_hint_shn  },
   { .enable=0, .file_hint=&file_hint_sib  },
   { .enable=0, .file_hint=&file_hint_sit  },
@@ -551,6 +576,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_wks  },
   { .enable=0, .file_hint=&file_hint_wmf  },
   { .enable=0, .file_hint=&file_hint_wnk  },
+  { .enable=0, .file_hint=&file_hint_woff  },
   { .enable=0, .file_hint=&file_hint_wpb  },
   { .enable=0, .file_hint=&file_hint_wpd  },
   { .enable=0, .file_hint=&file_hint_wtv  },
@@ -558,11 +584,13 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_x3f  },
   { .enable=0, .file_hint=&file_hint_xcf  },
   { .enable=0, .file_hint=&file_hint_xfi  },
+  { .enable=0, .file_hint=&file_hint_xfs  },
   { .enable=0, .file_hint=&file_hint_xm   },
   { .enable=0, .file_hint=&file_hint_xsv  },
   { .enable=0, .file_hint=&file_hint_xpt  },
   { .enable=0, .file_hint=&file_hint_xv   },
   { .enable=0, .file_hint=&file_hint_xz   },
+  { .enable=0, .file_hint=&file_hint_z2d  },
   { .enable=0, .file_hint=&file_hint_zip  },
   { .enable=0, .file_hint=NULL }
 };
