@@ -32,7 +32,7 @@
 #include "log.h"
 
 static void register_header_check_ogg(file_stat_t *file_stat);
-static int data_check_ogg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t data_check_ogg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_ogg= {
   .extension="ogg",
@@ -53,14 +53,14 @@ static int header_check_ogg(const unsigned char *buffer, const unsigned int buff
   /* http://en.wikipedia.org/wiki/Ogg#File_format */
   /* Return if not Beginning Of Stream and already saving the file */
   if((buffer[5]&0x02)!=0x02 &&
-      file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_ogg &&
-      (file_recovery->blocksize < (27+255)/2 ||
+      (file_recovery->blocksize < 27+255 ||
        file_recovery->calculated_file_size == file_recovery->file_size))
     return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->calculated_file_size=0;
-  if(file_recovery_new->blocksize > (27+255)/2)
+  if(file_recovery_new->blocksize > 27+255)
   {
     file_recovery_new->data_check=&data_check_ogg;
     file_recovery_new->file_check=&file_check_size;
@@ -68,13 +68,15 @@ static int header_check_ogg(const unsigned char *buffer, const unsigned int buff
   /* Ogg data, Theora video */
   if(memcmp(&buffer[28], sign_theora, sizeof(sign_theora))==0)
     file_recovery_new->extension="ogm";
+  else if(memcmp(&buffer[0x78], sign_theora, sizeof(sign_theora))==0)
+    file_recovery_new->extension="ogv";
   else
     file_recovery_new->extension=file_hint_ogg.extension;
   return 1;
 }
 
 /* http://www.ietf.org/rfc/rfc3533.txt */
-static int data_check_ogg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_ogg(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 27 +255 < file_recovery->file_size + buffer_size/2)
@@ -91,7 +93,7 @@ static int data_check_ogg(const unsigned char *buffer, const unsigned int buffer
         page_size+=buffer[i+27+j];
       if(page_size<27)
       {
-        return 2;
+        return DC_STOP;
       }
       /* By definition, page_size<=27+255+255*255=65307 */
       file_recovery->calculated_file_size+=page_size;
@@ -101,10 +103,10 @@ static int data_check_ogg(const unsigned char *buffer, const unsigned int buffer
     }
     else
     {
-      return 2;
+      return DC_STOP;
     }
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
 static void register_header_check_ogg(file_stat_t *file_stat)
