@@ -32,7 +32,6 @@
 
 
 static void register_header_check_pap(file_stat_t *file_stat);
-static int header_check_pap(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_pap= {
   .extension="pap",
@@ -44,20 +43,15 @@ const file_hint_t file_hint_pap= {
   .register_header_check=&register_header_check_pap
 };
 
-static const unsigned char pap_header[]  = {'P','A','P','1'};
+static int header_check_pap(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+{
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_pap.extension;
+  return 1;
+}
 
 static void register_header_check_pap(file_stat_t *file_stat)
 {
+  static const unsigned char pap_header[]  = {'P','A','P','1'};
   register_header_check(0, pap_header,sizeof(pap_header), &header_check_pap, file_stat);
 }
-
-static int header_check_pap(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
-{
-  if(memcmp(buffer,pap_header,sizeof(pap_header))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_pap.extension;
-    return 1;
-  }
-  return 0;
-}
