@@ -32,6 +32,12 @@
 #include "common.h"
 #include "log.h"
 
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
 static void register_header_check_par2(file_stat_t *file_stat);
 
 const file_hint_t file_hint_par2= {
@@ -48,7 +54,7 @@ static const unsigned char par2_header[8]=  {
   'P' , 'A' , 'R' , '2' , 0x00, 'P' , 'K' , 'T' 
 };
 
-static int data_check_par2(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_par2(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 16 < file_recovery->file_size + buffer_size/2)
@@ -56,12 +62,12 @@ static int data_check_par2(const unsigned char *buffer, const unsigned int buffe
     const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
     const uint64_t length=le64((*(const uint64_t *)(&buffer[i+8])));
     if(memcmp(&buffer[i], &par2_header, sizeof(par2_header))!=0)
-      return 2;
+      return DC_STOP;
     if(length % 4 !=0 || length < 16)
-      return 2;
+      return DC_STOP;
     file_recovery->calculated_file_size+=length;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
 static void file_rename_par2(const char *old_filename)
@@ -73,10 +79,10 @@ static void file_rename_par2(const char *old_filename)
   while(1)
   {
     uint64_t length;
-    int buffer_size;
+    size_t buffer_size;
     unsigned char buffer[4096];
     const uint64_t *lengthp=(const uint64_t *)&buffer[8];
-    if(fseek(file, offset, SEEK_SET)<0)
+    if(my_fseek(file, offset, SEEK_SET)<0)
     {
       fclose(file);
       return;
@@ -108,14 +114,20 @@ static void file_rename_par2(const char *old_filename)
 
 static int header_check_par2(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL &&
-      file_recovery->data_check==&data_check_par2)
+  const uint64_t length=le64((*(const uint64_t *)(&buffer[8])));
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_par2)
+    return 0;
+  if(length % 4 !=0 || length < 16)
     return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->extension=file_hint_par2.extension;
+  file_recovery_new->file_rename=&file_rename_par2;
+  file_recovery_new->min_filesize=64+length;
+  if(file_recovery_new->blocksize < 16)
+    return 1;
   file_recovery_new->data_check=&data_check_par2;
   file_recovery_new->file_check=&file_check_size;
-  file_recovery_new->file_rename=&file_rename_par2;
   return 1;
 }
 
