@@ -36,8 +36,7 @@
 #endif
 
 static void register_header_check_psd(file_stat_t *file_stat);
-static int header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static int psd_skip_color_mode(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+static data_check_t psd_skip_color_mode(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 static void file_check_psd(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_psd= {
@@ -50,28 +49,45 @@ const file_hint_t file_hint_psd= {
   .register_header_check=&register_header_check_psd
 };
 
-static const unsigned char psd_header[6]={'8', 'B', 'P', 'S', 0x00, 0x01};
 static uint64_t psd_image_data_size_max=0;
 
-static void register_header_check_psd(file_stat_t *file_stat)
+struct psd_file_header
 {
-  register_header_check(0, psd_header,sizeof(psd_header), &header_check_psd, file_stat);
-}
+  char signature[4];
+  uint16_t version;	/* must be 1 */
+  char reserved[6];	/* must be 0 */
+  uint16_t channels;	/* between 1 and 56 */
+  uint32_t height;	/* max of 30,000 */
+  uint32_t width;	/* max of 30,000 */
+  uint16_t depth;	/* 1, 8, 16 or 32 */
+  uint16_t color_mode;	/* Bitmap = 0; Grayscale = 1; Indexed = 2; RGB = 3; CMYK = 4; Multichannel = 7; Duotone = 8; Lab = 9 */
+} __attribute__ ((gcc_struct, __packed__));
 
 static int header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,psd_header,sizeof(psd_header))==0)
-  {
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->min_filesize=70;
-    file_recovery_new->extension=file_hint_psd.extension;
-    /* File header */
-    file_recovery_new->calculated_file_size=0x1a;
-    file_recovery_new->data_check=&psd_skip_color_mode;
-    file_recovery_new->file_check=&file_check_psd;
+  const struct psd_file_header *hdr=(const struct psd_file_header *)buffer;
+#ifdef DEBUG_PSD
+  log_info("channels %u\n", be16(hdr->channels));
+  log_info("height %u\n", be32(hdr->height));
+  log_info("width  %u\n", be32(hdr->width));
+  log_info("depth  %u\n", be16(hdr->depth));
+  log_info("color_mode %u\n", be16(hdr->color_mode));
+#endif
+  if(be16(hdr->channels)==0 || be16(hdr->channels)>56 ||
+      be32(hdr->height)==0 || be32(hdr->height)>30000 ||
+      be32(hdr->width)==0 || be32(hdr->width)>30000 ||
+      be16(hdr->depth)==0 || (be16(hdr->depth)!=1 && be16(hdr->depth)%8!=0))
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->min_filesize=70;
+  file_recovery_new->extension=file_hint_psd.extension;
+  if(file_recovery_new->blocksize < 16)
     return 1;
-  }
-  return 0;
+  /* File header */
+  file_recovery_new->calculated_file_size=0x1a;
+  file_recovery_new->data_check=&psd_skip_color_mode;
+  file_recovery_new->file_check=&file_check_psd;
+  return 1;
 }
 
 static uint32_t get_be32(const void *buffer, const unsigned int offset)
@@ -80,13 +96,13 @@ static uint32_t get_be32(const void *buffer, const unsigned int offset)
   return be32(*val);
 }
 
-static int psd_skip_image_data(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t psd_skip_image_data(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   file_recovery->file_check=NULL;
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int psd_skip_layer_info(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t psd_skip_layer_info(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 16 < file_recovery->file_size + buffer_size/2)
@@ -97,15 +113,15 @@ static int psd_skip_layer_info(const unsigned char *buffer, const unsigned int b
     log_info("Image data at 0x%lx\n", (long unsigned)(l + file_recovery->calculated_file_size));
 #endif
     if(l<4)
-      return 2;
+      return DC_STOP;
     file_recovery->calculated_file_size+=l;
     file_recovery->data_check=&psd_skip_image_data;
     return psd_skip_image_data(buffer, buffer_size, file_recovery);
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int psd_skip_image_resources(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t psd_skip_image_resources(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 16 < file_recovery->file_size + buffer_size/2)
@@ -116,20 +132,18 @@ static int psd_skip_image_resources(const unsigned char *buffer, const unsigned
     log_info("Layer info at 0x%lx\n", (long unsigned)(l + file_recovery->calculated_file_size));
 #endif
     if(l<4)
-      return 2;
+      return DC_STOP;
     file_recovery->calculated_file_size+=l;
     file_recovery->data_check=&psd_skip_layer_info;
     return psd_skip_layer_info(buffer, buffer_size, file_recovery);
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int psd_skip_color_mode(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t psd_skip_color_mode(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
-  psd_image_data_size_max=(buffer[buffer_size/2+12]<<8 | buffer[buffer_size/2+13]) *
-    (buffer[buffer_size/2+14]<<24 | buffer[buffer_size/2+15] <<16 | buffer[buffer_size/2+16]<<8 | buffer[buffer_size/2+17]) *
-    (buffer[buffer_size/2+18]<<24 | buffer[buffer_size/2+19] <<16 | buffer[buffer_size/2+20]<<8 | buffer[buffer_size/2+21]) *
-    buffer[buffer_size/2+23] / 8;
+  const struct psd_file_header *psd=(const struct psd_file_header *)&buffer[buffer_size/2];
+  psd_image_data_size_max=(uint64_t)be16(psd->channels) * be32(psd->height) * be32(psd->width) * be16(psd->depth) / 8;
 #ifdef DEBUG_PSD
   log_info("psd_image_data_size_max %lu\n", (long unsigned)psd_image_data_size_max);
 #endif
@@ -142,12 +156,12 @@ static int psd_skip_color_mode(const unsigned char *buffer, const unsigned int b
     log_info("Color mode at 0x%lx\n", (long unsigned)(l + file_recovery->calculated_file_size));
 #endif
     if(l<4)
-      return 2;
+      return DC_STOP;
     file_recovery->calculated_file_size+=l;
     file_recovery->data_check=&psd_skip_image_resources;
     return psd_skip_image_resources(buffer, buffer_size, file_recovery);
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
 static void file_check_psd(file_recovery_t *file_recovery)
@@ -157,3 +171,9 @@ static void file_check_psd(file_recovery_t *file_recovery)
   else if(file_recovery->file_size > file_recovery->calculated_file_size + psd_image_data_size_max)
     file_recovery->file_size=file_recovery->calculated_file_size + psd_image_data_size_max;
 }
+
+static void register_header_check_psd(file_stat_t *file_stat)
+{
+  static const unsigned char psd_header[6]={'8', 'B', 'P', 'S', 0x00, 0x01};
+  register_header_check(0, psd_header,sizeof(psd_header), &header_check_psd, file_stat);
+}
