@@ -34,7 +34,13 @@
 #include "log.h"
 #endif
 
-int data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
+data_check_t data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 static void register_header_check_riff(file_stat_t *file_stat);
 
 const file_hint_t file_hint_riff= {
@@ -64,7 +70,7 @@ typedef struct {
 static void log_riff_list(const uint64_t offset, const unsigned int depth, const riff_list_header *list_header)
 {
   unsigned int i;
-  log_info("0x%08lx - 0x%08lx ", offset, offset + 8 + le32(list_header->dwSize) - 1);
+  log_info("0x%08lx - 0x%08lx ", offset, offset + 8 - 1 + le32(list_header->dwSize));
   for(i = 0; i < depth; i++)
     log_info(" ");
   log_info("%c%c%c%c %c%c%c%c 0x%x\n",
@@ -84,7 +90,7 @@ static void log_riff_chunk(const uint64_t offset, const unsigned int depth, cons
   unsigned int i;
   if(le32(list_header->dwSize)==0)
     return ;
-  log_info("0x%08lx - 0x%08lx ", offset, offset + 8 + le32(list_header->dwSize) - 1);
+  log_info("0x%08lx - 0x%08lx ", offset, offset + 8 - 1 + le32(list_header->dwSize));
   for(i = 0; i < depth; i++)
     log_info(" ");
   log_info("%c%c%c%c 0x%x\n",
@@ -104,7 +110,7 @@ static void check_riff_list(file_recovery_t *fr, const unsigned int depth, const
     return;
   for(file_size=start; file_size < end;)
   {
-    if(fseek(fr->handle, file_size, SEEK_SET)<0)
+    if(my_fseek(fr->handle, file_size, SEEK_SET)<0)
     {
       fr->offset_error=file_size;
       return;
@@ -119,7 +125,7 @@ static void check_riff_list(file_recovery_t *fr, const unsigned int depth, const
 #ifdef DEBUG_RIFF
       log_riff_list(file_size, depth, &list_header);
 #endif
-      check_riff_list(fr, depth+1, file_size + sizeof(list_header), file_size + 8 + le32(list_header.dwSize) - 1);
+      check_riff_list(fr, depth+1, file_size + sizeof(list_header), file_size + 8 - 1 + le32(list_header.dwSize));
     }
     else
     {
@@ -143,7 +149,7 @@ static void file_check_avi(file_recovery_t *fr)
   {
     const uint64_t file_size=fr->file_size;
     riff_list_header list_header;
-    if(fseek(fr->handle, fr->file_size, SEEK_SET)<0)
+    if(my_fseek(fr->handle, fr->file_size, SEEK_SET)<0)
     {
       fr->file_size=0;
       return ;
@@ -161,7 +167,7 @@ static void file_check_avi(file_recovery_t *fr)
       fr->offset_error=fr->file_size;
       return;
     }
-    check_riff_list(fr, 1, file_size + sizeof(list_header), file_size + 8 + le32(list_header.dwSize) - 1);
+    check_riff_list(fr, 1, file_size + sizeof(list_header), file_size + 8 - 1 + le32(list_header.dwSize));
     if(fr->offset_error > 0)
     {
       fr->file_size=0;
@@ -171,7 +177,7 @@ static void file_check_avi(file_recovery_t *fr)
   }
 }
 
-static int data_check_avi(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_avi(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 12 < file_recovery->file_size + buffer_size/2)
@@ -181,12 +187,12 @@ static int data_check_avi(const unsigned char *buffer, const unsigned int buffer
     if(memcmp(&buffer[i], "RIFF", 4)==0 && memcmp(&buffer[i+8], "AVIX", 4)==0)
       file_recovery->calculated_file_size += 8 + le32(chunk_header->dwSize);
     else
-      return 2;
+      return DC_STOP;
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
-int data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+data_check_t data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 8 < file_recovery->file_size + buffer_size/2)
@@ -194,10 +200,18 @@ int data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer
     const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
     const riff_chunk_header *chunk_header=(const riff_chunk_header*)&buffer[i];
     if(buffer[i+2]!='d' || buffer[i+3]!='b')	/* Video Data Binary ?*/
-      return 2;
+    {
+#ifdef DEBUG_RIFF
+      log_info("data_check_avi_stream stop\n");
+#endif
+      return DC_STOP;
+    }
     file_recovery->calculated_file_size += 8 + le32(chunk_header->dwSize);
+#ifdef DEBUG_RIFF
+    log_info("data_check_avi_stream %llu\n", (long long unsigned)file_recovery->calculated_file_size);
+#endif
   }
-  return 1;
+  return DC_CONTINUE;
 }
 
 static void file_check_size_rifx(file_recovery_t *file_recovery)
@@ -208,32 +222,40 @@ static void file_check_size_rifx(file_recovery_t *file_recovery)
 
 static int header_check_riff(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  uint64_t size;
   if(!( buffer[8]>='A' && buffer[8]<='Z' &&
 	buffer[9]>='A' && buffer[9]<='Z' &&
 	buffer[10]>='A' && buffer[10]<='Z' &&
 	((buffer[11]>='A' && buffer[11]<='Z') || buffer[11]==' ' ||
 	 (buffer[11]>='0' && buffer[11]<='9'))))
     return 0;
-  reset_file_recovery(file_recovery_new);
-  file_recovery_new->file_check=&file_check_size;
-  file_recovery_new->data_check=&data_check_size;
-  file_recovery_new->calculated_file_size=(uint64_t)buffer[4]+(((uint64_t)buffer[5])<<8)+(((uint64_t)buffer[6])<<16)+(((uint64_t)buffer[7])<<24);
   if(memcmp(&buffer[8],"NUND",4)==0)
   {
     /* Cubase Project File */
+    reset_file_recovery(file_recovery_new);
     file_recovery_new->extension="cpr";
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->data_check=&data_check_size;
     file_recovery_new->calculated_file_size=(((uint64_t)buffer[4])<<24) +
       (((uint64_t)buffer[5])<<16) + (((uint64_t)buffer[6])<<8) +
       (uint64_t)buffer[7] + 12;
     return 1;
   }
+  size=(uint64_t)buffer[4]+(((uint64_t)buffer[5])<<8)+(((uint64_t)buffer[6])<<16)+(((uint64_t)buffer[7])<<24);
+
   /* Windows Animated Cursor */
-  else if(memcmp(&buffer[8],"ACON",4)==0)
+  if(memcmp(&buffer[8],"ACON",4)==0)
   {
+    if(size < 12)
+      return 0;
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->data_check=&data_check_size;
+    file_recovery_new->calculated_file_size=size;
     file_recovery_new->extension="ani";
     return 1;
   }
-  file_recovery_new->calculated_file_size+=8;
+  size+=8;
   if(memcmp(&buffer[8],"AVI ",4)==0)
   {
     const riff_list_header list_movi={
@@ -241,22 +263,35 @@ static int header_check_riff(const unsigned char *buffer, const unsigned int buf
       .dwSize=le32(4),
       .dwFourCC=be32(0x6d6f7669)	/* movi */
     };
+    reset_file_recovery(file_recovery_new);
     file_recovery_new->extension="avi";
     /* Is it a raw avi stream with Data Binary chunks ? */
-    if(file_recovery_new->calculated_file_size + 4 < buffer_size &&
-	memcmp(&buffer[file_recovery_new->calculated_file_size - sizeof(list_movi)], &list_movi, sizeof(list_movi)) ==0 &&
-	buffer[file_recovery_new->calculated_file_size+2]=='d' &&
-	buffer[file_recovery_new->calculated_file_size+3]=='b')
+    if(size + 4 < buffer_size &&
+	memcmp(&buffer[size - sizeof(list_movi)], &list_movi, sizeof(list_movi)) ==0 &&
+	buffer[size+2]=='d' &&
+	buffer[size+3]=='b')
     {
+      if(file_recovery_new->blocksize < 8)
+	return 1;
       file_recovery_new->data_check=&data_check_avi_stream;
+      file_recovery_new->file_check=&file_check_size_lax;
     }
     else
     {
+      if(file_recovery_new->blocksize < 12)
+	return 1;
       file_recovery_new->data_check=&data_check_avi;
       file_recovery_new->file_check=&file_check_avi;
     }
+    file_recovery_new->calculated_file_size=size;
     return 1;
   }
+  if(size < 12)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->calculated_file_size=size;
+  file_recovery_new->file_check=&file_check_size;
+  file_recovery_new->data_check=&data_check_size;
   if(memcmp(&buffer[8],"CDDA",4)==0)
     file_recovery_new->extension="cda";
   else if(memcmp(&buffer[8],"CDR",3)==0 || memcmp(&buffer[8],"cdr6",4)==0)
