@@ -39,8 +39,9 @@
 #include "file_tiff.h"
 #include "log.h"
 
+extern const file_hint_t file_hint_raf;
+
 static void register_header_check_tiff(file_stat_t *file_stat);
-static int header_check_tiff(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_diroff, const unsigned int depth, const unsigned int count, const char *tag_make);
 static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_diroff, const unsigned int depth, const unsigned int count, const char *tag_make);
 
@@ -54,9 +55,6 @@ const file_hint_t file_hint_tiff= {
   .register_header_check=&register_header_check_tiff
 };
 
-static const unsigned char tiff_header_be[4]= { 'M','M',0x00, 0x2a};
-static const unsigned char tiff_header_le[4]= { 'I','I',0x2a, 0x00};
-
 static unsigned int type2size(const unsigned int type)
 {
   switch(type)
@@ -144,17 +142,23 @@ static const char *find_tag_from_tiff_header_be(const TIFFHeader *tiff, const un
   const uint32_t *tiff_next_diroff;
   const TIFFDirEntry *ifd;
   unsigned int j;
+  if(tiff_size < sizeof(TIFFHeader))
+    return NULL;
+  if(tiff_size < be32(tiff->tiff_diroff)+sizeof(TIFFDirEntry))
+    return NULL;
   /* Bound checking */
   if((const char*)ifd0 < (const char*)tiff ||
       (const char*)(ifd0+1) > (const char*)tiff + tiff_size)
     return NULL;
   for(j=0, ifd=&ifd0->ifd;
-      (const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<be16(ifd0->nbr_fields);
+      (const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<(uint16_t)be16(ifd0->nbr_fields);
       j++, ifd++)
   {
     if(be16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+    {
       *potential_error = (const char*)&ifd->tdir_type+1;
-    if(be16(ifd->tdir_tag)==tag)
+    }
+    if((uint16_t)be16(ifd->tdir_tag)==tag)
       return (const char*)tiff+be32(ifd->tdir_offset);
     else if(be16(ifd->tdir_tag)==TIFFTAG_EXIFIFD)	/* Exif IFD Pointer */
       exififd=(const struct ifd_header *)((const char*)tiff + be32(ifd->tdir_offset));
@@ -164,13 +168,16 @@ static const char *find_tag_from_tiff_header_be(const TIFFHeader *tiff, const un
       (const char*)exififd > (const char*)tiff &&
       (const char*)(exififd+1) <= (const char*)tiff + tiff_size)
   {	/* Exif */
+    const unsigned int nbr_fields=be16(exififd->nbr_fields);
     for(j=0, ifd=&exififd->ifd;
-	(const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<be16(exififd->nbr_fields);
+	(const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<nbr_fields;
 	j++, ifd++)
     {
       if(be16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+      {
 	*potential_error = (const char*)&ifd->tdir_type+1;
-      if(be16(ifd->tdir_tag)==tag)
+      }
+      if((uint16_t)be16(ifd->tdir_tag)==tag)
 	return (const char*)tiff+be32(ifd->tdir_offset);
     }
   }
@@ -178,16 +185,20 @@ static const char *find_tag_from_tiff_header_be(const TIFFHeader *tiff, const un
   if(be32(*tiff_next_diroff)>0)
   {
     const struct ifd_header *ifd1=(const struct ifd_header*)((const char *)tiff+be32(*tiff_next_diroff));
+    unsigned int nbr_fields;
     if((const char*)ifd1 <= (const char*)tiff ||
 	(const char*)(ifd1+1) > (const char*)tiff+tiff_size)
       return NULL;
+    nbr_fields=be16(ifd1->nbr_fields);
     for(j=0, ifd=&ifd1->ifd;
-	(const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<be16(ifd1->nbr_fields);
+	(const char*)(ifd+1) <= (const char*)tiff+tiff_size && j<nbr_fields;
 	j++, ifd++)
     {
       if(be16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+      {
 	*potential_error = (const char*)&ifd->tdir_type+1;
-      if(be16(ifd->tdir_tag)==tag)
+      }
+      if((uint16_t)be16(ifd->tdir_tag)==tag)
 	return (const char*)tiff+be32(ifd->tdir_offset);
     }
   }
@@ -201,6 +212,10 @@ static const char *find_tag_from_tiff_header_le(const TIFFHeader *tiff, const un
   const uint32_t *tiff_next_diroff;
   const TIFFDirEntry *ifd;
   unsigned int j;
+  if(tiff_size < sizeof(TIFFHeader))
+    return NULL;
+  if(tiff_size < le32(tiff->tiff_diroff)+sizeof(TIFFDirEntry))
+    return NULL;
   /* Bound checking */
   if((const char*)ifd0 < (const char*)tiff ||
       (const char*)(ifd0+1) > (const char*)tiff + tiff_size)
@@ -210,7 +225,9 @@ static const char *find_tag_from_tiff_header_le(const TIFFHeader *tiff, const un
       j++, ifd++)
   {
     if(le16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+    {
       *potential_error = (const char*)&ifd->tdir_type+1;
+    }
     if(le16(ifd->tdir_tag)==tag)
       return (const char*)tiff+le32(ifd->tdir_offset);
     else if(le16(ifd->tdir_tag)==TIFFTAG_EXIFIFD)	/* Exif IFD Pointer */
@@ -226,7 +243,9 @@ static const char *find_tag_from_tiff_header_le(const TIFFHeader *tiff, const un
 	j++, ifd++)
     {
       if(le16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+      {
 	*potential_error = (const char*)&ifd->tdir_type+1;
+      }
       if(le16(ifd->tdir_tag)==tag)		/* DateTimeOriginal */
 	return (const char*)tiff+le32(ifd->tdir_offset);
     }
@@ -244,7 +263,9 @@ static const char *find_tag_from_tiff_header_le(const TIFFHeader *tiff, const un
 	j++, ifd++)
     {
       if(le16(ifd->tdir_type) > 18 && (*potential_error==NULL || *potential_error > (const char*)&ifd->tdir_type+1))
+      {
 	*potential_error = (const char*)&ifd->tdir_type+1;
+      }
       if(le16(ifd->tdir_tag)==tag)
 	return (const char*)tiff+le32(ifd->tdir_offset);
     }
@@ -254,20 +275,10 @@ static const char *find_tag_from_tiff_header_le(const TIFFHeader *tiff, const un
 
 const char *find_tag_from_tiff_header(const TIFFHeader *tiff, const unsigned int tiff_size, const unsigned int tag, const char **potential_error)
 {
-  if(tiff_size < sizeof(TIFFHeader))
-    return NULL;
   if(tiff->tiff_magic==TIFF_BIGENDIAN)
-  {
-    if(tiff_size < be32(tiff->tiff_diroff)+sizeof(TIFFDirEntry))
-      return NULL;
     return find_tag_from_tiff_header_be(tiff, tiff_size, tag, potential_error);
-  }
   else if(tiff->tiff_magic==TIFF_LITTLEENDIAN)
-  {
-    if(tiff_size < le32(tiff->tiff_diroff)+sizeof(TIFFDirEntry))
-      return NULL;
     return find_tag_from_tiff_header_le(tiff, tiff_size, tag, potential_error);
-  }
   return NULL;
 }
 
@@ -299,52 +310,76 @@ time_t get_date_from_tiff_header(const TIFFHeader *tiff, const unsigned int tiff
   return mktime(&tm_time);
 }
 
-static void register_header_check_tiff(file_stat_t *file_stat)
+static int header_check_tiff_be_new(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  register_header_check(0, tiff_header_be, sizeof(tiff_header_be), &header_check_tiff, file_stat);
-  register_header_check(0, tiff_header_le, sizeof(tiff_header_le), &header_check_tiff, file_stat);
+  const char *potential_error=NULL;
+  const char *tag_make;
+  const TIFFHeader *header=(const TIFFHeader *)buffer;
+  if((uint32_t)be32(header->tiff_diroff) < sizeof(TIFFHeader))
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_tiff.extension;
+  tag_make=find_tag_from_tiff_header_be(header, buffer_size, TIFFTAG_MAKE, &potential_error);
+  if(tag_make!=NULL && tag_make >= (const char *)buffer && tag_make < (const char *)buffer + buffer_size - 20)
+  {
+    if(strcmp(tag_make, "PENTAX Corporation ")==0 ||
+	strcmp(tag_make, "PENTAX             ")==0)
+      file_recovery_new->extension="pef";
+    else if(strcmp(tag_make, "NIKON CORPORATION")==0)
+      file_recovery_new->extension="nef";
+    else if(strcmp(tag_make, "Kodak")==0)
+      file_recovery_new->extension="dcr";
+  }
+  file_recovery_new->time=get_date_from_tiff_header(header, buffer_size);
+  file_recovery_new->file_check=&file_check_tiff;
+  return 1;
 }
 
-static int header_check_tiff(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
+static int header_check_tiff_le_new(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(memcmp(buffer,tiff_header_be,sizeof(tiff_header_be))==0 ||
-      memcmp(buffer,tiff_header_le,sizeof(tiff_header_le))==0)
+  const char raf_fp[15]={0x49, 0x49, 0x2a, 0x00, 0x08, 0x00, 0x00, 0x00,  0x01, 0x00, 0x00, 0xf0, 0x0d, 0x00, 0x01};
+  const char *potential_error=NULL;
+  const TIFFHeader *header=(const TIFFHeader *)buffer;
+  if((uint32_t)le32(header->tiff_diroff) < sizeof(TIFFHeader))
+    return 0;
+  /* Avoid a false positiv with some RAF files */
+  if(file_recovery->file_stat!=NULL &&
+    file_recovery->file_stat->file_hint==&file_hint_raf &&
+    memcmp(buffer, raf_fp, 15)==0)
+    return 0;
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->extension=file_hint_tiff.extension;
+  /* Canon RAW */
+  if(buffer[8]=='C' && buffer[9]=='R' && buffer[10]==2)
+    file_recovery_new->extension="cr2";
+  else if(find_tag_from_tiff_header_le(header, buffer_size, TIFFTAG_DNGVERSION, &potential_error)!=NULL)
   {
-    const char *potential_error=NULL;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_tiff.extension;
-    /* Canon RAW */
-    if(buffer[8]=='C' && buffer[9]=='R' && buffer[10]==2)
-      file_recovery_new->extension="cr2";
-    else if(find_tag_from_tiff_header((const TIFFHeader *)buffer, buffer_size, TIFFTAG_DNGVERSION, &potential_error)!=NULL)
-    {
-      /* Adobe Digital Negative */
-      file_recovery_new->extension="dng";
-    }
-    else
-    {
-      const char *tag_make;
-      tag_make=find_tag_from_tiff_header((const TIFFHeader *)buffer, buffer_size, TIFFTAG_MAKE, &potential_error);
-      if(tag_make!=NULL && tag_make >= (const char *)buffer && tag_make < (const char *)buffer + buffer_size - 20)
-      {
-	if(strcmp(tag_make, "PENTAX Corporation ")==0 ||
-	    strcmp(tag_make, "PENTAX             ")==0)
-	  file_recovery_new->extension="pef";
-	else if(strcmp(tag_make, "NIKON CORPORATION")==0)
-	  file_recovery_new->extension="nef";
-	else if(strcmp(tag_make, "Kodak")==0)
-	  file_recovery_new->extension="dcr";
-	else if(strcmp(tag_make, "SONY")==0)
-	  file_recovery_new->extension="sr2";
-	else if(strncmp(tag_make, "SONY ",5)==0)
-	  file_recovery_new->extension="arw";
-      }
-    }
-    file_recovery_new->time=get_date_from_tiff_header((const TIFFHeader *)buffer, buffer_size);
-    file_recovery_new->file_check=&file_check_tiff;
-    return 1;
+    /* Adobe Digital Negative */
+    file_recovery_new->extension="dng";
   }
-  return 0;
+  else
+  {
+    const char *tag_make;
+    tag_make=find_tag_from_tiff_header_le(header, buffer_size, TIFFTAG_MAKE, &potential_error);
+    if(tag_make!=NULL && tag_make >= (const char *)buffer && tag_make < (const char *)buffer + buffer_size - 20)
+    {
+      if(strcmp(tag_make, "SONY")==0)
+	file_recovery_new->extension="sr2";
+      else if(strncmp(tag_make, "SONY ",5)==0)
+	file_recovery_new->extension="arw";
+    }
+  }
+  file_recovery_new->time=get_date_from_tiff_header(header, buffer_size);
+  file_recovery_new->file_check=&file_check_tiff;
+  return 1;
+}
+
+static void register_header_check_tiff(file_stat_t *file_stat)
+{
+  static const unsigned char tiff_header_be[4]= { 'M','M',0x00, 0x2a};
+  static const unsigned char tiff_header_le[4]= { 'I','I',0x2a, 0x00};
+  register_header_check(0, tiff_header_be, sizeof(tiff_header_be), &header_check_tiff_be_new, file_stat);
+  register_header_check(0, tiff_header_le, sizeof(tiff_header_le), &header_check_tiff_le_new, file_stat);
 }
 
 static unsigned int tiff_le_read(const void *val, const unsigned int type)
@@ -377,7 +412,7 @@ static unsigned int tiff_be_read(const void *val, const unsigned int type)
   }
 }
 
-static uint64_t parse_strip_le(FILE *handle, TIFFDirEntry *entry_strip_offsets, TIFFDirEntry *entry_strip_bytecounts)
+static uint64_t parse_strip_le(FILE *handle, const TIFFDirEntry *entry_strip_offsets, const TIFFDirEntry *entry_strip_bytecounts)
 {
   const unsigned int nbr=(le32(entry_strip_offsets->tdir_count)<2048?
       le32(entry_strip_offsets->tdir_count):
@@ -387,17 +422,17 @@ static uint64_t parse_strip_le(FILE *handle, TIFFDirEntry *entry_strip_offsets,
   uint32_t *sizep;
   uint64_t max_offset=0;
   if(le32(entry_strip_offsets->tdir_count) != le32(entry_strip_bytecounts->tdir_count))
-    return 0;
+    return -1;
   if(le32(entry_strip_offsets->tdir_count)==0 ||
       le16(entry_strip_offsets->tdir_type)!=4 ||
       le16(entry_strip_bytecounts->tdir_type)!=4)
-    return 0;
+    return -1;
   offsetp=(uint32_t *)MALLOC(nbr*sizeof(*offsetp));
   if(fseek(handle, le32(entry_strip_offsets->tdir_offset), SEEK_SET) < 0 ||
       fread(offsetp, sizeof(*offsetp), nbr, handle) != nbr)
   {
     free(offsetp);
-    return 0;
+    return -1;
   }
   sizep=(uint32_t *)MALLOC(nbr*sizeof(*sizep));
   if(fseek(handle, le32(entry_strip_bytecounts->tdir_offset), SEEK_SET) < 0 ||
@@ -405,7 +440,7 @@ static uint64_t parse_strip_le(FILE *handle, TIFFDirEntry *entry_strip_offsets,
   {
     free(offsetp);
     free(sizep);
-    return 0;
+    return -1;
   }
   for(i=0; i<nbr; i++)
   {
@@ -418,7 +453,7 @@ static uint64_t parse_strip_le(FILE *handle, TIFFDirEntry *entry_strip_offsets,
   return max_offset;
 }
 
-static uint64_t parse_strip_be(FILE *handle, TIFFDirEntry *entry_strip_offsets, TIFFDirEntry *entry_strip_bytecounts)
+static uint64_t parse_strip_be(FILE *handle, const TIFFDirEntry *entry_strip_offsets, const TIFFDirEntry *entry_strip_bytecounts)
 {
   const unsigned int nbr=(be32(entry_strip_offsets->tdir_count)<2048?
       be32(entry_strip_offsets->tdir_count):
@@ -428,17 +463,17 @@ static uint64_t parse_strip_be(FILE *handle, TIFFDirEntry *entry_strip_offsets,
   uint32_t *sizep;
   uint64_t max_offset=0;
   if(be32(entry_strip_offsets->tdir_count) != be32(entry_strip_bytecounts->tdir_count))
-    return 0;
+    return -1;
   if(be32(entry_strip_offsets->tdir_count)==0 ||
       be16(entry_strip_offsets->tdir_type)!=4 ||
       be16(entry_strip_bytecounts->tdir_type)!=4)
-    return 0;
+    return -1;
   offsetp=(uint32_t *)MALLOC(nbr*sizeof(*offsetp));
   if(fseek(handle, be32(entry_strip_offsets->tdir_offset), SEEK_SET) < 0 ||
       fread(offsetp, sizeof(*offsetp), nbr, handle) != nbr)
   {
     free(offsetp);
-    return 0;
+    return -1;
   }
   sizep=(uint32_t *)MALLOC(nbr*sizeof(*sizep));
   if(fseek(handle, be32(entry_strip_bytecounts->tdir_offset), SEEK_SET) < 0 ||
@@ -446,7 +481,7 @@ static uint64_t parse_strip_be(FILE *handle, TIFFDirEntry *entry_strip_offsets,
   {
     free(offsetp);
     free(sizep);
-    return 0;
+    return -1;
   }
   for(i=0; i<nbr; i++)
   {
@@ -480,13 +515,13 @@ static uint64_t tiff_le_makernote(FILE *in, const uint32_t tiff_diroff)
   uint64_t tile_offsets=0;
   uint64_t tile_bytecounts=0;
   const TIFFDirEntry *entry;
-  if(tiff_diroff == 0)
-    return 0;
+  if(tiff_diroff < sizeof(TIFFHeader))
+    return -1;
   if(fseek(in, tiff_diroff, SEEK_SET) < 0)
-    return 0;
+    return -1;
   data_read=fread(buffer, 1, sizeof(buffer), in);
   if(data_read<2)
-    return 0;
+    return -1;
   if( memcmp(buffer, sign_nikon1, sizeof(sign_nikon1))==0 ||
       memcmp(buffer, sign_nikon2, sizeof(sign_nikon2))==0 ||
       memcmp(buffer, sign_pentax, sizeof(sign_pentax))==0 )
@@ -500,7 +535,7 @@ static uint64_t tiff_le_makernote(FILE *in, const uint32_t tiff_diroff)
   if(n > (unsigned)(data_read-2)/12)
     n=(data_read-2)/12;
   if(n==0)
-    return 0;
+    return -1;
   for(i=0;i<n;i++)
   {
     const uint64_t val=(uint64_t)le32(entry->tdir_count)*type2size(le16(entry->tdir_type));
@@ -519,7 +554,7 @@ static uint64_t tiff_le_makernote(FILE *in, const uint32_t tiff_diroff)
     {
       const uint64_t new_offset=le32(entry->tdir_offset)+val;
       if(new_offset==0)
-	return 0;
+	return -1;
       if(max_offset < new_offset)
 	max_offset=new_offset;
     }
@@ -575,25 +610,25 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
   uint64_t strip_bytecounts=0;
   uint64_t tile_offsets=0;
   uint64_t tile_bytecounts=0;
-  TIFFDirEntry *entry=(TIFFDirEntry *)&buffer[2];
-  TIFFDirEntry *entry_strip_offsets=NULL;
-  TIFFDirEntry *entry_strip_bytecounts=NULL;
-  TIFFDirEntry *entry_tile_offsets=NULL;
-  TIFFDirEntry *entry_tile_bytecounts=NULL;
+  const TIFFDirEntry *entry=(const TIFFDirEntry *)&buffer[2];
+  const TIFFDirEntry *entry_strip_offsets=NULL;
+  const TIFFDirEntry *entry_strip_bytecounts=NULL;
+  const TIFFDirEntry *entry_tile_offsets=NULL;
+  const TIFFDirEntry *entry_tile_bytecounts=NULL;
 #ifdef DEBUG_TIFF
   log_info("header_check_tiff_le(fr, %lu, %u, %u)\n", (long unsigned)tiff_diroff, depth, count);
 #endif
   if(depth>4)
-    return 0;
+    return -1;
   if(count>16)
-    return 0;
-  if(tiff_diroff == 0)
-    return 0;
+    return -1;
+  if(tiff_diroff < sizeof(TIFFHeader))
+    return -1;
   if(fseek(fr->handle, tiff_diroff, SEEK_SET) < 0)
-    return 0;
+    return -1;
   data_read=fread(buffer, 1, sizeof(buffer), fr->handle);
   if(data_read<2)
-    return 0;
+    return -1;
   n=buffer[0]+(buffer[1]<<8);
 #ifdef DEBUG_TIFF
   log_info("header_check_tiff_le(fr, %lu, %u, %u) => %u entries\n", (long unsigned)tiff_diroff, depth, count, n);
@@ -602,7 +637,7 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
   if(n > (unsigned)(data_read-2)/12)
     n=(data_read-2)/12;
   if(n==0)
-    return 0;
+    return -1;
   for(i=0;i<n;i++)
   {
     const uint64_t val=(uint64_t)le32(entry->tdir_count)*type2size(le16(entry->tdir_type));
@@ -622,7 +657,7 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
     {
       const uint64_t new_offset=le32(entry->tdir_offset)+val;
       if(new_offset==0)
-	return 0;
+	return -1;
       if(max_offset < new_offset)
 	max_offset=new_offset;
     }
@@ -645,8 +680,8 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
 	case TIFFTAG_KODAKIFD:
 	  {
 	    const uint64_t new_offset=header_check_tiff_le(fr, tmp, depth+1, 0, tag_make);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -664,8 +699,8 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
 #endif
 	  {
 	    const uint64_t new_offset=header_check_tiff_le(fr, tmp, depth+1, 0, tag_make);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -674,8 +709,8 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
 	case EXIFTAG_MAKERNOTE:
 	  {
 	    const uint64_t new_offset=tiff_le_makernote(fr->handle, tmp);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -697,21 +732,21 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
 	    uint32_t *subifd_offsetp;
 	    if(fseek(fr->handle, le32(entry->tdir_offset), SEEK_SET) < 0)
 	    {
-	      return 0;
+	      return -1;
 	    }
 	    subifd_offsetp=(uint32_t *)MALLOC(nbr*sizeof(*subifd_offsetp));
 	    if(fread(subifd_offsetp, sizeof(*subifd_offsetp), nbr, fr->handle) != nbr)
 	    {
 	      free(subifd_offsetp);
-	      return 0;
+	      return -1;
 	    }
 	    for(j=0; j<nbr; j++)
 	    {
 	      const uint64_t new_offset=header_check_tiff_le(fr, le32(subifd_offsetp[j]), depth+1, 0, tag_make);
-	      if(new_offset==0)
+	      if(new_offset==-1)
 	      {
 		free(subifd_offsetp);
-		return 0;
+		return -1;
 	      }
 	      if(max_offset < new_offset)
 		max_offset = new_offset;
@@ -750,12 +785,16 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
   if(entry_strip_offsets != NULL && entry_strip_bytecounts != NULL)
   {
     const uint64_t tmp=parse_strip_le(fr->handle, entry_strip_offsets, entry_strip_bytecounts);
+    if(tmp==-1)
+      return -1;
     if(max_offset < tmp)
       max_offset=tmp;
   }
   if(entry_tile_offsets != NULL && entry_tile_bytecounts != NULL)
   {
     const uint64_t tmp=parse_strip_le(fr->handle, entry_tile_offsets, entry_tile_bytecounts);
+    if(tmp==-1)
+      return -1;
     if(max_offset < tmp)
       max_offset=tmp;
   }
@@ -763,7 +802,7 @@ static uint64_t header_check_tiff_le(file_recovery_t *fr, const uint32_t tiff_di
   if(le32(*tiff_next_diroff) > 0)
   {
     const uint64_t new_offset=header_check_tiff_le(fr, le32(*tiff_next_diroff), depth, count+1, tag_make);
-    if(max_offset < new_offset)
+    if(new_offset != -1 && max_offset < new_offset)
       max_offset=new_offset;
   }
   return max_offset;
@@ -790,13 +829,13 @@ static uint64_t tiff_be_makernote(FILE *in, const uint32_t tiff_diroff)
   uint64_t tile_offsets=0;
   uint64_t tile_bytecounts=0;
   const TIFFDirEntry *entry;
-  if(tiff_diroff == 0)
-    return 0;
+  if(tiff_diroff < sizeof(TIFFHeader))
+    return -1;
   if(fseek(in, tiff_diroff, SEEK_SET) < 0)
-    return 0;
+    return -1;
   data_read=fread(buffer, 1, sizeof(buffer), in);
   if(data_read<2)
-    return 0;
+    return -1;
   if( memcmp(buffer, sign_nikon1, sizeof(sign_nikon1))==0 ||
       memcmp(buffer, sign_nikon2, sizeof(sign_nikon2))==0 ||
       memcmp(buffer, sign_pentax, sizeof(sign_pentax))==0 )
@@ -810,7 +849,7 @@ static uint64_t tiff_be_makernote(FILE *in, const uint32_t tiff_diroff)
   if(n > (unsigned)(data_read-2)/12)
     n=(data_read-2)/12;
   if(n==0)
-    return 0;
+    return -1;
   for(i=0;i<n;i++)
   {
     const uint64_t val=(uint64_t)be32(entry->tdir_count)*type2size(be16(entry->tdir_type));
@@ -829,7 +868,7 @@ static uint64_t tiff_be_makernote(FILE *in, const uint32_t tiff_diroff)
     {
       const uint64_t new_offset=be32(entry->tdir_offset)+val;
       if(new_offset==0)
-	return 0;
+	return -1;
       if(max_offset < new_offset)
 	max_offset=new_offset;
     }
@@ -885,22 +924,22 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
   uint64_t strip_bytecounts=0;
   uint64_t tile_offsets=0;
   uint64_t tile_bytecounts=0;
-  TIFFDirEntry *entry=(TIFFDirEntry *)&buffer[2];
-  TIFFDirEntry *entry_strip_offsets=NULL;
-  TIFFDirEntry *entry_strip_bytecounts=NULL;
-  TIFFDirEntry *entry_tile_offsets=NULL;
-  TIFFDirEntry *entry_tile_bytecounts=NULL;
+  const TIFFDirEntry *entry=(const TIFFDirEntry *)&buffer[2];
+  const TIFFDirEntry *entry_strip_offsets=NULL;
+  const TIFFDirEntry *entry_strip_bytecounts=NULL;
+  const TIFFDirEntry *entry_tile_offsets=NULL;
+  const TIFFDirEntry *entry_tile_bytecounts=NULL;
   if(depth>4)
-    return 0;
+    return -1;
   if(count>16)
-    return 0;
-  if(tiff_diroff == 0)
-    return 0;
+    return -1;
+  if(tiff_diroff < sizeof(TIFFHeader))
+    return -1;
   if(fseek(fr->handle, tiff_diroff, SEEK_SET) < 0)
-    return 0;
+    return -1;
   data_read=fread(buffer, 1, sizeof(buffer), fr->handle);
   if(data_read<2)
-    return 0;
+    return -1;
   n=(buffer[0]<<8)+buffer[1];
 #ifdef DEBUG_TIFF
   log_info("header_check_tiff_be(fr, %lu, %u, %u) => %u entries\n", (long unsigned)tiff_diroff, depth, count, n);
@@ -909,7 +948,7 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
   if(n > (unsigned)(data_read-2)/12)
     n=(data_read-2)/12;
   if(n==0)
-    return 0;
+    return -1;
   for(i=0;i<n;i++)
   {
     const uint64_t val=(uint64_t)be32(entry->tdir_count)*type2size(be16(entry->tdir_type));
@@ -928,7 +967,7 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
     {
       const uint64_t new_offset=be32(entry->tdir_offset)+val;
       if(new_offset==0)
-	return 0;
+	return -1;
       if(max_offset < new_offset)
 	max_offset=new_offset;
     }
@@ -951,8 +990,8 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
 	case TIFFTAG_KODAKIFD:
 	  {
 	    const uint64_t new_offset=header_check_tiff_be(fr, tmp, depth+1, 0, tag_make);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -970,8 +1009,8 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
 #endif
 	  {
 	    const uint64_t new_offset=header_check_tiff_be(fr, tmp, depth+1, 0, tag_make);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -980,8 +1019,8 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
 	case EXIFTAG_MAKERNOTE:
 	  {
 	    const uint64_t new_offset=tiff_be_makernote(fr->handle, tmp);
-	    if(new_offset==0)
-	      return 0;
+	    if(new_offset==-1)
+	      return -1;
 	    if(max_offset < new_offset)
 	      max_offset=new_offset;
 	  }
@@ -1003,21 +1042,21 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
 	    uint32_t *subifd_offsetp;
 	    if(fseek(fr->handle, be32(entry->tdir_offset), SEEK_SET) < 0)
 	    {
-	      return 0;
+	      return -1;
 	    }
 	    subifd_offsetp=(uint32_t *)MALLOC(nbr*sizeof(*subifd_offsetp));
 	    if(fread(subifd_offsetp, sizeof(*subifd_offsetp), nbr, fr->handle) != nbr)
 	    {
 	      free(subifd_offsetp);
-	      return 0;
+	      return -1;
 	    }
 	    for(j=0; j<nbr; j++)
 	    {
 	      const uint64_t new_offset=header_check_tiff_be(fr, be32(subifd_offsetp[j]), depth+1, 0, tag_make);
-	      if(new_offset==0)
+	      if(new_offset==-1)
 	      {
 		free(subifd_offsetp);
-		return 0;
+		return -1;
 	      }
 	      if(max_offset < new_offset)
 		max_offset = new_offset;
@@ -1069,7 +1108,7 @@ static uint64_t header_check_tiff_be(file_recovery_t *fr, const uint32_t tiff_di
   if(be32(*tiff_next_diroff) > 0)
   {
     const uint64_t new_offset=header_check_tiff_be(fr, be32(*tiff_next_diroff), depth, count+1, tag_make);
-    if(max_offset < new_offset)
+    if(max_offset!=-1 && max_offset < new_offset)
       max_offset=new_offset;
   }
   return max_offset;
