@@ -59,10 +59,11 @@ typedef struct {
         uint32_t          tdir_offset;    /* byte offset to field data */
 } TIFFDirEntry;
 
+/* Work around a gcc bug https://gcc.gnu.org/bugzilla/show_bug.cgi?id=52991 */
 struct ifd_header {
   uint16_t nbr_fields;
   TIFFDirEntry ifd;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 time_t get_date_from_tiff_header(const TIFFHeader *tiff, const unsigned int tiff_size);
 const char *find_tag_from_tiff_header(const TIFFHeader *tiff, const unsigned int tiff_size, const unsigned int tag, const char **potential_error);
