@@ -174,6 +174,8 @@ static const txt_header_t fasttxt_headers[] = {
   { "ISO-10303-21;",					13, "stp"},
   /* URL / Internet Shortcut */
   { "[InternetShortcut]",				18, "url"},
+  /* Veeam Backup Metadata */
+  { "<BackupMeta Id=\"",				16, "vbm"},
   /* Windows Play List*/
   {"<?wpl version=\"1.0\"?>",				21, "wpl"},
   /* Windows URL / Internet Shortcut */
@@ -416,7 +418,7 @@ static int UTFsize(const unsigned char *buffer, const unsigned int buf_len)
   return i;
 }
 
-static int data_check_html(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_html(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   const char sign_html_end[]	= "</html>";
   const unsigned int i=UTFsize(&buffer[buffer_size/2], buffer_size/2);
@@ -428,33 +430,33 @@ static int data_check_html(const unsigned char *buffer, const unsigned int buffe
     if(buffer[j]=='<' && strncasecmp((const char *)&buffer[j], sign_html_end, sizeof(sign_html_end)-1)==0)
     {
       file_recovery->calculated_file_size+=j-buffer_size/2+sizeof(sign_html_end)-1;
-      return 2;
+      return DC_STOP;
     }
   }
   if(i<buffer_size/2)
   {
     if(i>=10)
       file_recovery->calculated_file_size=file_recovery->file_size+i;
-    return 2;
+    return DC_STOP;
   }
   file_recovery->calculated_file_size=file_recovery->file_size+(buffer_size/2);
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   const unsigned int i=UTFsize(&buffer[buffer_size/2], buffer_size/2);
   if(i<buffer_size/2)
   {
     if(i>=10)
       file_recovery->calculated_file_size=file_recovery->file_size+i;
-    return 2;
+    return DC_STOP;
   }
   file_recovery->calculated_file_size=file_recovery->file_size+(buffer_size/2);
-  return 1;
+  return DC_CONTINUE;
 }
 
-static int data_check_ttd(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
+static data_check_t data_check_ttd(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   unsigned int i;
   for(i=buffer_size/2; i<buffer_size; i++)
@@ -463,14 +465,16 @@ static int data_check_ttd(const unsigned char *buffer, const unsigned int buffer
     if((car>='A' && car<='F') || (car >='0' && car <='9') || car==' ' || car=='\n')
       continue;
     file_recovery->calculated_file_size=file_recovery->file_size + i - buffer_size/2;
-    return 2;
+    return DC_STOP;
   }
   file_recovery->calculated_file_size=file_recovery->file_size+(buffer_size/2);
-  return 1;
+  return DC_CONTINUE;
 }
 
 static int header_check_ttd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  if(buffer[56]<'0' || buffer[56]>'9')
+    return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->data_check=&data_check_ttd;
   file_recovery_new->file_check=&file_check_size;
@@ -498,6 +502,8 @@ static int header_check_ics(const unsigned char *buffer, const unsigned int buff
 {
   const char *date_asc;
   char *buffer2;
+  if(buffer[15]=='\0')
+    return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->data_check=&data_check_txt;
   file_recovery_new->file_check=&file_check_size;
@@ -578,10 +584,12 @@ static int header_check_dc(const unsigned char *buffer, const unsigned int buffe
 
 static int header_check_html(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+  if(file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
       strcmp(file_recovery->extension,"mbox")==0)
     return 0;
+  if(buffer[14]==0)
+    return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->data_check=&data_check_html;
   file_recovery_new->file_check=&file_check_size;
@@ -691,8 +699,12 @@ static int header_check_xml(const unsigned char *buffer, const unsigned int buff
 
 static int header_check_rtf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  unsigned int i;
+  for(i=0; i<16; i++)
+    if(buffer[i]=='\0')
+      return 0;
   /* Avoid a false positive with .snt */
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+  if(file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_doc)
     return 0;
   reset_file_recovery(file_recovery_new);
@@ -705,7 +717,9 @@ static int header_check_rtf(const unsigned char *buffer, const unsigned int buff
 
 static int header_check_xmp(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+  if(buffer[35]=='\0')
+    return 0;
+  if(file_recovery->file_stat!=NULL &&
       (file_recovery->file_stat->file_hint==&file_hint_pdf ||
        file_recovery->file_stat->file_hint==&file_hint_tiff))
     return 0;
@@ -719,14 +733,17 @@ static int header_check_xmp(const unsigned char *buffer, const unsigned int buff
 
 static int header_check_mbox(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+  unsigned int i;
+  if(file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
       strcmp(file_recovery->extension,"mbox")==0)
     return 0;
+  for(i=0; i<64; i++)
+    if(buffer[i]==0)
+      return 0;
   if( memcmp(buffer, "From ", 5)==0 &&
       memcmp(buffer, "From MAILER-DAEMON ", 19)!=0)
   {
-    unsigned int i;
     /* From someone@somewhere */
     for(i=5; i<200 && buffer[i]!=' ' && buffer[i]!='@'; i++);
     if(buffer[i]!='@')
@@ -747,10 +764,13 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
   {
     if(memcmp(buffer, header->string, header->len)==0)
     {
+      if(buffer[header->len]=='\0')
+	return 0;
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
       file_recovery_new->file_check=&file_check_size;
       file_recovery_new->extension=header->extension;
+      file_recovery_new->min_filesize=header->len+1;
       return 1;
     }
     header++;
@@ -831,7 +851,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     if(buffer[i]==0x0a &&
       (memcmp(buffer+i+1, "Return-Path: ", 13)==0 ||
        memcmp(buffer+i+1, "Received: from", 14)==0) &&
-        !(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+        !(file_recovery->file_stat!=NULL &&
           file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
           strcmp(file_recovery->extension,"mbox")==0))
     {
@@ -846,6 +866,8 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(strncasecmp((const char *)buffer, "@echo off", 9)==0)
   {
+    if(buffer[9]=='\0')
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
@@ -855,6 +877,8 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(strncasecmp((const char *)buffer, "<%@ language=\"vbscript", 22)==0)
   {
+    if(buffer[22]=='\0')
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
@@ -862,8 +886,10 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     file_recovery_new->extension="asp";
     return 1;
   }
-  if(strncasecmp((const char *)buffer, "version 4.00\r\nbegin", 20)==0)
+  if(strncasecmp((const char *)buffer, "version 4.00\r\nbegin", 19)==0)
   {
+    if(buffer[19]=='\0')
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
@@ -873,6 +899,8 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(strncasecmp((const char *)buffer, "begin:vcard", 11)==0)
   {
+    if(buffer[11]=='\0')
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
     file_recovery_new->file_check=&file_check_size;
@@ -920,7 +948,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   {
     return 0;
   }
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
+  if(file_recovery->file_stat!=NULL)
   {
     if(file_recovery->file_stat->file_hint == &file_hint_doc)
     {
@@ -928,7 +956,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
 	return 0;
     }
     else if(file_recovery->file_stat->file_hint == &file_hint_fasttxt ||
-    file_recovery->file_stat->file_hint == &file_hint_txt)
+	file_recovery->file_stat->file_hint == &file_hint_txt)
     {
       if(strstr(file_recovery->filename,".html")==NULL)
 	return 0;
@@ -949,16 +977,6 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
 	  td_memmem(buffer, buffer_size_test, "[camera info]", 13)!=NULL)
 	return 0;
     }
-    else if(file_recovery->file_stat->file_hint == &file_hint_zip)
-    {
-      const unsigned char zip_header[4]  = { 'P', 'K', 0x03, 0x04};
-      /* .sh3d may contain text */
-      if(strstr(file_recovery->filename, ".sh3d")!=NULL)
-	return 0;
-      /* Return if still in a zip */
-      if(td_memmem(buffer, buffer_size_test, zip_header, 4)!=NULL)
-	return 0;
-    }
     else
       return 0;
   }
@@ -1086,7 +1104,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
 #endif
     }
     else if(strstr(buffer_lower, "class")!=NULL &&
-	(l>=100 || file_recovery==NULL))
+	(l>=100 || file_recovery->file_stat==NULL))
     {
 #ifdef DJGPP
       ext="jav";
@@ -1117,7 +1135,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     {
       ext="html";
     }
-    if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
+    if(file_recovery->file_stat!=NULL)
     {
       if(file_recovery->file_stat->file_hint == &file_hint_doc)
       {
