@@ -41,6 +41,13 @@
 #include "log.h"
 
 /* #define DEBUG_ZIP */
+
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
 extern const file_hint_t file_hint_doc;
 static void register_header_check_zip(file_stat_t *file_stat);
 static int header_check_zip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
@@ -92,8 +99,7 @@ struct zip_file_entry {
   uint32_t uncompressed_size;       /** Uncompressed size */
   uint16_t filename_length;         /** Filename length */
   uint16_t extra_length;            /** Extra fields length */
-} __attribute__ ((__packed__));
-typedef struct zip_file_entry zip_file_entry_t;
+} __attribute__ ((gcc_struct, __packed__));
 
 struct zip64_extra_entry
 {
@@ -103,7 +109,9 @@ struct zip64_extra_entry
   uint64_t compressed_size;
   uint64_t offset;		/* Offset of local header record */
   uint32_t disk_start_number;	/* Number of the disk on which this file starts  */
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
+
+typedef struct zip_file_entry zip_file_entry_t;
 typedef struct zip64_extra_entry zip64_extra_entry_t;
 
 static uint32_t expected_compressed_size=0;
@@ -128,7 +136,7 @@ static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size
       if (buffer[count]==*(const char *)needle && memcmp(buffer+count, needle, size)==0)
       {
 	free(buffer);
-        if(fseek(f, (long)count-read_size, SEEK_CUR)<0)
+        if(my_fseek(f, (long)count-read_size, SEEK_CUR)<0)
           return -1;
         return total;
       }
@@ -136,7 +144,7 @@ static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size
       total++;
       left--;
     }
-    if(feof(f) || fseek(f, (long)1-size, SEEK_CUR)<0)
+    if(feof(f) || my_fseek(f, (long)1-size, SEEK_CUR)<0)
     {
       free(buffer);
       return -1;
@@ -151,6 +159,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   zip_file_entry_t  file;
   zip64_extra_entry_t extra;
   uint64_t          len;
+  unsigned int krita=0;
   if (fread(&file, sizeof(file), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -203,27 +212,29 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
       {
 	msoffice=0;
 	sh3d=0;
-	if(len==8 && memcmp(filename, "mimetype", 8)==0 &&
-	    le16(file.extra_length)==0 &&
-	    le32(file.compressed_size)==le32(file.uncompressed_size) &&
-	    le32(file.compressed_size)<=128)
+	if(len==8 && memcmp(filename, "mimetype", 8)==0 && le16(file.extra_length)==0)
 	{
-	  const int compressed_size=le32(file.uncompressed_size);
 	  unsigned char buffer[128];
-	  if( fread(buffer, compressed_size, 1, fr->handle)!=1)
+	  const unsigned int compressed_size=le32(file.compressed_size);
+	  const int to_read=(compressed_size < 128 ? compressed_size: 128);
+	  if( fread(buffer, to_read, 1, fr->handle)!=1)
 	  {
 #ifdef DEBUG_ZIP
 	    log_trace("zip: Unexpected EOF in file_entry data: %u bytes expected\n",
 	    compressed_size);
 #endif
+	    free(filename);
 	    return -1;
 	  }
-	  if (fseek(fr->handle, -compressed_size, SEEK_CUR) < 0)
+	  if (my_fseek(fr->handle, -to_read, SEEK_CUR) < 0)
 	  {
-	    log_info("fseek failed CGR\n");
+	    log_info("fseek failed\n");
+	    free(filename);
 	    return -1;
 	  }
-	  if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.calc",28)==0)
+	  if(compressed_size==16      && memcmp(buffer,"image/openraster",16)==0)
+	    *ext="ora";
+	  else if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.calc",28)==0)
 	    *ext="sxc";
 	  else if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.draw",28)==0)
 	    *ext="sxd";
@@ -239,6 +250,11 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	    *ext="ods";
 	  else if(compressed_size==47 && memcmp(buffer,"application/vnd.oasis.opendocument.presentation",47)==0)
 	    *ext="odp";
+	  else if(memcmp(buffer,"application/x-krita",19)==0)
+	  {
+	    *ext="kra";
+	    krita=19;
+	  }
 	  else
 	  { /* default to writer */
 	    *ext="sxw";
@@ -274,10 +290,13 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	/* iWork */
       if(len==23 && memcmp(filename, "QuickLook/Thumbnail.jpg", 23)==0)
 	*ext="pages";
-      else if(len==20 && strcasecmp(filename, "META-INF/MANIFEST.MF")==0)
+      else if(len==20 && strncasecmp(filename, "META-INF/MANIFEST.MF", 20)==0)
 	*ext="jar";
-      else if(len==15 && strcasecmp(filename, "chrome.manifest")==0)
+      else if(len==15 && strncasecmp(filename, "chrome.manifest", 15)==0)
 	*ext="xpi";
+      /* SMART Notebook */
+      else if(len==15 && memcmp(filename, "imsmanifest.xml", 15)==0)
+	*ext="notebook";
       else if(len==30 && memcmp(filename, "xsd/MindManagerApplication.xsd", 30)==0)
 	*ext="mmap";
     }
@@ -296,8 +315,8 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
       log_trace("zip: Unexpected EOF in file_entry header: %lu bytes expected\n", len);
 #endif
     }
-    if (fseek(fr->handle, fr->file_size, SEEK_SET) == -1 ||
-	fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (my_fseek(fr->handle, fr->file_size, SEEK_SET) == -1 ||
+	my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
       log_trace("zip: Unexpected EOF in file_entry header: %lu bytes expected\n", len);
@@ -311,9 +330,11 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   {
     len = le64(extra.compressed_size);
   }
+  if(krita>0)
+    len=krita;
   if (len>0)
   {
-    if (fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
       log_trace("zip: Unexpected EOF in file_entry data: %lu bytes expected\n", len);
@@ -362,9 +383,8 @@ static int zip_parse_central_dir(file_recovery_t *fr)
     uint16_t internal_attr;           /** Internal file attributes */
     uint32_t external_attr;           /** External file attributes */
     uint32_t offset_header;           /** Relative offset of local header */
-  } __attribute__ ((__packed__)) dir;
-
-  if (fseek(fr->handle, 2, SEEK_CUR) == -1)
+  } __attribute__ ((gcc_struct, __packed__)) dir;
+  if (my_fseek(fr->handle, 2, SEEK_CUR) == -1)
   {
 #ifdef DEBUG_ZIP
     log_trace("Unexpected EOF skipping version from central_dir\n");
@@ -396,7 +416,7 @@ static int zip_parse_central_dir(file_recovery_t *fr)
 
   /* Rest of the block - could attempt CRC check */
   len = le16(file.extra_length) + le16(dir.comment_length) + le16(file.filename_length);
-  if (fseek(fr->handle, len, SEEK_CUR) == -1)
+  if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
   {
 #ifdef DEBUG_ZIP
     log_trace("zip: Unexpected EOF in central_dir: %u bytes expected\n", len);
@@ -422,7 +442,7 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
     uint64_t number_entries2;         /** Total number of entries in the central directory */
     uint64_t size;                    /** Size of the central directory */
     uint64_t offset;                  /** Offset of start of central directory */
-  } __attribute__ ((__packed__)) dir;
+  } __attribute__ ((gcc_struct, __packed__)) dir;
 
   if (fread(&dir, sizeof(dir), 1, fr->handle) != 1)
   {
@@ -436,7 +456,7 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
   if (dir.end_size > 0)
   {
     const uint64_t len = le64(dir.end_size);
-    if (fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
       log_trace("zip: Unexpected EOF in end_central_dir_64: expected %llu bytes\n", (long long unsigned)len);
@@ -462,7 +482,7 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
     uint32_t size;                    /** Size of the central directory */
     uint32_t offset;                  /** Offset of start of central directory */
     uint16_t comment_length;          /** Comment length */
-  } __attribute__ ((__packed__)) dir;
+  } __attribute__ ((gcc_struct, __packed__)) dir;
 
   if (fread(&dir, sizeof(dir), 1, fr->handle) != 1)
   {
@@ -476,7 +496,7 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
   if (dir.comment_length)
   {
     const uint16_t len = le16(dir.comment_length);
-    if (fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
       log_trace("zip: Unexpected EOF in end_central_dir: expected %u bytes\n", len);
@@ -497,7 +517,7 @@ static int zip_parse_data_desc(file_recovery_t *fr)
     uint32_t crc32;                  /** Checksum (CRC32) */
     uint32_t compressed_size;        /** Compressed size (bytes) */
     uint32_t uncompressed_size;      /** Uncompressed size (bytes) */
-  } __attribute__ ((__packed__)) desc;
+  } __attribute__ ((gcc_struct, __packed__)) desc;
 
   if (fread(&desc, sizeof(desc), 1, fr->handle) != 1)
   {
@@ -534,7 +554,7 @@ static int zip_parse_signature(file_recovery_t *fr)
   if (len)
   {
     len = le16(len);
-    if (fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (my_fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
       log_trace("zip: Unexpected EOF in zip_parse_signature: expected %u bytes\n", len);
@@ -553,7 +573,7 @@ static int zip64_parse_end_central_dir_locator(file_recovery_t *fr)
     uint32_t disk_number;       /** Number of the disk with the start of the zip64 end of central directory */
     uint64_t relative_offset;   /** Relative offset of the zip64 end of central directory record */
     uint32_t disk_total_number; /** Total number of disks */
-  } __attribute__ ((__packed__)) loc;
+  } __attribute__ ((gcc_struct, __packed__)) loc;
 
   if (fread(&loc, sizeof(loc), 1, fr->handle) != 1)
   {
@@ -574,7 +594,7 @@ static void file_check_zip(file_recovery_t *fr)
   fr->offset_error=0;
   fr->offset_ok=0;
   first_filename[0]='\0';
-  if(fseek(fr->handle, 0, SEEK_SET) < 0)
+  if(my_fseek(fr->handle, 0, SEEK_SET) < 0)
     return ;
   while (1)
   {
@@ -660,7 +680,7 @@ static void file_rename_zip(const char *old_filename)
   fr.file_size = 0;
   fr.offset_error=0;
   first_filename[0]='\0';
-  if(fseek(fr.handle, 0, SEEK_SET) < 0)
+  if(my_fseek(fr.handle, 0, SEEK_SET) < 0)
   {
     fclose(fr.handle);
     return ;
@@ -765,7 +785,7 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
   {
     const zip_file_entry_t *file=(const zip_file_entry_t *)&buffer[4];
     const unsigned int len=le16(file->filename_length);
-    if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+    if(file_recovery->file_stat!=NULL &&
 	file_recovery->file_stat->file_hint==&file_hint_doc &&
 	(strcmp(file_recovery->extension,"doc")==0 ||
 	 strcmp(file_recovery->extension,"psmodel")==0)
@@ -773,7 +793,7 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
       return 0;
     /* A zip file begins by ZIP_FILE_ENTRY, this signature can also be
      * found for each compressed file */
-    if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+    if(file_recovery->file_stat!=NULL &&
 	file_recovery->file_stat->file_hint==&file_hint_zip &&
 	safe_header_only==0)
       return 0;
@@ -782,8 +802,11 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
     file_recovery_new->file_check=&file_check_zip;
     if(len==8 && memcmp(&buffer[30],"mimetype",8)==0)
     {
-      const unsigned int compressed_size=le32(file->uncompressed_size);
-      if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.calc",28)==0)
+      const unsigned int compressed_size=le32(file->compressed_size);
+      /* Mypaint .ora */
+      if(compressed_size==16 && memcmp(&buffer[38],"image/openraster",16)==0)
+	file_recovery_new->extension="ora";
+      else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.calc",28)==0)
 	file_recovery_new->extension="sxc";
       else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.draw",28)==0)
 	file_recovery_new->extension="sxd";
@@ -799,6 +822,8 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
 	file_recovery_new->extension="ods";
       else if(compressed_size==47 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.presentation",47)==0)
 	file_recovery_new->extension="odp";
+      else if(memcmp(&buffer[38],"application/x-krita",19)==0)
+	file_recovery_new->extension="kra";
       else
       { /* default to writer */
 	file_recovery_new->extension="sxw";
