@@ -27,6 +27,7 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -308,7 +309,7 @@ static int interface_part_bad_log(disk_t *disk_car, list_part_t *list_part)
 #if defined(__CYGWIN__) || defined(__MINGW32__)
   if(disk_car->disk_size<=((uint64_t)1<<(28-1)) && disk_size>=((uint64_t)1<<(28-1)))
   {
-    log_warning("Hint: update Windows to support LBA48 (minimum: W2K SP4 or XP SP1)");
+    log_warning("Hint: update Windows to support LBA48 (minimum: W2K SP4 or XP SP1)\n");
   }
 #endif
   {
@@ -463,7 +464,7 @@ static uint64_t get_min_location(const disk_t *disk)
   else if(disk->arch==&arch_mac)
     return 4096;
   else if(disk->arch==&arch_sun)
-    return disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size;
+    return (uint64_t)disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size;
   else if(disk->arch==&arch_xbox)
     return 0x800;
   /* arch_none */
@@ -513,6 +514,7 @@ static void search_NTFS_from_backup(disk_t *disk_car, list_part_t *list_part, co
 }
 
 typedef enum { INDSTOP_CONTINUE=0, INDSTOP_STOP=1, INDSTOP_SKIP=2, INDSTOP_QUIT=3 } indstop_t;
+
 static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_org, const int verbose, const int dump_ind, const int fast_mode, char **current_cmd)
 {
   unsigned char *buffer_disk;
@@ -537,6 +539,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
       ((uint64_t) disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size) + 1 ) *
       ((uint64_t) disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size),
       disk_car->disk_real_size);
+  assert(disk_car->sector_size>0);
   partition=partition_new(disk_car->arch);
   buffer_disk=(unsigned char*)MALLOC(16*DEFAULT_SECTOR_SIZE);
   buffer_disk0=(unsigned char*)MALLOC(16*DEFAULT_SECTOR_SIZE);
@@ -568,15 +571,36 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
     CHS_t start;
     offset2CHS_inline(disk_car,search_location,&start);
 #ifdef HAVE_NCURSES
-    if(old_cylinder!=start.cylinder &&
-	(disk_car->geom.heads_per_cylinder>1 || (start.cylinder & 0x7FFF)==0))
+    if(disk_car->geom.heads_per_cylinder>1)
+    {
+      if(old_cylinder!=start.cylinder)
+      {
+	old_cylinder=start.cylinder;
+	wmove(stdscr,ANALYSE_Y,ANALYSE_X);
+	wclrtoeol(stdscr);
+	wprintw(stdscr,"Analyse cylinder %5u/%u: %02u%%",
+	    start.cylinder, disk_car->geom.cylinders-1,
+	    (unsigned int)(search_location*100/disk_car->disk_size));
+	wrefresh(stdscr);
+	switch(check_enter_key_or_s(stdscr))
+	{
+	  case 1:
+	    ind_stop=INDSTOP_STOP;
+	    break;
+	  case 2:
+	    ind_stop=INDSTOP_SKIP;
+	    break;
+	}
+      }
+    }
+    else if((start.cylinder & 0x7FFF)==0)
     {
-      old_cylinder=start.cylinder;
       wmove(stdscr,ANALYSE_Y,ANALYSE_X);
       wclrtoeol(stdscr);
-      wprintw(stdscr,"Analyse cylinder %5u/%u: %02u%%",
-	  start.cylinder, disk_car->geom.cylinders-1,
-	  (unsigned int)((uint64_t)start.cylinder*100/disk_car->geom.cylinders));
+      wprintw(stdscr,"Analyse sector %11llu/%lu: %02u%%",
+	  search_location / disk_car->sector_size,
+	  (disk_car->disk_size-1)/disk_car->sector_size,
+	    (unsigned int)(search_location*100/disk_car->disk_size));
       wrefresh(stdscr);
       switch(check_enter_key_or_s(stdscr))
       {
@@ -662,6 +686,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
 		((start.sector==7 && (start.head<=2 || fast_mode>1)) ||
 		 search_location%(2048*512)==(7-1)*512)) ||
 	      (disk_car->arch!=&arch_i386 && (search_location%location_boundary==(7-1)*512)) ||
+	      (disk_car->arch==&arch_gpt&& (search_location%(2048*512)==(7-1)*512)) ||
 	      (disk_car->arch==&arch_none && search_location==(7-1)*512))
 	    res=search_FAT_backup(buffer_disk,disk_car,partition,verbose,dump_ind);
 	  test_nbr++;
@@ -671,6 +696,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
 	  if((disk_car->arch==&arch_i386 &&
 		((start.sector==13 && (start.head<=2 || fast_mode>1)) ||
 		 search_location%(2048*512)==(13-1)*disk_car->sector_size)) ||
+	      (disk_car->arch==&arch_gpt&& (search_location%(2048*512)==(13-1)*512)) ||
 	      (disk_car->arch!=&arch_i386 && (search_location%location_boundary==(13-1)*disk_car->sector_size)))
 	    res=search_EXFAT_backup(buffer_disk, disk_car, partition);
 	  test_nbr++;
@@ -681,6 +707,7 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
                 ((start.sector==disk_car->geom.sectors_per_head &&
 		  (start.head==disk_car->geom.heads_per_cylinder-1 || fast_mode>1)) ||
                  search_location%(2048*512)==(2048-1)*512)) ||
+	      (disk_car->arch==&arch_gpt&& (search_location%(2048*512)==(2048-1)*512)) ||
               (disk_car->arch!=&arch_i386 && search_location%location_boundary==(location_boundary-512) &&
                search_location>0))
             res=search_NTFS_backup(buffer_disk,disk_car,partition,verbose,dump_ind);
@@ -870,11 +897,11 @@ static list_part_t *search_part(disk_t *disk_car, const list_part_t *list_part_o
                   free(new_partition);
               }
               if(verbose>0)
-                log_warning("This partition ends after the disk limits. (start=%lu, size=%lu, end=%lu, disk end=%lu)\n",
-                    (unsigned long)(partition->part_offset/disk_car->sector_size),
-                    (unsigned long)(partition->part_size/disk_car->sector_size),
-                    (unsigned long)(pos_fin/disk_car->sector_size),
-                    (unsigned long)(disk_car->disk_size/disk_car->sector_size));
+                log_warning("This partition ends after the disk limits. (start=%llu, size=%llu, end=%llu, disk end=%llu)\n",
+                    (unsigned long long)(partition->part_offset/disk_car->sector_size),
+                    (unsigned long long)(partition->part_size/disk_car->sector_size),
+                    (unsigned long long)(pos_fin/disk_car->sector_size),
+                    (unsigned long long)(disk_car->disk_size/disk_car->sector_size));
               else
                 log_warning("This partition ends after the disk limits.\n");
             }
@@ -1139,7 +1166,7 @@ static list_part_t *add_ext_part_i386(disk_t *disk, list_part_t *list_part, cons
       if(deb->part->part_offset%(1024*1024)==0)
 	tmp=1024*1024;
       else
-	tmp=disk->geom.sectors_per_head * disk->sector_size;
+	tmp=(uint64_t)disk->geom.sectors_per_head * disk->sector_size;
       if(tmp < part_extended_offset)
 	part_extended_offset=tmp;
     }
@@ -1322,10 +1349,26 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
     disk_car->arch->init_structure(disk_car,list_part,verbose);
     if(verbose>0)
     {
+#ifdef TARGET_LINUX
+      unsigned int i=0;
+#endif
       /* Write found partitions in the log file */
       log_info("\nResults\n");
       for(element=list_part;element!=NULL;element=element->next)
 	log_partition(disk_car,element->part);
+#ifdef TARGET_LINUX
+      if(list_part!=NULL)
+	log_info("\nHint for advanced users. dmsetup may be used if you prefer to avoid to rewrite the partition table for the moment:\n");
+      for(element=list_part;element!=NULL;element=element->next)
+      {
+	const partition_t *partition=element->part;
+	log_info("echo \"0 %llu linear %s %llu\" | dmsetup create test%u\n",
+	    (long long unsigned)(partition->part_size/512),
+	    disk_car->device,
+	    (long long unsigned)(partition->part_offset/512),
+	    i++);
+      }
+#endif
     }
     list_part=ask_structure(disk_car,list_part,verbose,current_cmd);
     if(disk_car->arch->test_structure(list_part)==0)
@@ -1404,7 +1447,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 		)
 	      {
 		log_info("write!\n");
-		if(disk_car->arch->write_part(disk_car,list_part,RW,verbose,align))
+		if(disk_car->arch->write_part(disk_car, list_part, RW, verbose))
 		{
 		  display_message(msg_PART_WR_ERR);
 		}
@@ -1423,7 +1466,7 @@ int interface_recovery(disk_t *disk_car, const list_part_t * list_part_org, cons
 	    if(disk_car->arch->write_part!=NULL)
 	    {
 	      log_info("simulate write!\n");
-	      disk_car->arch->write_part(disk_car,list_part,RO,verbose,align);
+	      disk_car->arch->write_part(disk_car, list_part, RO, verbose);
 	    }
 	    break;
 	  case 'S':
