@@ -135,7 +135,7 @@ struct tdewf_file_header
          * 0x00 0x00
          */
         uint16_t fields_end;
-} __attribute__ ((__packed__));
+} __attribute__ ((gcc_struct, __packed__));
 
 
 struct info_file_struct
@@ -369,6 +369,8 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testd
     list_disk=hd_glob_parse("/dev/sr?", list_disk, verbose, testdisk_mode);
     /* Software (ATA)Raid configured (disk level) via dmraid */
     list_disk=hd_glob_parse("/dev/dm-*", list_disk, verbose, testdisk_mode);
+    /* Xen virtual disks */
+    list_disk=hd_glob_parse("/dev/xvd?", list_disk, verbose, testdisk_mode);
 #endif
   }
 #elif defined(TARGET_SOLARIS)
@@ -873,7 +875,7 @@ typedef struct _scsi_inquiry_data
   uint8_t product_revision[4];
   uint8_t vendor_specific[20];
   uint8_t _reserved3[40];
-} __attribute__((packed)) scsi_inquiry_data_t;
+} __attribute__((gcc_struct,__packed__)) scsi_inquiry_data_t;
 #define INQ_CMD_LEN	6
 #define INQ_REPLY_LEN	sizeof(scsi_inquiry_data_t)
 
@@ -1065,9 +1067,9 @@ static const char *file_description(disk_t *disk)
   size_to_unit(disk->disk_size, buffer_disk_size);
   if(disk->geom.heads_per_cylinder == 1 && disk->geom.sectors_per_head == 1)
     snprintf(disk->description_txt, sizeof(disk->description_txt),
-	"Disk %s - %s - %lu sectors%s",
+	"Disk %s - %s - %llu sectors%s",
 	disk->device, buffer_disk_size,
-	disk->geom.cylinders,
+	(long long unsigned)(disk->disk_size / disk->sector_size),
 	((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
   else
     snprintf(disk->description_txt, sizeof(disk->description_txt),
@@ -1404,14 +1406,11 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
 #endif
     }
   }
-  if(try_readonly>0)
+  if(hd_h<0 && try_readonly>0)
   {
     testdisk_mode&=~TESTDISK_O_RDWR;
-    if(hd_h<0)
-    {
-      mode=O_RDONLY|O_EXCL|mode_basic;
-      hd_h = open(device, mode);
-    }
+    mode=O_RDONLY|O_EXCL|mode_basic;
+    hd_h = open(device, mode);
     if(hd_h<0 && (errno==EBUSY || errno==EINVAL))
     {
       mode=O_RDONLY|mode_basic;
@@ -1499,7 +1498,7 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
        By a bug in the Linux buffer cache, we will see the old
        contents of /dev/hda when the change was made to /dev/hda3.
        In order to avoid this, discard all blocks on /dev/hda. */
-    ioctl(hd_h, BLKFLSBUF);	/* ignore errors */
+    (void)ioctl(hd_h, BLKFLSBUF);	/* ignore errors */
 #endif
     disk_get_model(hd_h, disk_car, verbose);
     disk_get_hpa_dco(hd_h, disk_car);
@@ -1508,12 +1507,14 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
 #endif
   {
     unsigned char *buffer;
+    const struct tdewf_file_header *ewf;
     const uint8_t evf_file_signature[8] = { 'E', 'V', 'F', 0x09, 0x0D, 0x0A, 0xFF, 0x00 };
     if(verbose>1)
       log_verbose("file_test_availability %s is a file\n", device);
     disk_car->sector_size=DEFAULT_SECTOR_SIZE;
     buffer=(unsigned char*)MALLOC(DEFAULT_SECTOR_SIZE);
-    if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)<0)
+    ewf=(const struct tdewf_file_header *)buffer;
+    if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE) != DEFAULT_SECTOR_SIZE)
     {
       memset(buffer,0,DEFAULT_SECTOR_SIZE);
     }
@@ -1526,7 +1527,7 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
       disk_car->disk_real_size=(uint64_t)disk_car->geom.cylinders * disk_car->geom.heads_per_cylinder * disk_car->geom.sectors_per_head * disk_car->sector_size;
       disk_car->offset=*(unsigned long*)(buffer+19);
     }
-    else if(memcmp(buffer, evf_file_signature, 8)==0)
+    else if(memcmp(buffer, evf_file_signature, 8)==0 && le16(ewf->fields_segment)==1)
     {
       free(buffer);
       free(data);
@@ -1535,8 +1536,10 @@ disk_t *file_test_availability(const char *device, const int verbose, int testdi
       free(disk_car);
       close(hd_h);
 #if defined(HAVE_LIBEWF_H) && defined(HAVE_LIBEWF)
+      log_info("EWF format detected.\n");
       return fewf_init(device, testdisk_mode);
 #else
+      log_info("EWF format detected but missing library.\n");
       return NULL;
 #endif
     }
