@@ -2,7 +2,7 @@
 
     File: jfs.h
 
-    Copyright (C) 2004,2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004,2006,2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,10 +19,16 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
 /* real size is 184 */
 #define JFS_SUPERBLOCK_SIZE 512
 
 #define L2BPERDMAP 13      /* l2 num of blks per dmap */
-int check_JFS(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind);
-int set_JFS_info(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int debug, const int dump_ind);
+int check_JFS(disk_t *disk_car, partition_t *partition);
+int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind);
+
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
