@@ -46,9 +46,11 @@
 #include "exfat.h"
 #include "ext2.h"
 #include "fat.h"
+#include "fat_common.h"
 #include "fatx.h"
 #include "iso9660.h"
 #include "iso.h"
+#include "gfs2.h"
 #include "hfs.h"
 #include "hfsp.h"
 #include "hpfs.h"
@@ -65,6 +67,9 @@
 #include "swap.h"
 #include "ufs.h"
 #include "xfs.h"
+#include "vmfs.h"
+#include "wbfs.h"
+#include "zfs.h"
 #include "log.h"
 
 static int check_part_none(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
@@ -93,6 +98,7 @@ static const struct systypes none_sys_types[] = {
   {UP_FAT16,	"FAT16"},
   {UP_FAT32,	"FAT32"},
   {UP_FREEBSD,	"FreeBSD"},
+  {UP_GFS2,	"GFS2"},
   {UP_HFS,	"HFS"},
   {UP_HFSP,	"HFS+"},
   {UP_HFSX,	"HFSX"},
@@ -101,6 +107,9 @@ static const struct systypes none_sys_types[] = {
   {UP_JFS,	"JFS"},
   {UP_LINSWAP,	"Linux SWAP"},
   {UP_LINSWAP2,	"Linux SWAP 2"},
+  {UP_LINSWAP_8K,	"Linux SWAP"},
+  {UP_LINSWAP2_8K,	"Linux SWAP 2"},
+  {UP_LINSWAP2_8KBE,	"Linux SWAP 2"},
   {UP_LUKS,	"Linux LUKS"},
   {UP_LVM,	"Linux LVM"},
   {UP_LVM2,	"Linux LVM2"},
@@ -118,7 +127,11 @@ static const struct systypes none_sys_types[] = {
   {UP_SYSV4,	"SysV 4"},
   {UP_UFS,	"UFS"},
   {UP_UFS2,	"UFS 2"},
+  {UP_UFS_LE,	"UFS - Little Endian"},
+  {UP_UFS2_LE,	"UFS 2 - Little Endian"},
   {UP_UNK,	"Unknown"},
+  {UP_VMFS,	"VMFS"},
+  {UP_WBFS,	"WBFS"},
   {UP_XFS,	"XFS"},
   {UP_XFS2,	"XFS 2"},
   {UP_XFS3,	"XFS 3"},
@@ -190,7 +203,7 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
     partition_reset(partition,&arch_none);
   if(res<=0)
   {
-    if(disk->pread(disk, buffer_disk, 8 * DEFAULT_SECTOR_SIZE, partition->part_offset) == 8 * DEFAULT_SECTOR_SIZE)
+    if(disk->pread(disk, buffer_disk, 16 * DEFAULT_SECTOR_SIZE, partition->part_offset) == 16 * DEFAULT_SECTOR_SIZE)
       res=search_type_2(buffer_disk, disk, partition,verbose,0);
   }
   if(res<=0)
@@ -203,8 +216,7 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
   }
   if(res<=0)
   {
-    if(disk->pread(disk, buffer_disk, 4096, partition->part_offset + 4096) == 4096)
-      res=search_type_8(buffer_disk, disk, partition,verbose,0);
+    res=search_type_8(buffer_disk, disk, partition,verbose,0);
   }
   if(res<=0)
   {
@@ -217,7 +229,7 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
       res=search_type_64(buffer_disk, disk, partition,verbose,0);
   }
   if(res<=0)
-    res=(recover_ISO(disk, (const struct iso_primary_descriptor*)(buffer_disk+0x200), partition, verbose, 0)==0);
+    res=(recover_ISO((const struct iso_primary_descriptor*)(buffer_disk+0x200), partition)==0);
   if(res<=0)
   {
   /* 64k offset */
@@ -225,6 +237,10 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
       res=search_type_128(buffer_disk, disk, partition,verbose,0);
   }
   if(res<=0)
+  {
+    res=search_type_2048(buffer_disk, disk, partition,verbose,0);
+  }
+  if(res<=0)
   { /* Search FAT32 backup */
     partition->part_offset = 6*512;
     res=search_FAT_backup(buffer_disk, disk, partition, verbose, 0);
@@ -236,10 +252,13 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
   }
   if(res<=0)
   { /* Search NTFS backup */
-    partition->part_offset = disk->disk_size - disk->sector_size;
-    res=search_NTFS_backup(buffer_disk, disk, partition, verbose, 0);
-    if(res>0 && partition->part_offset!=0)
-      res=0;
+    if(disk->disk_size > disk->sector_size)
+    {
+      partition->part_offset = disk->disk_size - disk->sector_size;
+      res=search_NTFS_backup(buffer_disk, disk, partition, verbose, 0);
+      if(res>0 && partition->part_offset!=0)
+	res=0;
+    }
   }
   if(res<=0)
   {
@@ -249,10 +268,9 @@ static list_part_t *read_part_none(disk_t *disk, const int verbose, const int sa
       /* sparse superblock feature: The groups chosen are 0, 1 and powers of 3, 5 and 7. */
       /* Checking group 3 */
       const uint64_t hd_offset=3*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)+(s_log_block_size==0?2*DEFAULT_SECTOR_SIZE:0);
-      void *data=disk->pread_fast(disk, buffer_disk, 1024, hd_offset);
-      if(data!=NULL)
+      if(disk->pread(disk, buffer_disk, 1024, hd_offset)==1024)
       {
-	const struct ext2_super_block *sb=(const struct ext2_super_block*)data;
+	const struct ext2_super_block *sb=(const struct ext2_super_block*)buffer_disk;
 	partition->part_offset = hd_offset;
 	if(le16(sb->s_block_group_nr)>0 &&
 	    le16(sb->s_magic)==EXT2_SUPER_MAGIC &&
@@ -323,7 +341,7 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       ret=check_BeFS(disk_car,partition);
       break;
     case UP_BTRFS:
-      ret=check_btrfs(disk_car, partition, verbose);
+      ret=check_btrfs(disk_car, partition);
       break;
     case UP_CRAMFS:
       ret=check_cramfs(disk_car,partition,verbose);
@@ -349,6 +367,9 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
     case UP_FREEBSD:
       ret=check_BSD(disk_car,partition,verbose,BSD_MAXPARTITIONS);
       break;
+    case UP_GFS2:
+      ret=check_gfs2(disk_car, partition);
+      break;
     case UP_HFS:
       ret=check_HFS(disk_car,partition,verbose);
       break;
@@ -367,6 +388,9 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       break;
     case UP_LINSWAP:
     case UP_LINSWAP2:
+    case UP_LINSWAP_8K:
+    case UP_LINSWAP2_8K:
+    case UP_LINSWAP2_8KBE:
       ret=check_Linux_SWAP(disk_car, partition);
       break;
     case UP_LUKS:
@@ -412,21 +436,26 @@ static int check_part_none(disk_t *disk_car,const int verbose,partition_t *parti
       break;
     case UP_UFS:
     case UP_UFS2:
+    case UP_UFS_LE:
+    case UP_UFS2_LE:
       ret=check_ufs(disk_car,partition,verbose);
       break;
+    case UP_VMFS:
+      ret=check_VMFS(disk_car, partition);
+      break;
+    case UP_WBFS:
+      ret=check_WBFS(disk_car, partition);
+      break;
     case UP_XFS:
     case UP_XFS2:
     case UP_XFS3:
     case UP_XFS4:
       ret=check_xfs(disk_car,partition,verbose);
       break;
-    case UP_UNK:
+    case UP_ZFS:
+      ret=check_ZFS(disk_car, partition);
       break;
-    default:
-      if(verbose>0)
-      {
-        log_info("check_part_none %u type %02X: no test\n",partition->order,partition->upart_type);
-      }
+    case UP_UNK:
       break;
   }
   return ret;
