@@ -96,9 +96,10 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
   params->disk->pread(params->disk, buffer, READ_SIZE, offset);
   while(current_search_space!=list_search_space)
   {
-    uint64_t old_offset=offset;
+    const uint64_t old_offset=offset;
     {
       file_recovery_t file_recovery_new;
+      file_recovery_new.blocksize=blocksize;
       if(file_recovery.file_stat!=NULL &&
           file_recovery.file_stat->file_hint->min_header_distance > 0 &&
           file_recovery.file_size<=file_recovery.file_stat->file_hint->min_header_distance)
@@ -141,12 +142,11 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
     /* Check for data EOF */
     if(file_recovery.file_stat!=NULL)
     {
-      int res=1;
+      data_check_t res=DC_CONTINUE;
       if(file_recovery.data_check!=NULL)
 	res=file_recovery.data_check(buffer_olddata, 2*blocksize, &file_recovery);
       file_recovery.file_size+=blocksize;
-      file_recovery.file_size_on_disk+=blocksize;
-      if(res==2)
+      if(res==DC_STOP || res==DC_ERROR)
       {
 	/* EOF found */
 	reset_file_recovery(&file_recovery);
