@@ -22,7 +22,7 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
-int photorec_find_blocksize(disk_t *disk_car, partition_t *partition, const int verbose, const int interface, file_stat_t *file_stats, unsigned int *file_nbr, unsigned int *blocksize, alloc_data_t *list_search_space, const time_t real_start_time);
+pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
