@@ -55,108 +55,35 @@
 #include "setdate.h"
 #include "dfxml.h"
 
+#if defined(HAVE_FSEEKO) && !defined(__MINGW32__)
+#define my_fseek fseeko
+#else
+#define my_fseek fseek
+#endif
+
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
 /* #define DEBUG_FREE */
 
-static void update_search_space(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize);
 static void update_search_space_aux(alloc_data_t *list_search_space, uint64_t start, uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset);
-static alloc_data_t *file_truncate(alloc_data_t *space, file_recovery_t *file, const unsigned int sector_size, const unsigned int blocksize);
-static alloc_data_t *file_error(alloc_data_t *space, file_recovery_t *file, const unsigned int blocksize);
-static void list_free_add(const file_recovery_t *file_recovery, alloc_data_t *list_search_space);
-static void list_space_used(const file_recovery_t *file_recovery, const unsigned int sector_size);
+static void file_block_truncate_zero(const file_recovery_t *file_recovery, alloc_data_t *list_search_space);
+static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize);
 
-static void list_space_used(const file_recovery_t *file_recovery, const unsigned int sector_size)
+void file_block_log(const file_recovery_t *file_recovery, const unsigned int sector_size)
 {
   struct td_list_head *tmp;
-  uint64_t file_size=0;
-  uint64_t file_size_on_disk=0;
-  if(file_recovery->filename==NULL)
+  if(file_recovery->filename[0]=='\0')
     return;
   log_info("%s\t",file_recovery->filename);
   td_list_for_each(tmp, &file_recovery->location.list)
   {
     const alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
-    file_size_on_disk+=(element->end-element->start+1);
     if(element->data>0)
-    {
       log_info(" %lu-%lu", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
-      file_size+=(element->end-element->start+1);
-    }
     else
-    {
       log_info(" (%lu-%lu)", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
-    }
   }
   log_info("\n");
-  /*
-  log_trace("list file_size %lu, file_size_on_disk %lu\n",
-      (unsigned long)file_size, (unsigned long)file_size_on_disk);
-  log_trace("file_size %lu, file_size_on_disk %lu\n",
-      (unsigned long)file_recovery->file_size, (unsigned long)file_recovery->file_size_on_disk);
-   */
-}
-
-static void list_free_add(const file_recovery_t *file_recovery, alloc_data_t *list_search_space)
-{
-  struct td_list_head *search_walker = NULL;
-#ifdef DEBUG_FREE
-  log_trace("list_free_add %lu\n",(long unsigned)(file_recovery->location.start/512));
-#endif
-  td_list_for_each(search_walker, &list_search_space->list)
-  {
-    alloc_data_t *current_search_space;
-    current_search_space=td_list_entry(search_walker, alloc_data_t, list);
-    if(current_search_space->start < file_recovery->location.start && file_recovery->location.start < current_search_space->end)
-    {
-      alloc_data_t *new_free_space;
-      new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
-      new_free_space->start=file_recovery->location.start;
-      new_free_space->end=current_search_space->end;
-      new_free_space->file_stat=NULL;
-      new_free_space->data=1;
-      current_search_space->end=file_recovery->location.start-1;
-      td_list_add(&new_free_space->list, search_walker);
-    }
-    if(current_search_space->start==file_recovery->location.start)
-    {
-      current_search_space->file_stat=file_recovery->file_stat;
-      return ;
-    }
-  }
-}
-
-/*
- * new_current_search_space!=NULL
- * offset!=NULL
- */
-static void update_search_space(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize)
-{
-  struct td_list_head *search_walker = NULL;
-#ifdef DEBUG_UPDATE_SEARCH_SPACE
-  log_trace("update_search_space\n");
-  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
-#endif
-
-  td_list_for_each(search_walker, &list_search_space->list)
-  {
-    struct td_list_head *tmp;
-    alloc_data_t *current_search_space;
-    current_search_space=td_list_entry(search_walker, alloc_data_t, list);
-    if(current_search_space->start <= file_recovery->location.start &&
-        file_recovery->location.start <= current_search_space->end)
-    {
-      *offset=file_recovery->location.start;
-      *new_current_search_space=current_search_space;
-      td_list_for_each(tmp, &file_recovery->location.list)
-      {
-	const alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
-        uint64_t end=(element->end-(element->start%blocksize)+blocksize-1+1)/blocksize*blocksize+(element->start%blocksize)-1;
-        update_search_space_aux(list_search_space, element->start, end, new_current_search_space, offset);
-      }
-      return ;
-    }
-  }
 }
 
 void del_search_space(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end)
@@ -354,6 +281,26 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
   return dir_num;
 }
 
+void get_prev_location(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset, const uint64_t prev_location)
+{
+  int nbr;
+  alloc_data_t *file_space=*current_search_space;
+  uint64_t size=0;
+  /* Search backward the first fragment of a file not successfully recovered
+   * Limit the search to 10 fragments or 1GB */
+  for(nbr=0; nbr<3 && size < (uint64_t)200*1024*1024; nbr++)
+  {
+    file_space=td_list_entry(file_space->list.prev, alloc_data_t, list);
+    if(file_space==list_search_space)
+      return;
+    size+=file_space->end - file_space->start + 1;
+    if(file_space->start < prev_location)
+      return ;
+    *current_search_space=file_space;
+    *offset=file_space->start;
+  }
+}
+
 int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
 {
   int nbr;
@@ -361,7 +308,7 @@ int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current
   uint64_t size=0;
   /* Search backward the first fragment of a file not successfully recovered
    * Limit the search to 10 fragments or 1GB */
-  for(nbr=0; nbr<10 && size < (uint64_t)1024*1024*1024; nbr++)
+  for(nbr=0; nbr<3 && size < (uint64_t)200*1024*1024; nbr++)
   {
     file_space=td_list_entry(file_space->list.prev, alloc_data_t, list);
     if(file_space==list_search_space)
@@ -525,21 +472,54 @@ unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int
   return blocksize;
 }
 
-void update_blocksize(unsigned int blocksize, alloc_data_t *list_search_space, const uint64_t offset)
+void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_space, const uint64_t offset)
 {
   struct td_list_head *search_walker = NULL;
-  struct td_list_head *search_walker_next = NULL;
-  td_list_for_each_safe(search_walker,search_walker_next,&list_search_space->list)
+  struct td_list_head *search_walker_prev = NULL;
+  log_info("blocksize=%u, offset=%u\n", blocksize, (unsigned int)(offset%blocksize));
+  /* Align end of last range (round up) */
+  search_walker=list_search_space->list.prev;
   {
-    uint64_t old_start;
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
-    old_start=current_search_space->start;
-    current_search_space->start=(current_search_space->start-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize;
-    if(current_search_space->start!=old_start)
-      current_search_space->file_stat=NULL;
+    current_search_space->end=(current_search_space->end+1-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize-1;
+  }
+  /* Align start of each range */
+  td_list_for_each_prev_safe(search_walker,search_walker_prev,&list_search_space->list)
+  {
+    alloc_data_t *current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    const uint64_t aligned_start=(current_search_space->start-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize;
+    if(current_search_space->start!=aligned_start)
+    {
+      alloc_data_t *prev_search_space=td_list_entry(search_walker_prev, alloc_data_t, list);
+      if(prev_search_space->end + 1 == current_search_space->start)
+      {
+	/* merge with previous block */
+	prev_search_space->end = current_search_space->end;
+	td_list_del(search_walker);
+	free(current_search_space);
+      }
+      else
+      {
+	current_search_space->start=aligned_start;
+	current_search_space->file_stat=NULL;
+	if(current_search_space->start>=current_search_space->end)
+	{
+	  /* block too small - delete it */
+	  td_list_del(search_walker);
+	  free(current_search_space);
+	}
+      }
+    }
+  }
+  /* Align end of each range (truncate) */
+  td_list_for_each_prev_safe(search_walker, search_walker_prev, &list_search_space->list)
+  {
+    alloc_data_t *current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    current_search_space->end=(current_search_space->end+1-offset%blocksize)/blocksize*blocksize+offset%blocksize-1;
     if(current_search_space->start>=current_search_space->end)
     {
+      /* block too small - delete it */
       td_list_del(search_walker);
       free(current_search_space);
     }
@@ -548,7 +528,7 @@ void update_blocksize(unsigned int blocksize, alloc_data_t *list_search_space, c
 
 uint64_t free_list_allocation_end=0;
 
-static void free_list_allocation(alloc_list_t *list_allocation)
+void file_block_free(alloc_list_t *list_allocation)
 {
   struct td_list_head *tmp = NULL;
   struct td_list_head *tmp_next = NULL;
@@ -569,65 +549,62 @@ static void free_list_allocation(alloc_list_t *list_allocation)
 static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *params, const int paranoid)
 {
   if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
-      params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
-  {
-    if(file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && paranoid>0)
+      params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING &&
+      file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && paranoid>0)
     { /* Check if recovered file is valid */
       file_recovery->file_check(file_recovery);
     }
-    /* FIXME: need to adapt read_size to volume size to avoid this */
-    if(file_recovery->file_size > params->disk->disk_size)
-      file_recovery->file_size = params->disk->disk_size;
-    if(file_recovery->file_size > params->disk->disk_real_size)
-      file_recovery->file_size = params->disk->disk_real_size;
-    if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
-	file_recovery->file_size < file_recovery->min_filesize)
-    { 
-      log_info("File too small ( %llu < %llu), reject it\n",
-	  (long long unsigned) file_recovery->file_size,
-	  (long long unsigned) file_recovery->min_filesize);
-      file_recovery->file_size=0;
-      file_recovery->file_size_on_disk=0;
-    }
+  /* FIXME: need to adapt read_size to volume size to avoid this */
+  if(file_recovery->file_size > params->disk->disk_size)
+    file_recovery->file_size = params->disk->disk_size;
+  if(file_recovery->file_size > params->disk->disk_real_size)
+    file_recovery->file_size = params->disk->disk_real_size;
+  if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
+      file_recovery->file_size < file_recovery->min_filesize)
+  { 
+    log_info("%s File too small ( %llu < %llu), reject it\n",
+	file_recovery->filename,
+	(long long unsigned) file_recovery->file_size,
+	(long long unsigned) file_recovery->min_filesize);
+    file_recovery->file_size=0;
   }
   if(file_recovery->file_size==0)
   {
+    if(paranoid==2)
+      return ;
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
     /* File is zero-length; erase it */
     unlink(file_recovery->filename);
+    return;
   }
-  else
-  {
 #ifdef HAVE_FTRUNCATE
-    fflush(file_recovery->handle);
-    if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
-    {
-      log_critical("ftruncate failed.\n");
-    }
-#endif
-    fclose(file_recovery->handle);
-    file_recovery->handle=NULL;
-    if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
-      set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
-    if(file_recovery->file_rename!=NULL)
-      file_recovery->file_rename(file_recovery->filename);
-    if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
-    {
-      params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
-    }
-    if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
-	params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
-      file_recovery->file_stat->recovered++;
+  fflush(file_recovery->handle);
+  if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
+  {
+    log_critical("ftruncate failed.\n");
   }
+#endif
+  fclose(file_recovery->handle);
+  file_recovery->handle=NULL;
+  if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
+    set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
+  if(file_recovery->file_rename!=NULL)
+    file_recovery->file_rename(file_recovery->filename);
+  if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
+  {
+    params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
+  }
+  if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
+      params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING &&
+      file_recovery->file_stat!=NULL)
+    file_recovery->file_stat->recovered++;
 }
 
-/** file_finish()
+/** file_finish_bf()
     @param file_recovery - 
     @param struct ph_param *params
     @param alloc_data_t *list_search_space
-    @param alloc_data_t **current_search_space
-    @param *offset
 
     @returns:
    -1: file not recovered, file_size=0 offset_error!=0
@@ -635,53 +612,33 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
     1: file recovered
  */
 
-int file_finish(file_recovery_t *file_recovery, struct ph_param *params,
-    alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
+int file_finish_bf(file_recovery_t *file_recovery, struct ph_param *params,
+    alloc_data_t *list_search_space)
 {
-  int file_recovered=0;
-#ifdef DEBUG_FILE_FINISH
-  log_debug("file_finish start %lu (%lu-%lu)\n", (long unsigned int)((*offset)/params->blocksize),
-      (unsigned long int)((*current_search_space)->start/params->blocksize),
-      (unsigned long int)((*current_search_space)->end/params->blocksize));
-  log_debug("file_recovery->offset_error=%llu\n", (long long unsigned)file_recovery->offset_error);
-  log_debug("file_recovery->handle %s NULL\n", (file_recovery->handle!=NULL?"!=":"=="));
-  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
-#endif
+  if(file_recovery->file_stat==NULL)
+    return 0;
   if(file_recovery->handle)
-    file_finish_aux(file_recovery, params, 1);
-  if(file_recovery->file_stat!=NULL)
+    file_finish_aux(file_recovery, params, 2);
+  if(file_recovery->file_size==0)
   {
-    list_truncate(&file_recovery->location,file_recovery->file_size);
-    if(file_recovery->file_size==0)
+    if(file_recovery->offset_error!=0)
+      return -1;
+    file_block_truncate_zero(file_recovery, list_search_space);
+    if(file_recovery->handle!=NULL)
     {
-      /* File hasn't been sucessfully recovered, remember where it begins */
-      list_free_add(file_recovery, list_search_space);
-      if((*current_search_space)!=list_search_space &&
-          !((*current_search_space)->start <= *offset && *offset <= (*current_search_space)->end))
-        *current_search_space=td_list_entry((*current_search_space)->list.next, alloc_data_t, list);
+      fclose(file_recovery->handle);
+      unlink(file_recovery->filename);
     }
-    else
-    {
-      list_space_used(file_recovery, params->disk->sector_size);
-#ifdef ENABLE_DFXML
-      xml_log_file_recovered(file_recovery);
-#endif
-      update_search_space(file_recovery, list_search_space, current_search_space, offset, params->blocksize);
-      file_recovered=1;			/* note that file was recovered */
-    }
-    free_list_allocation(&file_recovery->location);
-  }
-  if(file_recovery->file_size==0 && file_recovery->offset_error!=0)
-    file_recovered = -1;
-  else
     reset_file_recovery(file_recovery);
-#ifdef DEBUG_FILE_FINISH
-  log_debug("file_finish end %lu (%lu-%lu)\n\n", (long unsigned int)((*offset)/params->blocksize),
-      (unsigned long int)((*current_search_space)->start/params->blocksize),
-      (unsigned long int)((*current_search_space)->end/params->blocksize));
-  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
+    return 0;
+  }
+  file_block_truncate(file_recovery, list_search_space, params->blocksize);
+  file_block_log(file_recovery, params->disk->sector_size);
+#ifdef ENABLE_DFXML
+  xml_log_file_recovered(file_recovery);
 #endif
-  return file_recovered;
+  file_block_free(&file_recovery->location);
+  return 1;
 }
 
 /*  file_finish2()
@@ -689,60 +646,32 @@ int file_finish(file_recovery_t *file_recovery, struct ph_param *params,
     @param struct ph_param *params
     const struct ph_options *options
     @param alloc_data_t *list_search_space
-    @param alloc_data_t **current_search_space
-    @param *offset
 
     @returns:
-   -1: file not recovered, file_size=0 offset_error!=0
     0: file not recovered
     1: file recovered
  */
-int file_finish2(file_recovery_t *file_recovery, struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
+pfstatus_t file_finish2(file_recovery_t *file_recovery, struct ph_param *params, const int paranoid, alloc_data_t *list_search_space)
 {
-  int file_recovered=0;
-#ifdef DEBUG_FILE_FINISH
-  log_debug("file_recovery->offset_error=%llu\n", (long long unsigned)file_recovery->offset_error);
-  log_debug("file_recovery->handle %s NULL\n", (file_recovery->handle!=NULL?"!=":"=="));
-  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
-#endif
+  int file_truncated;
+  if(file_recovery->file_stat==NULL)
+    return PFSTATUS_BAD;
   if(file_recovery->handle)
-    file_finish_aux(file_recovery, params, options->paranoid);
-  if(file_recovery->file_stat!=NULL)
+    file_finish_aux(file_recovery, params, (paranoid==0?0:1));
+  if(file_recovery->file_size==0)
   {
-    if(file_recovery->file_size==0)
-    {
-      /* File hasn't been sucessfully recovered */
-      if(file_recovery->offset_error>0)
-      {
-	alloc_data_t *datanext=file_error(list_search_space, file_recovery, params->blocksize);
-	if(datanext!=NULL)
-	{
-	  *current_search_space=datanext;
-	  *offset=datanext->start;
-	  file_recovered=-1;
-	}
-      }
-    }
-    else
-    {
-#ifdef ENABLE_DFXML
-      xml_log_file_recovered2(list_search_space, file_recovery);
-#endif
-      *current_search_space=file_truncate(list_search_space, file_recovery, params->disk->sector_size, params->blocksize);
-      *offset=(*current_search_space)->start;
-      file_recovered=1;
-    }
-    free_list_allocation(&file_recovery->location);
-  }
-  if(file_recovery->file_size==0 && file_recovery->offset_error!=0)
-  {
-  }
-  else
+    file_block_truncate_zero(file_recovery, list_search_space);
     reset_file_recovery(file_recovery);
-#ifdef DEBUG_FILE_FINISH
-  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
+    return PFSTATUS_BAD;
+  }
+  file_truncated=file_block_truncate(file_recovery, list_search_space, params->blocksize);
+  file_block_log(file_recovery, params->disk->sector_size);
+#ifdef ENABLE_DFXML
+  xml_log_file_recovered(file_recovery);
 #endif
-  return file_recovered;
+  file_block_free(&file_recovery->location);
+  reset_file_recovery(file_recovery);
+  return (file_truncated>0?PFSTATUS_OK_TRUNCATED:PFSTATUS_OK);
 }
 
 void info_list_search_space(const alloc_data_t *list_search_space, const alloc_data_t *current_search_space, const unsigned int sector_size, const int keep_corrupted_file, const int verbose)
@@ -777,114 +706,6 @@ void info_list_search_space(const alloc_data_t *list_search_space, const alloc_d
       (keep_corrupted_file>0?"but saved":"and rejected"));
 }
 
-static alloc_data_t *file_truncate_aux(alloc_data_t *space, alloc_data_t *file, const uint64_t file_size, const unsigned int sector_size, const unsigned int blocksize)
-{
-  struct td_list_head *tmp;
-  struct td_list_head *next;
-  uint64_t size=0;
-  const uint64_t file_size_on_disk=(file_size+blocksize-1)/blocksize*blocksize;
-  for(tmp=&file->list, next=tmp->next; tmp!=&space->list; tmp=next, next=tmp->next)
-  {
-    alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
-    if(size >= file_size)
-      return element;
-    if(element->data>0)
-    {
-      const uint64_t len=element->end - element->start + 1;
-      if(size + len <= file_size_on_disk)
-      {
-	size+=len;
-	log_info(" %lu-%lu", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
-	td_list_del(tmp);
-	free(element);
-      }
-      else
-      {
-	log_info(" %lu-%lu",
-	    (unsigned long)(element->start/sector_size),
-	    (unsigned long)((element->start + file_size_on_disk - size - 1)/sector_size));
-	element->start+=file_size_on_disk - size;
-	element->file_stat=NULL;
-	element->data=1;
-	return element;
-      }
-    }
-    else
-    {
-      log_info(" (%lu-%lu)", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
-      td_list_del(tmp);
-      free(element);
-    }
-  }
-  return space;
-}
-
-static alloc_data_t *file_truncate(alloc_data_t *space, file_recovery_t *file, const unsigned int sector_size, const unsigned int blocksize)
-{
-  alloc_data_t *datanext;
-  if(file->filename!=NULL)
-    log_info("%s\t", file->filename);
-  else
-    log_info("?\t");
-  datanext=file_truncate_aux(space, file->loc, file->file_size, sector_size, blocksize);
-  log_info("\n");
-  return datanext;
-}
-
-static alloc_data_t *file_error_aux(alloc_data_t *space, alloc_data_t *file, const uint64_t file_size, const unsigned int blocksize)
-{
-  struct td_list_head *tmp;
-  struct td_list_head *next;
-  uint64_t size=0;
-  const uint64_t file_size_on_disk=file_size/blocksize*blocksize;
-  for(tmp=&file->list, next=tmp->next; tmp!=&space->list; tmp=next, next=tmp->next)
-  {
-    alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
-    if(size >= file_size)
-      return NULL;
-    if(element->data>0)
-    {
-      if(size + (element->end-element->start+1) <= file_size_on_disk)
-      {
-	size=size + (element->end-element->start+1);
-      }
-      else if(file_size_on_disk > size)
-      {
-	if(element->file_stat==NULL)
-	  return NULL;
-	if(next!=&space->list)
-	{
-	  alloc_data_t *new_element;
-	  new_element=td_list_entry(next, alloc_data_t, list);
-	  if(element->end+1==new_element->start && new_element->file_stat==NULL)
-	  {
-	    new_element->start-=file_size_on_disk - size;
-	    element->end=new_element->start - 1;
-	    return new_element;
-	  }
-	}
-	{
-	  alloc_data_t *new_element;
-	  new_element=(alloc_data_t*)MALLOC(sizeof(*new_element));
-	  memcpy(new_element, element, sizeof(*new_element));
-	  new_element->start+=file_size_on_disk - size;
-	  new_element->file_stat=NULL;
-	  new_element->data=1;
-	  td_list_add(&new_element->list, &element->list);
-	  element->end=new_element->start - 1;
-	  return new_element;
-	}
-      }
-    }
-  }
-  return NULL;
-}
-
-static alloc_data_t *file_error(alloc_data_t *space, file_recovery_t *file, const unsigned int blocksize)
-{
-  return file_error_aux(space, file->loc, file->offset_error, blocksize);
-}
-
 void free_search_space(alloc_data_t *list_search_space)
 {
   struct td_list_head *search_walker = NULL;
@@ -1058,3 +879,283 @@ list_part_t *init_list_part(disk_t *disk, const struct ph_options *options)
   }
   return list_part;
 }
+
+/* file_block_remove_from_sp: remove block from list_search_space, update offset and new_current_search_space in consequence */
+static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize)
+{
+  if(tmp->start == *offset)
+  {
+    tmp->start+=blocksize;
+    *offset += blocksize;
+    tmp->file_stat=NULL;
+    if(tmp->start <= tmp->end)
+      return ;
+    *new_current_search_space=td_list_entry(tmp->list.next, alloc_data_t, list);
+    *offset=(*new_current_search_space)->start;
+    td_list_del(&tmp->list);
+    free(tmp);
+    return ;
+  }
+  if(*offset + blocksize == tmp->end + 1)
+  {
+    tmp->end-=blocksize;
+    *new_current_search_space=td_list_entry(tmp->list.next, alloc_data_t, list);
+    *offset=(*new_current_search_space)->start;
+    return ;
+  }
+  {
+    alloc_data_t *new_sp;
+    new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    new_sp->start=*offset + blocksize;
+    new_sp->end=tmp->end;
+    new_sp->file_stat=NULL;
+    new_sp->data=tmp->data;
+    new_sp->list.prev=&new_sp->list;
+    new_sp->list.next=&new_sp->list;
+    tmp->end=*offset - 1;
+    td_list_add(&new_sp->list, &tmp->list);
+    *new_current_search_space=new_sp;
+    *offset += blocksize;
+  }
+}
+
+static inline void file_block_remove_from_sp(alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize)
+{
+  struct td_list_head *search_walker = &(*new_current_search_space)->list;
+  if(search_walker!=NULL)
+  {
+    alloc_data_t *tmp;
+    tmp=td_list_entry(search_walker, alloc_data_t, list);
+    if(tmp->start <= *offset && *offset + blocksize <= tmp->end + 1)
+      return file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+  }
+  td_list_for_each(search_walker, &list_search_space->list)
+  {
+    alloc_data_t *tmp;
+    tmp=td_list_entry(search_walker, alloc_data_t, list);
+    if(tmp->start <= *offset && *offset + blocksize <= tmp->end + 1)
+      return file_block_remove_from_sp_aux(tmp, new_current_search_space, offset, blocksize);
+  }
+  log_critical("file_block_remove_from_sp(list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize) failed\n");
+}
+
+static inline void file_block_add_to_file(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const unsigned int data)
+{
+  if(!td_list_empty(&list->list))
+  {
+    alloc_list_t *prev=td_list_entry(list->list.prev, alloc_list_t, list);
+    if(prev->end+1==offset && prev->data==data)
+    {
+      prev->end=offset+blocksize-1;
+      return ;
+    }
+  }
+  {
+    alloc_list_t *new_list=(alloc_list_t *)MALLOC(sizeof(*new_list));
+    new_list->start=offset;
+    new_list->end=offset+blocksize-1;
+    new_list->data=data;
+    td_list_add_tail(&new_list->list, &list->list);
+  }
+}
+
+void file_block_append(file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize, const unsigned int data)
+{
+  file_block_add_to_file(&file_recovery->location, *offset, blocksize, data);
+  file_block_remove_from_sp(list_search_space, new_current_search_space, offset, blocksize);
+}
+
+static void file_block_truncate_aux(const uint64_t start, const uint64_t end, alloc_data_t *list_search_space)
+{
+  struct td_list_head *search_walker = NULL;
+  if(start >= end)
+    return ;
+  td_list_for_each(search_walker, &list_search_space->list)
+  {
+    alloc_data_t *tmp;
+    tmp=td_list_entry(search_walker, alloc_data_t, list);
+    if(tmp->start == end + 1 && tmp->file_stat==NULL)
+    {
+      tmp->start=start;
+      return;
+    }
+    if(tmp->end + 1 == start)
+    {
+      tmp->end=end;
+      return;
+    }
+    if(end < tmp->start)
+    {
+      alloc_data_t *new_sp;
+      new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+      new_sp->start=start;
+      new_sp->end=end;
+      new_sp->file_stat=NULL;
+      new_sp->data=1;
+      new_sp->list.prev=&new_sp->list;
+      new_sp->list.next=&new_sp->list;
+      td_list_add(&new_sp->list, tmp->list.prev);
+      return;
+    }
+  }
+  {
+    alloc_data_t *new_sp;
+    new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    new_sp->start=start;
+    new_sp->end=end;
+    new_sp->file_stat=NULL;
+    new_sp->data=1;
+    new_sp->list.prev=&new_sp->list;
+    new_sp->list.next=&new_sp->list;
+    td_list_add_tail(&new_sp->list, &list_search_space->list);
+  }
+}
+
+static void file_block_truncate_zero_aux(const uint64_t start, const uint64_t end, alloc_data_t *list_search_space, file_stat_t *file_stat)
+{
+  struct td_list_head *search_walker = NULL;
+  if(start >= end)
+    return ;
+  td_list_for_each(search_walker, &list_search_space->list)
+  {
+    alloc_data_t *tmp;
+    tmp=td_list_entry(search_walker, alloc_data_t, list);
+    if(tmp->start == end + 1 && tmp->file_stat==NULL)
+    {
+      tmp->start=start;
+      tmp->file_stat=file_stat;
+      return;
+    }
+    if(end < tmp->start)
+    {
+      alloc_data_t *new_sp;
+      new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+      new_sp->start=start;
+      new_sp->end=end;
+      new_sp->file_stat=file_stat;
+      new_sp->data=1;
+      new_sp->list.prev=&new_sp->list;
+      new_sp->list.next=&new_sp->list;
+      td_list_add(&new_sp->list, tmp->list.prev);
+      return;
+    }
+  }
+  {
+    alloc_data_t *new_sp;
+    new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    new_sp->start=start;
+    new_sp->end=end;
+    new_sp->file_stat=file_stat;
+    new_sp->data=1;
+    new_sp->list.prev=&new_sp->list;
+    new_sp->list.next=&new_sp->list;
+    td_list_add_tail(&new_sp->list, &list_search_space->list);
+  }
+}
+
+static void file_block_truncate_zero(const file_recovery_t *file_recovery, alloc_data_t *list_search_space)
+{
+  struct td_list_head *tmp;
+  struct td_list_head *next;
+  int first=1;
+  td_list_for_each_safe(tmp, next, &file_recovery->location.list)
+  {
+    alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
+    if(first)
+    {
+      file_block_truncate_zero_aux(element->start, element->end, list_search_space, file_recovery->file_stat);
+      first=0;
+    }
+    else
+      file_block_truncate_aux(element->start, element->end, list_search_space);
+    td_list_del(tmp);
+    free(element);
+  }
+}
+
+static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize)
+{
+  struct td_list_head *tmp;
+  struct td_list_head *next;
+  uint64_t size=0;
+  int file_truncated=0;
+  td_list_for_each_safe(tmp, next, &file_recovery->location.list)
+  {
+    alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
+    if(size >= file_recovery->file_size)
+    {
+      file_block_truncate_aux(element->start, element->end, list_search_space);
+      td_list_del(tmp);
+      free(element);
+      file_truncated=1;
+    }
+    else if(element->data>0)
+    {
+      if(size + element->end - element->start + 1 > file_recovery->file_size)
+      {
+	const uint64_t diff=(file_recovery->file_size - size + blocksize - 1) / blocksize * blocksize;
+	file_block_truncate_aux(element->start + diff, element->end, list_search_space);
+	element->end-=element->end - element->start + 1 - diff;
+	size=file_recovery->file_size;
+      }
+      else
+	size+=(element->end-element->start+1);
+    }
+  }
+  return file_truncated;
+}
+
+static uint64_t file_offset_end(const file_recovery_t *file_recovery)
+{
+  const struct td_list_head *tmp=file_recovery->location.list.prev;
+  const alloc_list_t *element=td_list_entry_const(tmp, const alloc_list_t, list);
+  return element->end;
+}
+
+static void file_block_move(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset)
+{
+  const uint64_t end=file_offset_end(file_recovery);
+  struct td_list_head *tmp;
+  td_list_for_each(tmp, &list_search_space->list)
+  {
+    alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
+    if(element->start > end)
+    {
+      *new_current_search_space=element;
+      *offset=element->start;
+      return;
+    }
+  }
+  *new_current_search_space=list_search_space;
+}
+
+void file_block_truncate_and_move(file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize,  alloc_data_t **new_current_search_space, uint64_t *offset, unsigned char *buffer)
+{
+  file_block_truncate(file_recovery, list_search_space, blocksize);
+  file_block_move(file_recovery, list_search_space, new_current_search_space, offset);
+  if(file_recovery->offset_ok > file_recovery->file_size)
+    file_recovery->offset_ok=file_recovery->file_size;
+  if(file_recovery->offset_error > file_recovery->file_size)
+    file_recovery->offset_error=0;
+  file_recovery->calculated_file_size=0;
+  if(file_recovery->data_check!=NULL)
+  {
+    uint64_t i;
+    unsigned char *block_buffer;
+    block_buffer=&buffer[blocksize];
+    if(my_fseek(file_recovery->handle, 0, SEEK_SET) < 0)
+      return ;
+    for(i=0; i< file_recovery->file_size; i+= blocksize)
+    {
+      if(fread(block_buffer, blocksize, 1, file_recovery->handle) != 1)
+	return ;
+      file_recovery->data_check(buffer, 2*blocksize, file_recovery);
+      memcpy(buffer, block_buffer, blocksize);
+    }
+  }
+  else
+  {
+    if(my_fseek(file_recovery->handle, file_recovery->file_size, SEEK_SET) < 0)
+      return ;
+  }
+}
