@@ -229,108 +229,6 @@ static void gen_image(const char *filename, disk_t *disk, const alloc_data_t *li
   fclose(out);
 }
 
-#if 0
-static inline void list_append_block(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const unsigned int data)
-{
-  if(!td_list_empty(&list->list))
-  {
-    alloc_list_t *prev=td_list_entry(list->list.prev, alloc_list_t, list);
-    if(prev->end+1==offset && prev->data==data)
-    {
-      prev->end=offset+blocksize-1;
-      return ;
-    }
-  }
-  {
-    alloc_list_t *new_list=(alloc_list_t *)MALLOC(sizeof(*new_list));
-    new_list->start=offset;
-    new_list->end=offset+blocksize-1;
-    new_list->data=data;
-    td_list_add_tail(&new_list->list, &list->list);
-  }
-}
-
-static void test_files_aux(file_recovery_t *file_recovery, struct ph_param *params, const uint64_t start, const uint64_t end)
-{
-  uint64_t datasize=end-start+1;
-  unsigned char *buffer=(unsigned char *) MALLOC(datasize);
-  params->disk->pread(params->disk, buffer, datasize, start);
-  if(file_recovery->file_stat==NULL)
-  {
-    struct td_list_head *tmpl;
-    td_list_for_each(tmpl, &file_check_list.list)
-    {
-      struct td_list_head *tmp;
-      const file_check_list_t *pos=td_list_entry(tmpl, file_check_list_t, list);
-      td_list_for_each(tmp, &pos->file_checks[buffer[pos->offset]].list)
-      {
-	const file_check_t *file_check=td_list_entry(tmp, file_check_t, list);
-	if((file_check->length==0 || memcmp(buffer + file_check->offset, file_check->value, file_check->length)==0) &&
-	    file_check->header_check(buffer, datasize, 0, file_recovery, file_recovery)!=0)
-	{
-	  file_recovery->file_stat=file_check->file_stat;
-	  break;
-	}
-      }
-      if(file_recovery->file_stat!=NULL)
-	break;
-    }
-    if(file_recovery->file_stat==NULL)
-    {
-      free(buffer);
-      return ;
-    }
-    /* file_recovery->loc is used by file_truncate */
-    file_recovery->loc=NULL;
-    /* list_free_add, list_space_used, update_search_space, list_truncate, free_list_allocation */
-    file_recovery->location.start=start;
-  }
-  if(file_recovery->handle==NULL)
-  {
-    set_filename(file_recovery, params);
-    file_recovery->handle=fopen(file_recovery->filename, "w+b");
-    if(file_recovery->handle==NULL)
-    {
-      log_critical("Cannot create file %s: %s\n", file_recovery->filename, strerror(errno));
-      free(buffer);
-      return;
-    }
-  }
-  if(fwrite(buffer, datasize, 1, file_recovery->handle)<1)
-  {
-    log_critical("Cannot write to file %s: %s\n", file_recovery->filename, strerror(errno));
-    fclose(file_recovery->handle);
-    file_recovery->handle=NULL;
-    free(buffer);
-    return;
-  }
-  list_append_block(&file_recovery->location, start, datasize, 1);
-  file_recovery->calculated_file_size=0;
-  file_recovery->file_size+=datasize;
-  file_recovery->file_size_on_disk+=datasize;
-  free(buffer);
-}
-
-static void test_files(alloc_data_t *list_search_space, struct ph_param *params)
-{
-  alloc_data_t *current_search_space=list_search_space;
-  uint64_t offset;
-  file_recovery_t file_recovery;
-  reset_file_recovery(&file_recovery);
-  file_recovery.blocksize=512;
-  offset=current_search_space->start;
-  /* Recover a file with a known location */
-  test_files_aux(&file_recovery, params, 1289*512, (1304+1)*512-1);
-  test_files_aux(&file_recovery, params, 2881*512, (3259+1)*512-1);
-  file_finish(&file_recovery, params, list_search_space, &current_search_space, &offset);
-
-  /* Exclude some sectors from the search space */
-  del_search_space(list_search_space, 121407*512, (121416+1)*512-1);
-  del_search_space(list_search_space, 121445*512, (121448+1)*512-1);
-  del_search_space(list_search_space, 121865*512, (122195+1)*512-1);
-}
-#endif
-
 int photorec(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space)
 {
   pstatus_t ind_stop=PSTATUS_OK;
@@ -444,7 +342,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	  }
 #ifdef HAVE_NCURSES
 	  if(options->expert>0)
-	    params->blocksize=menu_choose_blocksize(params->blocksize, params->disk->sector_size, &start_offset);
+	    menu_choose_blocksize(&params->blocksize, &start_offset, params->disk->sector_size);
 #endif
 	  update_blocksize(params->blocksize, list_search_space, start_offset);
 	}
@@ -473,7 +371,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
 	    if(res!=NULL)
 	      *res='\0';
 	  }
-	  res=ask_location("Warning: no free space available. Please select a destination to save the recovered files.\nDo not choose to write the files to the same partition they were stored on.", "", dst);
+	  res=ask_location("Warning: not enough free space available. Please select a destination to save the recovered files.\nDo not choose to write the files to the same partition they were stored on.", "", dst);
 	  free(dst);
 	  if(res==NULL)
 	    params->status=STATUS_QUIT;
@@ -543,7 +441,7 @@ int photorec(struct ph_param *params, const struct ph_options *options, alloc_da
     struct td_list_head *search_walker = NULL;
     td_list_for_each(search_walker, &list_search_space->list)
     {
-      alloc_data_t *current_search_space;
+      const alloc_data_t *current_search_space;
       current_search_space=td_list_entry(search_walker, alloc_data_t, list);
       data_size += current_search_space->end - current_search_space->start + 1;
     }
@@ -654,17 +552,17 @@ void interface_options_photorec_ncurses(struct ph_options *options)
       case key_ESC:
       case 'q':
       case 'Q':
+	interface_options_photorec_log(options);
 	return;
     }
   }
-  interface_options_photorec_log(options);
 }
 
 #define INTER_FSELECT_X	0
 #define INTER_FSELECT_Y	(LINES-2)
 #define INTER_FSELECT	(LINES-10)
 
-static void interface_file_select_ncurses(file_enable_t *files_enable)
+void interface_file_select_ncurses(file_enable_t *files_enable)
 {
   int current_element_num=0;
   int offset=0;
@@ -676,6 +574,7 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
     {'q',"Quit","Return to main menu"},
     {0,NULL,NULL}
   };
+  log_info("\nInterface File Select\n");
 #if defined(KEY_MOUSE) && defined(ENABLE_MOUSE)
   mousemask(ALL_MOUSE_EVENTS, NULL);
 #endif
@@ -733,7 +632,7 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
     if(enable_status==0)
       wprintw(stdscr," for default selection, ");
     else
-      wprintw(stdscr," to disable all file famillies, ");
+      wprintw(stdscr," to disable all file families, ");
     if(has_colors())
       wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
     wprintw(stdscr,"b");
@@ -848,81 +747,3 @@ static void interface_file_select_ncurses(file_enable_t *files_enable)
   }
 }
 #endif
-
-void interface_file_select(file_enable_t *files_enable, char**current_cmd)
-{
-  log_info("\nInterface File Select\n");
-  if(*current_cmd!=NULL)
-  {
-    int keep_asking;
-    do
-    {
-      file_enable_t *file_enable;
-      keep_asking=0;
-      while(*current_cmd[0]==',')
-	(*current_cmd)++;
-      if(strncmp(*current_cmd,"everything",10)==0)
-      {
-	int enable_status;
-	keep_asking=1;
-	(*current_cmd)+=10;
-	while(*current_cmd[0]==',')
-	  (*current_cmd)++;
-	if(strncmp(*current_cmd,"enable",6)==0)
-	{
-	  (*current_cmd)+=6;
-	  enable_status=1;
-	}
-	else if(strncmp(*current_cmd,"disable",7)==0)
-	{
-	  (*current_cmd)+=7;
-	  enable_status=0;
-	}
-	else
-	{
-	  log_critical("Syntax error %s\n",*current_cmd);
-	  return;
-	}
-	for(file_enable=&files_enable[0];file_enable->file_hint!=NULL;file_enable++)
-	  file_enable->enable=enable_status;
-      }
-      else
-      {
-	unsigned int cmd_length=0;
-	while((*current_cmd)[cmd_length]!='\0' && (*current_cmd)[cmd_length]!=',')
-	  cmd_length++;
-	for(file_enable=&files_enable[0];file_enable->file_hint!=NULL;file_enable++)
-	{
-	  if(file_enable->file_hint->extension!=NULL &&
-	      strlen(file_enable->file_hint->extension)==cmd_length &&
-	      memcmp(file_enable->file_hint->extension,*current_cmd,cmd_length)==0)
-	  {
-	    keep_asking=1;
-	    (*current_cmd)+=cmd_length;
-	    while(*current_cmd[0]==',')
-	      (*current_cmd)++;
-	    if(strncmp(*current_cmd,"enable",6)==0)
-	    {
-	      (*current_cmd)+=6;
-	      file_enable->enable=1;
-	    }
-	    else if(strncmp(*current_cmd,"disable",7)==0)
-	    {
-	      (*current_cmd)+=7;
-	      file_enable->enable=0;
-	    }
-	    else
-	    {
-	      log_critical("Syntax error %s\n",*current_cmd);
-	      return;
-	    }
-	  }
-	}
-      }
-    } while(keep_asking>0);
-    return;
-  }
-#ifdef HAVE_NCURSES
-  interface_file_select_ncurses(files_enable);
-#endif
-}
