@@ -1,3 +1,25 @@
+/*
+
+    File: qphotorec.cpp
+
+    Copyright (C) 2009-2014 Christophe GRENIER <grenier@cgsecurity.org>
+
+    This software is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -42,6 +64,7 @@
 #include <QHeaderView>
 #include <QStandardItemModel>
 #include <QStandardItem>
+#include <QDialogButtonBox>
 #include <QSortFilterProxyModel>
 #include <QGroupBox>
 #include <QRadioButton>
@@ -113,6 +136,8 @@ QPhotorec::QPhotorec(QWidget *my_parent) : QWidget(my_parent)
   {
     no_disk_warning();
   }
+  else
+    select_disk(list_disk->disk);
   setupUI();
 }
 
@@ -147,12 +172,15 @@ void QPhotorec::newSourceFile()
       "Raw Files (*.dd *.raw *.img)");
   if(!filename.isEmpty())
   {
+    disk_t *new_disk=NULL;
     QByteArray filenameArray= (filename).toUtf8();
-    list_disk=insert_new_disk(list_disk, file_test_availability(filenameArray.constData(), options->verbose, testdisk_mode));
-    if(list_disk!=NULL)
-      select_disk(list_disk->disk);
-    HDDlistWidget_updateUI();
-    PartListWidget_updateUI();
+    list_disk=insert_new_disk_aux(list_disk, file_test_availability(filenameArray.constData(), options->verbose, testdisk_mode), &new_disk);
+    if(new_disk!=NULL)
+    {
+      select_disk(new_disk);
+      HDDlistWidget_updateUI();
+      PartListWidget_updateUI();
+    }
   }
 }
 
@@ -175,6 +203,12 @@ void QPhotorec::partition_selected()
 	return ;
       }
     }
+    if(list_part!=NULL)
+    {
+      selected_partition=list_part->part;
+      buttons_updateUI();
+      return ;
+    }
     return ;
   }
   for(tmp=list_part; tmp!=NULL; tmp=tmp->next)
@@ -242,6 +276,9 @@ void QPhotorec::PartListWidget_updateUI()
 	item=new QTableWidgetItem(QString(sizeinfo));
 	item->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
 	PartListWidget->setItem(currentRow, 4, item);
+	/* Select the partition if it's already known */
+	if(selected_partition == partition)
+	  PartListWidget->setCurrentItem(item);
       }
       {
 	QString partname="";
@@ -274,6 +311,15 @@ void QPhotorec::select_disk(disk_t *disk)
   log_info("%s\n", selected_disk->description_short(selected_disk));
   part_free_list(list_part);
   list_part=init_list_part(selected_disk, NULL);
+  /* If only whole disk is listed, select it */
+  /* If there is the whole disk and only one partition, select the partition */
+  if(list_part!=NULL)
+  {
+    if(list_part->next==NULL)
+      selected_partition=list_part->part;
+    else if(list_part->next->next==NULL)
+      selected_partition=list_part->next->part;
+  }
   log_all_partitions(selected_disk, list_part);
 }
 
@@ -352,6 +398,8 @@ void QPhotorec::buttons_updateUI()
   if(selected_disk==NULL || selected_partition==NULL)
   {
     button_search->setEnabled(false);
+    qwholeRadioButton->setChecked(true);
+    qfreeRadioButton->setEnabled(false);
     return ;
   }
   if(selected_partition->upart_type==UP_EXT2 || selected_partition->upart_type==UP_EXT3 || selected_partition->upart_type==UP_EXT4)
@@ -386,13 +434,18 @@ void QPhotorec::buttons_updateUI()
 void QPhotorec::HDDlistWidget_updateUI()
 {
   list_disk_t *element_disk;
+  int i;
   HDDlistWidget->clear();
-  for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
+  for(element_disk=list_disk, i=0;
+      element_disk!=NULL;
+      element_disk=element_disk->next, i++)
   {
     disk_t *disk=element_disk->disk;
     HDDlistWidget->addItem(
 	QIcon::fromTheme("drive-harddisk", QIcon(":res/gnome/drive-harddisk.png")),
 	disk->description_short(disk));
+    if(disk==selected_disk)
+      HDDlistWidget->setCurrentIndex(i);
   }
   HDDlistWidget->addItem(
       QIcon::fromTheme("application-x-cd-image", QIcon(":res/gnome/application-x-cd-image.png")),
@@ -402,7 +455,7 @@ void QPhotorec::HDDlistWidget_updateUI()
 void QPhotorec::setupUI()
 {
   QWidget *t_copy = copyright(this);
-  QLabel *t_free_soft = new QLabel("PhotoRec is free software, and\ncomes with ABSOLUTELY NO WARRANTY.");
+  QLabel *t_free_soft = new QLabel("PhotoRec is free software, and comes with ABSOLUTELY NO WARRANTY.");
   QLabel *t_select = new QLabel("Please select a media to recover from");
 
   HDDlistWidget = new QComboBox();
@@ -485,9 +538,13 @@ void QPhotorec::setupUI()
   button_search = new QPushButton(QIcon::fromTheme("go-next", QIcon(":res/gnome/go-next.png")), "&Search");
   button_search->setEnabled(false);
   QPushButton *button_quit= new QPushButton(QIcon::fromTheme("application-exit", QIcon(":res/gnome/application-exit.png")), "&Quit");
+  QPushButton *button_about= new QPushButton(QIcon::fromTheme("help-about", QIcon(":res/gnome/help-about.png")), "&About");
+  QPushButton *button_formats= new QPushButton(QIcon::fromTheme("image-x-generic.png", QIcon(":res/gnome/image-x-generic.png")),"&File Formats");
 
   QWidget *B_widget = new QWidget(this);
   QHBoxLayout *B_layout = new QHBoxLayout(B_widget);
+  B_layout->addWidget(button_about);
+  B_layout->addWidget(button_formats);
   B_layout->addWidget(button_search);
   B_layout->addWidget(button_quit);
   B_widget->setLayout(B_layout);
@@ -509,6 +566,8 @@ void QPhotorec::setupUI()
   HDDlistWidget_updateUI();
   buttons_updateUI();
 
+  connect(button_about, SIGNAL(clicked()), this, SLOT(qphotorec_about()) );
+  connect(button_formats, SIGNAL(clicked()), this, SLOT(qphotorec_formats()) );
   connect(button_search, SIGNAL(clicked()), this, SLOT(qphotorec_search()) );
   connect(button_quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
   connect(HDDlistWidget, SIGNAL(activated(int)),this,SLOT(disk_changed(int)));
@@ -596,7 +655,9 @@ void QPhotorec::qphotorec_search_updateUI()
   {
     tmp.sprintf("Pass %u - Reading sector %10llu/%llu",
 	params->pass,
-	(unsigned long long)((params->offset-partition->part_offset)/sector_size),
+	(unsigned long long)(params->offset>partition->part_offset && params->offset < partition->part_size ?
+	  ((params->offset-partition->part_offset)/sector_size):
+	  0),
 	(unsigned long long)(partition->part_size/sector_size));
   }
   progress_info->setText(tmp);
@@ -638,8 +699,8 @@ void QPhotorec::qphotorec_search_setupUI()
   QSizePolicy c_sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
 
   QLabel *disk_img=new QLabel();
-  QPixmap *disk_pixmap = new QPixmap(":res/gnome/drive-harddisk.png");
-  disk_img->setPixmap(*disk_pixmap);
+  QPixmap disk_pixmap = QPixmap(":res/gnome/drive-harddisk.png");
+  disk_img->setPixmap(disk_pixmap);
   disk_img->setSizePolicy(c_sizePolicy);
 
   QLabel *disk_txt=new QLabel();
@@ -768,6 +829,40 @@ int QPhotorec::photorec(alloc_data_t *list_search_space)
     session_save(list_search_space, params, options);
     switch(ind_stop)
     {
+      case PSTATUS_EACCES:
+	{
+	  int ret=QMessageBox::warning(this,"QPhotoRec: Failed to create file!", "Failed to create file! Please choose another destination", QMessageBox::Ok| QMessageBox::Cancel, QMessageBox::Ok);
+	  if(ret==QMessageBox::Cancel)
+	  {
+	    params->status=STATUS_QUIT;
+	  }
+	  else
+	  {
+	    setExistingDirectory();
+	    free(params->recup_dir);
+	    QByteArray byteArray = (directoryLabel->text() + "/" + DEFAULT_RECUP_DIR).toUtf8();
+	    params->recup_dir=strdup(byteArray.constData());
+	    params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num);
+	  }
+	}
+	break;
+      case PSTATUS_ENOSPC:
+	{
+	  int ret=QMessageBox::warning(this,"QPhotoRec: Not enough space!", "There is not enough space left! Please free disk space and/or choose another destination", QMessageBox::Ok| QMessageBox::Cancel, QMessageBox::Ok);
+	  if(ret==QMessageBox::Cancel)
+	  {
+	    params->status=STATUS_QUIT;
+	  }
+	  else
+	  {
+	    setExistingDirectory();
+	    free(params->recup_dir);
+	    QByteArray byteArray = (directoryLabel->text() + "/" + DEFAULT_RECUP_DIR).toUtf8();
+	    params->recup_dir=strdup(byteArray.constData());
+	    params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num);
+	  }
+	}
+break;
       case PSTATUS_OK:
 	status_inc(params, options);
 	if(params->status==STATUS_QUIT)
@@ -814,4 +909,91 @@ void QPhotorec::qphotorec_search()
     params->blocksize=remove_used_space(params->disk, params->partition, &list_search_space);
   }
   photorec(&list_search_space);
+  free(params->recup_dir);
+  params->recup_dir=NULL;
+}
+
+void QPhotorec::qphotorec_about()
+{
+  QPixmap pixmap_img = QPixmap(":res/photorec_64x64.png");
+  QMessageBox msg;
+  msg.setText("QPhotoRec is is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version.\n\nQPhotoRec is is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License along with QPhotoRec.  If not, see <http://www.gnu.org/licenses/>.");
+  msg.setWindowTitle("QPhotoRec: About");
+  msg.addButton(QMessageBox::Close);
+  msg.setIconPixmap(pixmap_img);
+  msg.exec();
+}
+
+void QPhotorec::qphotorec_formats()
+{
+  file_enable_t *file_enable;
+  QStringList list;
+  formats=new QListWidget();
+  for(file_enable=list_file_enable;
+      file_enable->file_hint!=NULL;
+      file_enable++)
+  {
+    QListWidgetItem * item;
+    char descr[128];
+    sprintf(descr, "%-4s %s",
+	    (file_enable->file_hint->extension!=NULL?
+	     file_enable->file_hint->extension:""),
+	    file_enable->file_hint->description);
+    item = new QListWidgetItem(descr, formats);
+    if(file_enable->enable)
+      item->setCheckState (Qt::Checked);
+    else
+      item->setCheckState (Qt::Unchecked);
+  }
+
+  QDialog fenetre3;
+  fenetre3.setWindowTitle("QPhotoRec: File Formats");
+  QDialogButtonBox buttonBox(Qt::Horizontal);
+
+  QPushButton *bt_reset= new QPushButton("&Reset");
+  QPushButton *bt_restore= new QPushButton("Res&tore");
+
+  buttonBox.addButton(bt_reset, QDialogButtonBox::ResetRole);
+  buttonBox.addButton(bt_restore, QDialogButtonBox::ResetRole);
+  buttonBox.addButton(QDialogButtonBox::Ok);
+  QVBoxLayout vbox;
+  vbox.addWidget(formats);
+  vbox.addWidget(&buttonBox);
+  fenetre3.setLayout(&vbox);
+  connect(&buttonBox, SIGNAL(accepted()), &fenetre3, SLOT(accept()));
+  connect(bt_reset, SIGNAL(clicked()), this, SLOT(formats_reset()));
+  connect(bt_restore, SIGNAL(clicked()), this, SLOT(formats_restore()));
+  fenetre3.exec();
+  int i;
+  for (i = 0, file_enable=list_file_enable;
+      i < formats->count() && file_enable->file_hint!=NULL;
+      i++, file_enable++)
+  {
+    QListWidgetItem *item = formats->item(i);
+    file_enable->enable=(item->checkState()==Qt::Checked?1:0);
+  }
+}
+
+void QPhotorec::formats_reset()
+{
+  for (int i = 0; i < formats->count(); i++) {
+    QListWidgetItem *item = formats->item(i);
+    item->setCheckState (Qt::Unchecked);
+  }
+}
+
+void QPhotorec::formats_restore()
+{
+  file_enable_t *file_enable;
+  int i;
+  for (i = 0, file_enable=list_file_enable;
+      i < formats->count() && file_enable->file_hint!=NULL;
+      i++, file_enable++)
+  {
+    QListWidgetItem *item = formats->item(i);
+    if(file_enable->file_hint->enable_by_default)
+      item->setCheckState (Qt::Checked);
+    else
+      item->setCheckState (Qt::Unchecked);
+  }
 }
