@@ -19,17 +19,23 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "list.h"
-typedef struct backup_disk backup_disk_t;
-struct backup_disk
+typedef struct
 {
-  struct list_head list;
+  struct td_list_head list;
   time_t my_time;
   char description[128];
   list_part_t *list_part;
-};
+} backup_disk_t;
 
-int save_header(disk_t *disk_car,partition_t *partition, const int debug);
-int partition_save(disk_t *disk_car, list_part_t *list_part, const int debug);
-backup_disk_t *partition_load(const disk_t *disk_car, const int debug);
+int save_header(disk_t *disk_car,partition_t *partition, const int verbose);
+int partition_save(disk_t *disk_car, list_part_t *list_part, const int verbose);
+backup_disk_t *partition_load(const disk_t *disk_car, const int verbose);
 
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
