@@ -47,7 +47,7 @@ static int test_sun_i386(const disk_t *disk_car, const sun_partition_i386 *sunla
 int check_sun_i386(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(SUN_PARTITION_I386_SIZE);
-  sun_partition_i386 *sunlabel=(sun_partition_i386*)buffer;
+  const sun_partition_i386 *sunlabel=(const sun_partition_i386*)buffer;
   if(disk_car->pread(disk_car, buffer, SUN_PARTITION_I386_SIZE, partition->part_offset + 0x200) != SUN_PARTITION_I386_SIZE)
   {
     free(buffer);
